pro cal32,result1,result2,ipar
;
;  To do: include a responsivity drift correction to the fcs as a further
;         option to options 0, 6 (eg options 8 (for no implementation)
;         and 9 (for implementation).
;
;  NB CAL32 must be changed again. The PIA routines must be run such that
;  the reset correction is applied before the dark. Darks can be found from
;  comparison SPD data sets. RESET from the internal subroutine without having 
;  to read datasets. Flux calibration from SCP AAP calibration as before.
;  Thus maps only require the SCP and AAP saves. FCS will requirer SRD save
;  from PASS1 ie with all instrument specific corrections linearity, deglitch,
;  reset, drift recog already applied before input to FCSTAU.

;
; uses internal files written from PIA to determine instrument calibration
; for P32 observations for use within the ERDTOMAP suite of routines
;
; eg run line
; cal32,'53801001','01','/d13/vlk/rjt/ISO/PHT/P32P22P25_PRODUCTS/v8_1',$
; result1,result2,0
;
; input (common NAMES):
;    TDTNAM: string denoting name of TDT eg '53801001'
;    ISEQ:   string denoting sequence number of target source map
;            (normally '01' or '03' or '05' for multifilter sequence separated
;             by FCS measurements) 
;    DIRNAM: dstring denoting irectory name for internal PIA files
;            eg '/d13/vlk/rjt/pia/p32p22p25/v7_3/'
; input:
;    S       (common SIGNAL) (NPIX,NPTIN)
;    RMS     (common SIGNAL) (NPIX,NPTIN)
;    NPIX    (common STEER1) = number of pixels in array
;    NINRAMP (common STEER1) = number of non-destructive reads in ramp
;    ARRAY   (common STEERIN) = 'c100' or 'c200'
;    COMPLETE (common TIMER) = 1/float(data completeness factor)
;    IPAR:   determines action and output parameter to be written to result:
;            = 0 for average MJy/str per V/s determined from actual response 
;            = 1 for average MJy/str per V/s determined from default response 
;            = 2 for dark (correction to be subtracted) assumed constant during
;                an observation, determined from comparions of .ISRD and
;                unmodified .IERD (s_in) files of target map
;            = 3 as for 2 but dark current is subtracted from S and
;                1 added to IDARKDONE 
;            = 4 for reset interval correction (correction to be multiplied)
;                determined from comparison of .ISRD and .ISCP files of
;                target map.
;            = 6 as 0 but S and RMS arrays multiplied by a linear fit to 
;              AAP/SCP to reproduce the linear interpolation between the 
;              two FCS measurements (actual response calibration)
;            = 7 as 1 but S and RMS arrays multiplied by default response
;    ICALDONE (common CAL) = 0 if data is uncalibrated >= 1 otherwise
;             (S and RMS are not overwritten if ICALDONE >= 1)
;    IDARKDONE (common CAL) = 0 if data has not had dark subtracted 
;              >= 1 otherwise (S is not overwritten if IDARKDONE >= 1)
;    IRESETDONE (common CAL) = 0 if reset correction has been applied
;              >= 1 otherwise (S and RMS are not overwritten if IRESETDONE >= 1)
;
; output:
;    RESULT1(npix) content depends on value of IPAR
;    RESULT2(npix) content depends on value of IPAR
;    S        overwritten for IPAR 6 or 7 and ICALDONE=0
;    RMS      overwritten for IPAR 6 or 7 and ICALDONE=0
;    ICALDONE incremented by 1 if S and RMS are overwritten
;    IDARKDONE incremented by 1 if S has had dark current subtracted.
;    IRESETDONE incremented by 1 if S and RMS are overwritten
;    CALRESULT (set for ipar = 6 or 7 only) FLTARR(npix,nptin) 
;              with multiplitive
;              factors applied to convert V/s to MJy/sterad
;    DARKRESULT (set for ipar = 3 only) FLTARR(NPIX) with dark currents
;                which have been subtracted 
;    RESETRESULT1 (set for ipar=5 only) FLTARR(NPIX) with additive
;    correction factors for reset effect which have been applied
;    RESETRESULT2 (set for ipar=5 only) FLTARR(NPIX) with multiplitive
;    correction factors for reset effect which have been applied
;
; Note: the non-linearity correction should applied at the ERD stage
;       before writing the IERD to disk.
;
;       It is assumed the two sets of input data sets have had the following 
;       PIA analysis, (normally applied as a batch sequence to both FCS and 
;       maps):
;
; (I)   In directory DIRNAM
;
;       ERD: deglitch, non-linearity, linear ramp fit, standard selection
;            (ignores 1st and destructive read), save.
;       SRD: deglitch, reset correction dark, responsivity drift recognition,
;            save (though only the fcs is read at SRD level)
;            (Notes: the drift recognition will be ignored for maps but will 
;            have effect on FCS. The responsivity drift recognition gives 
;            very close results, within a few percent (though typically 
;            biased), to a full drift fit and is more robust)
;       SCP  no processing (especially no vignetting) except for . save
;       SPD  actual response calibration (linear interpolation of FCS) from
;            average within each FCS, save
;       AAP  converted Watts to MJy/sterad. save.
;
; (II)  In directory DIRNAM+,'/PASS2/' 
;           (eg use pia_envi_p322225_pass2 for pia_envi):
;
;       ERD: as above (but not saved)
;       SRD: as above but not saved and no dark correction
;       SCP: as above, save
;       SPD: default response, save
;       AAP: as above, save.
;
; Note: the dark correction and reset corrections, and the initializations
;       icaldone,idarkdone,iresetdone should be done immediately
;       or perhaps even within SIGCREATE routine. The calibration to MJy/sterad
;       should be done as the last step before the flat field, and after the
;       responsivity drift correction which uses engineering units for the
;       detector model parameters.
;
;common_names,tdtnam,iseq,dirnam,caldir,cfilter
@common_names
;common signal,rs,cs,nch,s_in,s,badin,bad,rel,rms,glitch,jf,njf,ninchmax,rd,$
;nkf,kf,nkfmax,irelstart,nchopmed,gbad,npolyrms,nrmsmed,sigmaxfract
@common_signal
;common steer1,npix,pixsize,pixpitch,pixoff,nchop,dcy,dry,drz
@common_steer1
;common steerin,array,nptin,npt,ninramp,imeas,osamp
@common_steerin
;common cal,icaldone,idarkdone,iresetdone,calresult,darkresult,$
;resetresult1,resetresult2
@common_cal
;common timer,time,dtime,complete
@common_timer
common phtintt_corr, phtintt_corr ; PIA common block 
;                                   for reset correction factors
icalswitch=0 ; =0 to find calibration from saved files
             ; =1 to find calibration for dark,responsivity from 
;              PIA subroutines
; calibration for signal non-linearity (supplementary to linearizing of
; readout ramps) is done in any case frmo PIA subroutines 
;
if(ipar eq 6 or ipar eq 7)then begin ; avoid calibrating arrays twice
if(icaldone gt 0)then begin
print,'CAL32: S and RMS not overwritten as calibration already applied'
goto,c999
endif ; icaldone gt 0
endif ; ipar eq 6 or ipar eq 7
;
if(ipar eq 3)then begin ; avoid dark correction twice
if(icaldone gt 0)then begin
print,'CAL32: S not overwritten as dark correction already applied'
goto,c999
endif ; icaldone gt 0
endif ; ipar eq 3
;
if(ipar eq 5)then begin ; avoid reset correction twice
if(icaldone gt 0)then begin
print,'CAL32: S and RMS not overwritten as reset correction already applied'
goto,c999
endif ; icaldone gt 0
endif ; ipar eq 5
;
defresponse=fltarr(npix)
if(array eq 'c100')then begin
; default responsivities from PIA V7.3 ("obselete") in A/W
; note: amps = capacitance * volts/s
; note it is better to take responsivities from near centre of orbit
; in the orbit dependent file
capac=9.0d-14 ; capacitance
; the following are "obsolete" values
defresponse(0)=44.2
defresponse(1)=34.3
defresponse(2)=43.0
defresponse(3)=38.7
defresponse(4)=26.8
defresponse(5)=30.7
defresponse(6)=45.4
defresponse(7)=46.5
defresponse(8)=27.8
; the following are for orbit position 50 after 15th April 1996
; (when a new mid-orbit curing procedure was introduced).
defresponse(0)=48.6
defresponse(1)=47.8
defresponse(2)=57.8
defresponse(3)=44.3
defresponse(4)=35.8
defresponse(5)=33.7
defresponse(6)=48.5
defresponse(7)=51.0
defresponse(8)=37.2
endif
if(array eq 'c200')then begin
; default responsivities from PIA V7.3 in A/W
capac=1.4d-13 ; capacitance
; the following are "obsolete" values
defresponse(0)=14.0
defresponse(1)=16.9
defresponse(2)=13.3
defresponse(3)=14.4
; the following are for orbit position 50 after 15th April 1996
defresponse(0)=22.7
defresponse(1)=27.3
defresponse(2)=22.7
defresponse(3)=26.5
endif
;
result1=fltarr(npix)
result2=result1
darkresult=result1
resetresult1=result1
resetresult2=result1
calresult=s
calresult(*,*)=0.0
;
if(array eq 'c100')then root=dirnam+'C1'+iseq+tdtnam
if(array eq 'c200')then root=dirnam+'C2'+iseq+tdtnam
iscp = root+'.ISCP'
if(array eq 'c100')then $
iscpnodark = dirnam+'PASS2/'+'C1'+iseq+tdtnam+'.ISCP'
if(array eq 'c200')then $
iscpnodark = dirnam+'PASS2/'+'C2'+iseq+tdtnam+'.ISCP'
ispd = root+'.ISPD'
iaap = root+'.IAAP'
if(array eq 'c100')then $
iaapdef = dirnam+'PASS2/'+'C1'+iseq+tdtnam+'.IAAP'
if(array eq 'c200')then $
iaapdef = dirnam+'PASS2/'+'C2'+iseq+tdtnam+'.IAAP'
;print,'input files:'
;print,iscp
;print,iaap
;
if(ipar eq 0 or ipar eq 6)then begin ; calculate average MJy/str per V/s
;                                      determined from actual response
;
; Note that second (signal) non-linearity correction is made here, 
; always from PIA subroutine independently of value of ICALSWITCH
; 
if(icalswitch eq 0)then begin ; calibration from saved files
print,'CAL32: calculating actual MJy/str per V/s from saved files'
@phtscp
print,'CAL32: reading ',iscp
read_iscp,iscp
imeas=N_ELEMENTS(phtscp)-1
handle_value,phtscp(imeas).aver,aver
@phtaap
print,'CAL32: reading ',iaap
read_iaap,iaap
imeas=N_ELEMENTS(phtaap)-1
handle_value,phtaap(imeas).mnbr,mnbr
handle_value,phtaap(imeas).time,aaptime
aaptime=aaptime-aaptime(0)+0.0D
erdtime=time-time(0) ; erdtime
;
for ipix=0,npix-1 do begin
;
scpaver=total(aver(ipix,*))/float(n_elements(aver(ipix,*)))
aapaver=total(mnbr(ipix,*))/float(n_elements(mnbr(ipix,*)))
result1(ipix)=aapaver/scpaver
;print,'ipix,aapaver,scpaver = ',ipix,aapaver,scpaver 
;
if(ipar eq 6)then begin ; linear interpolation from fcs
;
if(ipix eq 0)then print,$
'CAL32: calibrating using interpolation between actual (FCS) responses:'
;
ii=where(aver(ipix,*) ne 0.0 and mnbr(ipix,*) ne 0.0,nii)
i1=0
i2=nii/2
i3=i2+1
i4=nii-1
av1=median(mnbr(ipix,i1:i2))/median(aver(ipix,i1:i2))
av2=median(mnbr(ipix,i3:i4))/median(aver(ipix,i3:i4))
;print,'ii((i1+i2)/2),ii((i3+i4)/2)',ii((i1+i2)/2),ii((i3+i4)/2)
t1=float(aaptime(ii((i1+i2)/2)))
t2=float(aaptime(ii((i3+i4)/2)))
xm=(av2-av1)/(t2-t1)
xc=av1-xm*t1
calresult(ipix,*)=erdtime*xm+xc
;window,0
;plot,erdtime*xm+xc
print,'CAL32: pixel, mean (MJy/St) per (V/s) = ',ipix,result1(ipix)
s(ipix,*)=s(ipix,*)*calresult(ipix,*)
icaldone=icaldone+1
rms(ipix,*)=rms(ipix,*)*calresult(ipix,*)
;
endif ; ipar eq 6
;
endfor ; ipix
;
endif ; icalswitch eq 0
;
endif ; ipar eq 0 or ipar eq 6
;
if(ipar eq 1 or ipar eq 7)then begin ; av. MJy/str per V/s 
;                                      determined from default response 
;
; independent of the value of ICALSWITCH,
; default corrections are found from a combination of the SCP and SCP files
; and the PIA subroutine for non-linearity correction
;
; comparison SCP and AAP in /PASS2/ 
; block which should be correct for actual orbital position
; the fact that no dark is included in both should not affect the 
; determination of the factor.
;
print,'CAL32: calculating default MJy/str per V/s from saved files'
@phtscp
print,'CAL32: reading ',iscpnodark
read_iscp,iscpnodark
imeas=N_ELEMENTS(phtscp)-1
handle_value,phtscp(imeas).aver,avernodark
@phtaap
print,'CAL32: reading ',iaapdef
read_iaap,iaapdef
imeas=N_ELEMENTS(phtaap)-1
handle_value,phtaap(imeas).mnbr,mnbrdef
;
for ipix=0,npix-1 do begin
;
scpaver=total(avernodark(ipix,*))/float(n_elements(avernodark(ipix,*)))
aapdefaver=total(mnbrdef(ipix,*))/float(n_elements(mnbrdef(ipix,*)))
result1(ipix)=aapdefaver/scpaver ; method II
;
if(ipar eq 7)then begin
;
if(ipix eq 0)then print,$
'CAL32: calibrating using orbit-dependent default responses:'
;
print,'CAL32: pixel, mean (MJy/St) per (V/s) = ',ipix,result1(ipix)
calresult(ipix,*)=result1(ipix)
s(ipix,*)=s(ipix,*)*result1(ipix)
rms(ipix,*)=rms(ipix,*)*result1(ipix)
icaldone=icaldone+1
endif ; ipar eq 7
;
endfor ; ipix
;
endif ; ipar eq 1 or ipar eq 7
;
if(ipar eq 2 or ipar eq 3) then begin ; find dark current to be subtracted
;
if(icalswitch eq 0)then begin 
print,'CAL32: finding dark from saved files:'
;
@phtscp
print,'CAL32: reading ',iscp
read_iscp,iscp
imeas=N_ELEMENTS(phtscp)-1
handle_value,phtscp(imeas).aver,aver
@phtscp
print,'CAL32: reading ',iscpnodark
read_iscp,iscpnodark
imeas=N_ELEMENTS(phtscp)-1
handle_value,phtscp(imeas).aver,avernodark
;
for ipix=0,npix-1 do begin
;
afterdark=median(aver(ipix,*))
beforedark=median(avernodark(ipix,*))
result1(ipix)=beforedark-afterdark
;print,'ipix,beforedark,afterdark,dark',ipix,beforedark,afterdark,result1(ipix)
;
endif ; if icalswitch eq 0
;
; *** include here if(icalswitch eq 1)then begin
;
if(ipar eq 3)then begin ; implement dark correction in S and write DARKRESULT
darkresult(ipix)=result1(ipix)
print,'CAL32 subtracting dark: pixel, dark = ',ipix,darkresult(ipix)
s(ipix,*)=s(ipix,*)-darkresult(ipix)
idarkdone=idarkdone+1
endif ; ipar eq 3
;
endfor ; ipix
;
endif ; (ipar eq 2 or ipar eq 3)
;
if(ipar eq 4 or ipar eq 5) then begin ; find reset correction by
;                                     ; calling the pia routine CORR_RESET_INT
; to find multiplicative and offset factors (implementation within PIA is
; in routine DO_CORR_RESET_INT). Same correction for each pixel.
;
dred=round(1.0/complete) ; data completeness factor
if(array eq 'c100')then det='C1'
if(array eq 'c200')then det='C2'
tread=dtime*float(ninramp) ; destructive read interval
intt=7+(alog10(1.0/tread))/alog10(2.0)
intt=round(intt)
test=[1.0,1.0,1.0] ; dummy data set
corr_reset_int,test,det,intt,dred,testout,corr,uncert=test
help,corr
a0=corr.a0
a1=corr.a1
print,'CAL32 found reset correction a0, a1 = ',corr.a0,corr.a1
;
for ipix=0,npix-1 do begin
;
result1(ipix)=a0 ; same for all pixels additive factor 
result2(ipix)=a1 ; same for all pixels multiplative factor
;
if(ipar eq 5)then begin ; implement reset correction in S and RMS and write 
;                         RESETRESULT
resetresult1(ipix)=result1(ipix) 
resetresult2(ipix)=result2(ipix) 
print,'CAL32: pixel, applied reset correction = ',ipix,$
resetresult1(ipix),resetresult2(ipix)
s(ipix,*)=resetresult1(ipix)+s(ipix,*)*resetresult2(ipix)
rms(ipix,*)=rms(ipix,*)*resetresult2(ipix)
iresetdone=iresetdone+1
endif ; ipar eq 5
;
endfor ; ipix
;
endif ; (ipar eq 4 or ipar eq 5)
;
c999:
;
return
end





