pro createbeam,sy,sz
;
; Creates a beam BEAM in the common block BM. Beams do not depend on detector
; pixel as this is not an input parameter.
; for each detector 
; If ICAM is 0 real beams from the CALG
; FITS files are used. If ICAM is 1 then 
; a scaled version of the CAM simulator 
; is used (a call to one of the standard 
; CIA routines is needed for this).
; If ICAM is 2 a gaussian beam is output
; For ICAM 0 the beam for the 0th detector pixel is used
;     for C200 and the central pixel for the C100 array. 
;
; input:
;   common NAMES: 
;   CFILTER (integer) = either 50,60,70,90,100,105,120,135,160,180,200
;   CALDIR   string denoting CALG directory (needed eg by CREATEBEAM)
;                    eg '/d12/vlk/cgabriel/pia_7.3/calg/' 
;
;   BEAMEXTENT (common block BM) ; radius in arcsec out to which the
;                beam is to be specified
;   SPECPAR (common block BM)  2 element real array specifying spectrum of
;                source assumed in calculating beam:
;         for monocromatic source (ICAM=1) SPECPAR = [0,lamba/um]
;         for planckian source (ICAM=1)    SPECPAR = [1,temperature]
;         for power law source (ICAM=1)    SPECPAR = [2,spectral index +ve red]
;                specpar is only used for ICAM = 1 or 2  
;   ICAM    (common block BM) is 1 if CAM simulated beam to be used
;                is 0 for real PHT beams from CALG to be used
;                is 2 for a gaussian beam with FWHM of detector pixel
;   SY,SZ        beam sampling in arcsec
;   CALDIR       string denoting CALG directory
;                this is '/d12/vlk/cgabriel/pia_7.3/calg/'
;
; output (to common block BM):
;  
;   BEAM         BEAM(nyB,nzB) normalised to 1.0 with all values
;                outside BEAMEXTENT set to zero (ie circular beam)
;   NYB,NZB      dimensions of beam array (always odd)
;   YGRIDB,ZGRIDB offset in arcsec of positions in each coordinate
;                 of BEAM (1D arrays dimension NYB or NZB)      
;
;common names,tdtnam,iseq,dirnam,caldir,cfilter
@common_names
@common_bm
common bm,beam,nyb,nzb,ygridb,zgridb,beamextent,specpar,icam
;
calgextent=127.5
if(cfilter ge 110)then calgextent=255.0 
; radial extent of all PHT-C beams in calg
if(icam eq 0 and beamextent gt calgextent)then begin
beamextent=calgextent
print,'CREATEBEAM: reducing field for beam to fit CALG extent'
endif ; icam eq 0 and beamextent gt calgextent
;
;nyb=round(beamextent/sy) ; half radius of beam in units of SKy pixels
;nyb=1+2*nyb ; always odd
;ygrid=sy*findgen(nyb)
;ygrid=ygrid-sy*float(nyb/2)
;nzb=round(beamextent/sz)
;nzb=1+2*nzb ; always odd
;zgrid=sz*findgen(nzb)
;zgrid=zgrid-sz*float(nzb/2)
;
findnb,beamextent,sy,nybr,nyb,ygrid
findnb,beamextent,sz,nzbr,nzb,zgrid
;
;
if(icam eq 1)then begin
print,'CREATEBEAM: option ICAM=1 not yet implemented - '
print,'... using beam created from PHT CALG files instead'
icam=0
endif ; icam eq 1
;
if(icam eq 2)then begin
if(cfilter gt 105)then fwhm=89.4
if(cfilter le 105)then fwhm=43.8
print,'CREATEBEAM: truncated circular gaussian with FWHM, BEAMEXTENT = '
print,fwhm,beamextent
creategauss,1.0,fwhm/sy,fwhm/sz,beamextent/sy,beamextent/sz,nyb,nzb,beam
endif ; icam eq 2
;
if(icam eq 0)then begin
print,'CREATEBEAM: beam from PHT CALG files'
;
pixsize=[sy,sz]
if(cfilter gt 105)then begin
detector='C2'
apert=0
endif ; cfilter gt 105
if(cfilter le 105)then begin
detector='C1'
apert=0 ; only of use for PHT-P no distinction between PHT-C pixels
endif ; cfilter le 105
if(cfilter eq 50)then filter='C1_50'
if(cfilter eq 60)then filter='C1_60'
if(cfilter eq 70)then filter='C1_70'
if(cfilter eq 90)then filter='C1_90'
if(cfilter eq 100)then filter='C1_100'
if(cfilter eq 105)then filter='C1_105'
if(cfilter eq 120)then filter='C2_120'
if(cfilter eq 135)then filter='C2_135'
if(cfilter eq 160)then filter='C2_160'
if(cfilter eq 180)then filter='C2_180'
if(cfilter eq 200)then filter='C2_200'
caldir0=caldir
foot,detector,filter,apert,pixsize,fp,conv_fp,fpred,caldir=caldir0
i1=0
i2=i1+nyb-1
j1=0
j2=j1+nzb-1
nfpy=n_elements(fpred(*,0))
nfpz=n_elements(fpred(0,*))
;
if(nyb lt nfpy)then begin
i1=(nfpy-nyb)/2 ; both odd so nfpy-nyb always even
i2=i1+nyb-1
endif ; nyb lt nfpy
if(nzb lt nfpz)then begin
j1=(nfpz-nzb)/2 ; both odd so nfpz-nzb always even
j2=j1+nzb-1
endif ; nzb lt nfpz
;
if(nyb ge nfpy)then begin
nyb=nfpy
ygrid=sy*findgen(nyb)     ; recalculate ygrid
ygrid=ygrid-sy*float(nyb/2)
i1=0
i2=nyb-1
endif ; nyb gt nfpy
if(nzb ge nfpz)then begin
nzb=nfpz
zgrid=sz*findgen(nzb)     ; recalculate ygrid
zgrid=zgrid-sz*float(nzb/2)
j1=0
j2=nzb-1
endif ; nzb ge nfp
;
c250:
;
beam=fpred(i1:i2,j1:j2)
;
endif ; icam eq 0
;
; circular truncation - create array of radial distances 
;
r=fltarr(nyb,nzb)
test=fltarr(nyb)
for j=0,nzb-1 do begin
y=zgrid(j)
r(*,j)=sqrt(y*y+ygrid*ygrid)
endfor ; j
ii=where(r gt beamextent,nii)
if(nii gt 0)then beam(ii)=0.0
;
ygridb=ygrid
zgridb=zgrid
beam=beam/total(beam) ; normalisation
print,'CREATEBEAM: nyb, nzb = ',nyb,nzb
;
;window,xs=300,ys=300
ii=where(r le beamextent,nii)
lev=min(beam(ii))+(max(beam(ii))-min(beam(ii)))*$
[0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9,1.0]
;print,'min,max,lev: ',min(beam(ii)),max(beam(ii)),lev
x=(findgen(nyb)-float(nyb/2))*sy
y=(findgen(nzb)-float(nzb/2))*sz
;contour,beam,x,y,lev=lev,xsty=1,ysty=1
;print,'ygridb: ',ygridb
;print,'zgridb: ',zgridb
;
return
end








