pro detinit,beta,tau,signalprev,signalprevshoot,sillumprev,$
nplatinit,ipt1,iptend1,fromorig,toorig,keyporig,vigporig,$
t,signal,rms,itrans,mask,keypoint,ech,$
bestsky,skypass,fitprec,imodel,silmin,silmax,isquare
;
; it is assumed that immediately prior to the start of a P32 observation
; the detector points to the central position of the P32 chopper sweep
;
; DETINIT finds initial condition of a detector - the signal and illumination
; immediately prior to the first sample of the first plateau being
; processed by DRIFTCORR, which must be a fine pointing plateau. Since
; the s/c pointing flags set up by POINT are not synchronised to the chopper 
; this will in general be an incomplete chopper plateau. DETINIT works by 
; finding the SILLUMCALC array for the first NPLATINIT (internally set in 
; DRIFT32 and passed to DRIFTCORR) plateaus of the first s/c fine pointing 
; processed in DRIFTCORR, which starts at (expanded) sample IPT1 and ends
; at (expanded) sample IPTEND1. SILLUMCALC is found from the SKYPASS array
; written in the last pass over the first s/c pointing in DRIFTCORR; the
; last occurence of the NPLATINIT plateaus in the fine pointing are used 
; to find SILLUMCALC. The use of SKYPASS will provide a quicker convergence 
; than the use of BESTSKY, since SKYPASS is writen with the actual solution
; from SOLVEILLUM1 for each individual plateau, unlike BESTSKY, which is some
; statistical combination of all previous sloutions for a given plateau,
; including the first solutions which will be systematically wrong for poor
; initial values for the detector starting state. 
; *** rjt Jan00 We use BESTSKY afterall to get a completely formal self
; consistent solution. Reversed Mar00 - old solution gives better chis.
;
;
; If SILLUMCALC is set up successfully (this depends on whether 
; the NPLATINIT plateaus can be found at the end of the s/c fine pointing not 
; overlapping with the first NPLATINIT plateaus) DETINIT finds SIGNALPREV 
; SILLUMPREV by calling SOLVEBACKILLUM1, where SILLUMPREV is 
; constrained to be the same as the found illumination from the last occurence
; of the first plateau (see description of SOLVEBACKILLUM). After calling 
; DETINIT from DRIFTCORR, the solution is started again with the revised 
; SIGNALPREV, SILLUMPREV, after initialising all the output quantities of 
; DRIFTCORR in DROUINIT, like all the things in common TESTDRIFTCORR, and
; BESTSKY. DETINIT does NDETINIT such calculations of SIGNALPREV, SILLUMPREV
; (where NDETINIT is a user input parameter to DRIFT32 passed to DRIFTCORR.
; If NDETINIT is 0 no call to DETINIT is made and the values input to 
; DRIFTCORR for SIGNALPREV and SILLUMPREV are used. NDETINIT should be
;  set up > 0 even for self cal applications.
;
; Note: no attempt is made to censor points for which no good solution
; could be found (ie plateaus for which solveillum1 does not converge in 
; search range). This is because inclusion of such points will worsen
; chi and exclusion will, falsely, improve chi, giving the danger of
; producing an apparanently good solution from a small minority of points
; for which solveillum1 found a solution.
;
; Note: Nov 29 made signalprev sillumprev their true values, rather than
;       an arbitrary combination satisfying s0, as output by SOLVEBACKILLUM1
;
; input:  BETA detector model constant
;         TAU (VOLTS) detector model constant
;         SIGNAL      (array in V/s over all observation) measured detector 
;                      signal at times in T since illumination step at time 
;                      zero. 
;         T           (array in s over all observation) time since flux step 
;                      at start of plateau at time 0. T refers to the time at 
;         RMS         (array in V/s over all observation) RMS uncertainty in 
;                      SIGNAL
;         ITRANS     (1D array INTEGER of whole timeline of observation). 
;                     This is set to 1 for data which is
;                     transmitted to the ground and zero for data not
;                     transmitted (in the fast readout modes).  
;         ECH        (1D array INTEGER of whole timeline of observation). 
;                      Chopper plateau identifier (eg for P32
;                      1 to 13 for C100 1 to 7 for C200) for each time sample
;         MASK       (1D array integer of whole timeline of observation)
;                     = 1 for data to be ignored. We have
;                     to import all time sampling points in the time array
;                     T in order to follow the illumination history
;                     so for ease of programming we import all data
;                     too, necessitating a mask.
;-----------------------------------------------------------------------------
;         the following plateau dependent arrays are input as their values
;         as input to DRIFTCORR (ie they do not reflect any changes in the 
;         plateau structure due to the drift trap in DRIFTCORR). This is
;         done to prevent a solution being found on the basis of only NPLATINIT
;         elementary samples of a virtual slew defined in the glitch trap.
;         On input to DRIFTCORR the plateau structure should be the original.
;         TOORIG   - (NPLAT) identifier of first point on a plateau
;         FROMORIG - (NPLAT) identifier of last point on a plateau
;         VIGPORIG - (NPLAT) value of vignetting factor (normally <=1) on a 
;                     plateau
;         KEYPORIG - (NPLAT) value of natural grid identifier on a plateau
;-----------------------------------------------------------------------------
;         SILMIN,SILMAX minimum and maximum possible values for SILLUMPREV 
;                     (used to control the cutting algorithm in 
;                      SOLVEBACKILLUM1).
;         FITPREC      gives the precision to which the best fit is found in
;                      SOLVEBACKILLUM1. 
;         IMODEL       = 1 for Foukes-Schubert =2 for exponential.
;         ISQUARE      parameter specifying which quantity the optimisation
;                      seeks to minimise (see desrption of eg SOLVEBACKILLUM1.
;         SKYPASS      Array of dimension total number of samples
;                      in plateaus processed up to the call to DETINT.
;                      Contains solution for sky brightness (ie solution from
;                      SOLVEILLUM1 divided by vignetting) found sequentially 
;                      for each plateau.
;         KEYPOINT   (1D array (INTEGER) of whole timeline of observation).  
;                     For each time sample 
;                    (including data taken on board but not transmitted 
;                     in fast readout modes) this contains the integer key K
;                     to the nearest direction on the P32 natural grid.
;                     K= iy + iz * nygrid,  where iy,iz are the 2D
;                     pointing positions and the grid has dimensions NYGRID,
;                     NZGRID, though this information is not needed, and not
;                     used by this routine.
;         NPLATINIT    Number of plateaus in sequence at start of fine pointing
;                      to use in the determination of SIGNALPREV, SILLUMPREV.
;         IPT1         First sample of initial s/c fine pointing being 
;                      processed in calling (DRIFTCORR) routine.
;         IPTEND1      Last sample of s/c fine pointing being 
;                      processed in calling (DRIFTCORR) routine.
;                       
; common MOREPAR:     BETASHOOT
;
; output: 
;         SIGNALPREV (single number in V/s): actual detector response at time
;                     zero immediately prior to the initial flux step from 
;                     SILLUMPREV to the SILLUMCALC. Only meaningful when 
;                     combined with
;                     SILLUMPREV to define a unique detector starting state.
;         SILLUMPREV  (single number in V/s): illumination immediately prior
;                     to illumination step at time zero, constrained to
;                     be SILLUMCALC(0). Only meaningful when combined with
;                     SIGNALPREV to define a unique detector starting state.
;
; internal variables:
;         SILLUMCALC  array containing illumination for the detector at the 
;                     times in TCALC, derived from the last occurence of the
;                     first NPLATINIT plateaus in the initial s/c pointing
;                     being processed by the calling routine (DRIFTCORR)
;         TCALC       array containing times at which good transmitted
;                     data were received in the first s/c fine pointing over
;                     the  first NPLATINIT plateaus in the initial s/c pointing
;                     being processed by the calling routine (DRIFTCORR)
;         SIGNALCALC  array containing signal for the detector at the 
;                     times in TCALC   
;/usr/users/vlk/rjt/IDL/PROG/ERDTOMAP
;
@common_morepar
;common morepar,gammaexp,zerobias,betashoot,taushoot,gammaexpshoot,$
;betacoeffshoot,betaexpshoot,tauoffsetshoot,tauoffset,betaexp,betacoeff
;
;save,/xdr,beta,tau,nplatinit,ipt1,iptend1,fromorig,toorig,keyporig,vigporig,$
;t,signal,rms,itrans,mask,skypass,fitprec,imodel,silmin,silmax,isquare,$
;keypoint,bestsky,file='detinit.xdr'
;
if(imodel ne 1 and imodel ne 2 and imodel ne 3)then begin
print,'DETINIT: ****** warning implementation for model not yet made:'
print,'... imodel requested = ',imodel
endif ; imodel ne 1 and imodel ne 2 and imodel ne 3
;
; find first and last plateaus IRAMP1 IRAMP2 corresponding to IPT1 and IPTEND1
; modified if necessary to avoid short plateaus
;
findramp,fromorig,toorig,ipt1,jpt1,iramp1
findramp,fromorig,toorig,iptend1,jpt2,iramp2
;to avoid incomplete ramps take previous ramp to iramp2
iramp2=iramp2-1
; ... but reverse this if we by chance happen to be within
; 4 points of the end of a plateau
if(iptend1+4 lt n_elements(keypoint)) then begin
if(keypoint(iptend1+4) ne keypoint(iptend1))then iramp2=iramp2+1
endif ; iptend1 lt n_elements(keypoint)-1
;to avoid short initial plateau take first ramp to be iramp1+1
if(ipt1 ge 4)then begin ; no change if we are at start of data set
iramp1=iramp1+1
; ... but reverse this if we by chance happen to be within
; 4 points of the start of a plateau
if(keypoint(ipt1-4) ne keypoint(ipt1))then iramp1=iramp1-1
endif ; ipt1 gt 4
;
irampmid=iramp1+nplatinit-1
if(irampmid ge iramp2)then begin
print,'DETINIT warning - not enough plateaus on fine pointing ...'
print,'    ... iramp1,irampmid,iramp2 = ',iramp1,irampmid,iramp2
goto, c999
endif ; irampmid ge iramp2
;
skypassall=skypass(fromorig(iramp1):toorig(iramp2))
keypointall=keypoint(fromorig(iramp1):toorig(iramp2))
signalall=signal(fromorig(iramp1):toorig(iramp2))
itransall=itrans(fromorig(iramp1):toorig(iramp2))
maskall=mask(fromorig(iramp1):toorig(iramp2))
;
; find last plateau of initial nplatinit which is repeated before end of fine
; pointing
;
iplat=irampmid+1
c100:
iplat=iplat-1
if(iplat lt iramp1)then begin
print,'DETINIT warning - no plateau repeated on fine pointing ...'
print,'    ... iramp1,irampmid,iramp2 = ',iramp1,irampmid,iramp2
goto,c999
endif ; iplat lt iramp1
ktest=where(keyporig(iramp1:iramp2) eq keyporig(iplat),nktest)
if(nktest gt 1)then begin
iplat1=iramp1
iplat2=iplat
goto, c200
endif ; nktest gt 1
goto,c100
c200:
;
; set up SILLUMCALC TCALC SIGNALCALC RMSCALC
;
ncalc=toorig(iplat2)-fromorig(iplat1)+1
sillumcalc=fltarr(ncalc)
sigendcalc=sillumcalc ; to be filled with average of signal in each
;                       plateau on last sweep
;
; find median signal FIRSTSIG on first plateau and first illumination 
; FIRSTILL (to use as a bias in call to
; SOLVEBACKILLUM2) from whole timeline from plateau iramp1 to plateau iramp2:
;
ktest=where(keypointall eq keyporig(iplat1) and maskall eq 0 and $
itransall eq 1,nktest)
firstsig=signalall(ktest)
firstsig=median(firstsig)
firstill=skypassall(ktest)
firstill=median(firstill)
;print,'DETINIT: firstsig = ',firstsig
;
; find number of complete cycles nktestmin
;
for iplat=iplat1,iplat2 do begin
;
ktest=where(keyporig(iramp1:iramp2) eq keyporig(iplat),nktest)
if(iplat eq iplat1)then nktestmin=nktest
if(iplat gt iplat1 and nktest lt nktestmin)then nktestmin=nktest
if(nktestmin lt 2)then begin
print,'DETINIT warning - no plateau repeated on for iplat = ',iplat
print,'    ... where iplat1,iplat2 = ',iplat1,iplat2
goto, c999
endif ; nktest lt 2
;
endfor ;  iplat=iplat1,iplat2 

for iplat=iplat1,iplat2 do begin
;
ktest=where(keyporig(iramp1:iramp2) eq keyporig(iplat),nktest)
;
iplatlast=iplat1+ktest(nktest-1)
dummy=skypass(fromorig(iplatlast):toorig(iplatlast))*vigporig(iplatlast)
dummysig=signal(fromorig(iplatlast):toorig(iplatlast))
itdum=itrans(fromorig(iplatlast):toorig(iplatlast))
maskdum=mask(fromorig(iplatlast):toorig(iplatlast))
ii=where(itdum eq 1 and maskdum eq 0,nii)
if(nii gt 0)then begin
dummy=dummy(ii)
dummysig=dummysig(ii)
i1=fromorig(iplat)-fromorig(iplat1)
i2=toorig(iplat)-fromorig(iplat1)
sigendcalc(i1:i2)=total(dummysig)/float(nii) ; average signal on plateau
; assign sillumcalc to skypass if only two sweeps available, to avoid
; 50 percent statistical contamination with first sweep. Otherwise
; take best statistical estimate of illumination.
;print,'*** nktest = ',nktest 
if(nktestmin eq 2)then sillumcalc(i1:i2)=total(dummy)/float(nii)
if(nktestmin gt 2)then $
sillumcalc(i1:i2)=bestsky(keyporig(iplatlast))*vigporig(iplatlast) ; rjt Jan00
endif ; nii gt 0
;
endfor ; iplat=iplat1,iplat2
;
tcalc=t(fromorig(iplat1):toorig(iplat2))
signalcalc=signal(fromorig(iplat1):toorig(iplat2))
maskcalc=mask(fromorig(iplat1):toorig(iplat2))
rmscalc=rms(fromorig(iplat1):toorig(iplat2))
itranscalc=itrans(fromorig(iplat1):toorig(iplat2))
kcalc=keypoint(fromorig(iplat1):toorig(iplat2))
;
;save,/xdr,beta,tau,nplatinit,ipt1,iptend1,fromorig,toorig,keyporig,vigporig,$
;t,signal,rms,itrans,mask,skypass,fitprec,imodel,silmin,silmax,isquare,$
;keypoint,bestsky,sillumcalc,tcalc,signalcalc,rmscalc,maskcalc,itranscalc,$
;sigendcalc,ech,kcalc,iplat1,iplat2,iramp1,iramp2,firstill,firstsig,$
;skypassall,keypointall,signalall,itransall,maskall,file='detinit.xdr'
;
igood=where(itranscalc eq 1 and maskcalc eq 0,nigood)
if(nigood le 0)then begin
print,'DETINIT warning - no good transmitted points on repeated plateaus ...'
print,'    ... where iplat1,iplat2 = ',iplat1,iplat2
goto,c999
endif ; nigood le 0
signalcalc=signalcalc(igood)
tcalc=tcalc(igood)
rmscalc=rmscalc(igood)
sillumcalc=sillumcalc(igood)
sigendcalc=sigendcalc(igood)
kcalc=kcalc(igood)
;
; to avoid offsets in illumination which don't cause much difference in chi
; bias ILLUMCALC half way towards SIGENDCALC, the equivalent signal
; on the last sweep. By running sucessive iterations we should get
; a good chi-squared fit without being divergent.
; JUL00 we constrain now to the level of the signal - not half way to this.
;
pow=1.0 ; set to 1.0 for statistical equality illumination and signal on last
        ; chop 
sillendav=median(sillumcalc)
;sillendav=total(sillumcalc)/float(n_elements(sillumcalc))
sigendav=median(sigendcalc)
;sigendav=total(sigendcalc)/float(n_elements(sigendcalc))
if(pow ne 0.0)then begin
pincf=pow*(sigendav-sillendav)
firstill=firstill+pincf
sillumcalc=sillumcalc+pincf
if(min(sillumcalc) le silmin)then begin
print,'DETINIT: warning - SILLUMCALC positivity correction needed'
sillumcalc=sillumcalc-min(sillumcalc)+silmin
endif ; min(sillumcalc) le silmin
endif ; pow ne 0.0
;
; find signalprev,sillumprev
;
if(imodel eq 1 or imodel eq 2)then begin
solvebackillum1,beta,tau,signalprev,sillumprev,$
sillumcalc,tcalc,signalcalc,rmscalc,$
fitprec,imodel,silmin,silmax,isquare,fitresult,modelfit,ierr
signalprevshoot=0.0
endif ; imodel eq 1 or imodel eq 2
;
if(imodel eq 3)then $
;ifindshoot=0 ; 1d search in s0 only - s0shoot set to 0.0
ifindshoot=1 ; 2d search in s0 and s0shoot
solvebackillum2,beta,tau,signalprev,signalprevshoot,sillumprev,$
sillumcalc,tcalc,signalcalc,rmscalc,firstsig,firstill,$
fitprec,imodel,silmin,silmax,isquare,ifindshoot,fitresult,modelfit,ierr
;
goto,c999 ; to re-comment this one must ensure that ipt1 is replace by the
          ; the first point in the timeline for solvebackillum2 etc, which
;           may have been change by te iramp1=iramp1+1 statement above.
;
; now constrain sillumprev to be the actual illumination on the sample 
; prior to the starting sample, by finding the last occurance of this 
; illumination in the timeline. Ammend signalprev to satisfy the s0 constrain
; (see comments in SOLVEBACKILLUM1). In the following calls to setshootbeta,
; setshoottime, SILLUMPREV is SILLUMCALC(0), as set in SOLVEBACKILLUM2
; to force s0=signalprev etc.  This can only be done if there are good 
; points on the first plateau.
;
setshootbeta_model3,beta,sillumcalc(0),betalong,betash
setshootbeta_model3,beta,sillumprev,betalongprev,betashprev
setshoottime_model3,tau,sillumprev,tlongprev,tshootprev
setshoottime_model3,tau,sillumcalc(0),tlong,tshoot  
;
; for imodel=3 find s0, s0shoot, which (as defined in SOLVEBACKILLUM2) are 
; unique quantities as combinations of signalprev, sillumcalc(0) and sillumprev
; and of signalprevshoot, sillumcalc(0) and sillumprev respectively:
; note there will always be a value for S0shoot even when betashoot is set to
; zero due to the artificial constraint sillumprev=sillumcalc(0) applied in
; SOLVEBACKILLUM2.
;
setjump_model3,sillumcalc(0),sillumprev,signalprev,signalprevshoot,$
tlong,tlongprev,betalong,betalongprev,betash,betashprev,$
tshoot,tshootprev,s0,s0shoot
;
if(ipt1 eq 0)then begin ; we are at first point of data set which must be
                        ; the start of a chopper sweep
;klast=max(kcalc) ; must be end of chopper sweep (actually wrong since 
;                  observation may start from central position for P32)
klast=keypoint(ipt1)+max(ech)/2 ; for P32 +6 for C100 and +3 for C200
endif ; ipt1 eq 0
;
if(ipt1 gt 0)then begin
klast=keypoint(ipt1-1) ; pointing of previous sample
endif ; ipt1 gt 0
;
;ii=where(kcalc eq klast,nii)
;if(nii gt 0)then begin
;iii=ii(nii-1)
;
ii=where(keypointall eq klast,nii)
if(nii gt 0)then begin
iii=ii(nii-1)
;
;sillumprev=sillumcalc(iii) ; illum. at last occurence of pointing prior to
;                            first in timeline 
sillumprev=skypassall(iii)
;
kfirst=keypoint(ipt1)
;
;jj=where(kcalc eq kfirst,njj)
jj=where(keypointall eq kfirst,njj)
if(njj le 0)then goto,c999 ; no points on first plateau so can't perform
;                            calculation
;
jjj=jj(njj-1)
sillumstart=skypassall(jjj)
;sillumstart=sillumcalc(jjj) ; illum at last occurence of first pointing 
;                             direction in timeline
;
if(imodel eq 1 or imodel eq 2)then signalprev=s0-beta*(sillumstart-sillumprev)
if(imodel eq 3)then begin $
;
setshootbeta_model3,beta,sillumstart,betalong,betash
setshootbeta_model3,beta,sillumprev,betalongprev,betashprev
setshoottime_model3,tau,sillumprev,tlongprev,tshootprev
setshoottime_model3,tau,sillumstart,tlong,tshoot  
;
inversejump_model3,sillumstart,sillumprev,signalprev,signalprevshoot,$
tlong,tlongprev,betalong,betalongprev,betash,betashprev,$
tshoot,tshootprev,s0,s0shoot
endif ; imodel eq 3
endif ; nii gt 0
if(nii le 0)then goto,c999
;
c999:
;
return
end








