pro dg2,pixarray,iviguse,ireplace,ibias,iusefine,fractbadscp,$
tglitch,thresherd1,thresherd2,implementerd,implementscp,ifine1,ifine2
;
; deglitches at "SCP" level the ERD data for P32, using the redundancy of the 
; different sky directions at different times. This is to catch proton hits.
; The contents of the SIGNAL common block (ERD resolution) are changed 
; according to the 
; deglitch if IMPLEMENTSCP is 1: BAD is set to the internal parameter 
; IDG2BADVALUE1 for plateaus found to be bad. If ireplace is 1, and the first 
; occurence of a plateau for a given pointing direction is found to be bad, 
; the ERD data for that plateau is overwritten by the mean of the data
; for all complete subsequent plateaus seen in the same direction seen in the 
; same spacecraft pointing, scaled to the level of the preceding and 
; postceding plateaus, if these are good. In this case, the BAD array is 
; incremented by to the value IDG2BADVALUE2, and the array DG2BAD is set to
; IDG2BADVALUE2 for the points incremented in BAD. This is to allow the
; patched plateaus to become temporarily visible to DRIFT32 (via PREPAREDRIFT)
; which should allow the illumination history to be calculated without
; breaks in the data sequence for which illumination has to be guessed. 
; PREPAREDRIFT subtracts DG2BAD from BAD, a procedure which is reversed
; after DRIFTCORR of DETSELFCAL runs have been completed. This propcedure 
; leaves the original bad points such as from the start of the read cycle bad. 
;
; DG2 uses information from the ERD spike deglitching from the DG1 call
; in ERDTOMAP to decide on the probability that an SCP plateau is good. This
; is the case if the internal parameter ITWOMETHOD is 2, in which case 
; bad pairs are distinguished between on the basis of how close they were
; to spikes found by DG1. The user must supply an estimate for the time 
; constant of the proton tail decay TGLITCH (in s) and also THRESHERD1,
; the minimum sigma of ERD spikes to be used in this determination.
;
; Additionally, and independently of whether the ERD glitch information
; is really used in the SCP deglitch, the ERD arrays can be flagged bad
; for all the region where the cumulated ERD glitch tails are above
; THRESHERD2. This is useful as an additional check since DG1 cannot flag 
; more than a certain proportion of chopper plateau samples bad within each
; plateau. This additional masking is done by setting IMPLEMENTERD to 1.
; The plateau patching, if requested, is done after the SCP and the ERD 
; masking by running ERDTOSCP again, and then if IREPLACE is 1, doing the
; patching.
;
; Note that the routine can be run more than once - eg first to flag the ERD
; glitch tails and then to do the SCP deglitch. It may not be sensible to
; do both SCP and ERD deglitching in the same run, though this is possible;
; in this case the SCP deglitch is done before the ERD, as there is no explicit
; coupling between the two deglitch stages (ie the SCP plateaus are not 
; censored according to the ERD glitch tails).
;
; The SCP deglitch is done as a clip, removing a fraction of FRACTBADSCP of the
; SCP plateaus for each pixel.
;
; If IMPLEMENTSCP is 1 the ERD and SCP mask arrays are overwritten with the
; bad plateaus found in the SCP deglitch, and plateaus patched where the mask 
; is applied to the first time a direction is viewed, according to the value 
; of IREPLACE.
;
; If IMPLEMENTSCP is not 1 and implementERD is not 1, 
; IREPLACE is ignored and there is no mask overwriting and no patching.
;
; If IUSEFINE is 1, only the data on fine pointings is used.
;
; To be added: if IREPLACE is 3, replace the first occurance of a bad
; plateau on each s/c fine pointing, rather than the first occurance in the 
; whole data set. This is to allow DRIFT32 to run in a sequential mode, taking
; each s/c pointing independently.
;
; To be added: - iverbose for plots
;
; As explained by the comments in the code, DG2 incorporates measures 
; to attempt to take into account the conflicting demands of
; an often small (down to 2) number of plateaus in a given sky direction on a
; spacecraft pointing, the systematic different level of plateaus in a given 
; sky direction on difference spacecraft pointings (at least, prior to the
; responsivity drift), and the expectation that there will be a dependence
; spread in plateau levels for a given direction depending on the
; step interval from the previous plateau (which may depend on the s/c 
; pointing).   
;
; It may be useful to run DG2 after, as well as before a drift correction 
; implementscpation. This is because an imperfection in the drift correction,
; combined with a situation where there is a varying gap between the
; fit to the signal and the illumination accross the plateaus on a given
; fine pointing (which is a common situation if the fine pointing is entered
; with a wildly wrong initial detector state) will give rise to discrepent
; corrected plateau levels, especially in the first plateaus of a fine
; pointing. This effect is not always masked out by DRIFTCORR due to the
; individual SOLVEILLUM1 fits to the plateaus going out of bounds.
;
; DG2 is run from, and operated on unexpanded ERD data in all cases,
; and so must be called before EXPAND32 in DRIFT32.
;
; example of use before a DRIFT32 implementation (pure SCP deglitch)
; also deglitching on slews,
; using whole time line, replacing the "worst" 0.1 fraction of the data,
; and not implementscping the solution and no bias:
; (parameters are: 
;
; pixarray,iviguse,ireplace,ibias,iusefine,fractbadscp,tglitch,
;$thresherd1,thresherd2,implementerd,implementscp,ifine1,ifine2
;
; dg2,pixarray,1,1,0,0,0.1,1.0,20.0,5.0,0,0,0,nyraster*nzraster-1
;
; example of use after a DRIFT32 implementation, again pure SCP deglitch,
; only deglitching on fine pointings, using only the first 3 s/c pointing 
; (eg the first subscan), not implementing the solution and not patching the 
; instances where the first occurance of a plateau is found to be bad. 
; No bias.
;
; dg2,pixarray,0,0,0,1,0.1,1.0,20.0,5.0,0,0,0,2
;
; note that DG2 only operates on data for the detector pixels specified by
; PIXARRAY for optimum speed in interactive work. This also means that
; by calling DG2 successively for different detector pixels or a 
; different pixel/fine pointing range combination a taylored deglitch solution
; can be obtained.
;
; Input:
;  PIXARRAY  array specifying detector pixels to deglitch.
;  IMPLEMENTSCP =1 to overwrite the BAD array in the common block SIGNAL
;  FRACTBADSCP     fraction of points in the fine pointing range specified 
;               to flag bad
;  IVIGUSE   =1 to divide by vignetting factors before deglitch (case where
;               DG2 is called before DRIFT correction) =0 otherwise
;  IREPLACE  =1 to replace ERD data in the first instance of a bad plateau
;               found in the timeline with all elements bad with a suitably 
;               scaled average of subsequent plateaus viewing the same sky 
;               direction on the same s/c pointing.
;  IREPLACE  =2 to replace ERD data in the first instance of a bad plateau
;               found in the timeline with all elements bad with a suitably 
;               scaled average of subsequent plateaus viewing the same sky 
;               direction on the same s/c pointing. If there are no 
;               other SCP plateaus viewing the same sky direction on the same 
;               s/c pointing the patch is done with the average of all 
;               other SCP plateaus taken in that direction from other 
;               pointings. If this is not possible, the patch is done with 
;               the median of all scp plateaus on the current pointing.
;  IBIAS     =1 to bias SCP glitcher away from bright source peaks
;  IFINE1,IFINE2 range off fine pointings for deglitch. Usually, these should
;               start at the beginning of a scan line and end at the end of
;               a succeeding scan line. Otherwise the statistics for the
;               deglitch will be less than optimum.
;  THRESHERD1   threshold for amplitude of ERD glitches to be included
;  THRESHERD2   threshold for amplitude of ERD glitche tails to be flagged
;  TDTNAM ISEQ  (common names)
;  S,BAD,RMS    (common SIGNAL)
;  DTIME         (common TIMER)
;  MAP3DREF,MAP3DREFN (common MAP3DREF)
;
; Output:
;  COMMON block DG2BLOCK
;  IDG2BADVALUE1 value assigned by DG2 to the ERD BAD array for plateaus 
;                found to be glitchy.
;  IDG2BADVALUE2 value added by DG2 to the ERD BAD array for a plateau found 
;                to be glitchy if it is the first fine pointing 
;                view of a sky direction in the timeline and has been 
;                substituted by a suitably scaled statistical average of good 
;                plateaus in the same pointing plateau.
;  DG2BAD        integer array (NPIX,NPTIN) with value IDG2BADVALUE2 for data
;                patched (for which IDG2BADVALUE2 has been added to BAD)
;                = 0 otherwise. This is initialised in ERDTOMAP. It is
;                re-initialised to zero only for the pixels and data range
;                to be calculated by DG2, and only if IREPLACE is set to 1 or
;                2. This allows sequential use of DG2 for various pixel and
;                s/c pointing ranges. For all patches implemented the
;                corresponding values of DG2BAD are set to IDG2BADVALUE2 by 
;                PATCHER. This allows a simple subtraction of DG2BAD from
;                BAD in PREPAREDRIFT called by DRIFT32 to allow DRIFT32 to use
;                the patched data to maintain a timeline calculation.
;  COMMON block SCPBLOCK
;  S,BAD,RMS    (common SIGNAL)
;  STEP         (common PLOTTER)
;  FPT          (common PINFO)
;
@common_signal
;common signal,rs,cs,nch,s_in,s,badin,bad,rel,rms,glitch,jf,njf,ninchmax,rd,$
;nkf,kf,nkfmax,irelstart,nchopmed,gbad,npolyrms,nrmsmed,sigmaxfract
;
@common_plotter
;common plotter,otfstart,otfstop,rdstat,chop,step,otf,rpid,yrel,zrel,$
;ytrue,ztrue,yzrpidcen,nyraster,nzraster,racenout,deccenout
;
@common_pinfo
;common pinfo,fpt
;
@common_timer
;common timer,time,dtime,complete
;
@common_names
;common names,tdtnam,iseq,dirnam,caldir,cfilter
;
@common_dg2block 
;common dg2block,idg2badvalue1,idg2badvalue2,dg2bad
;
@common_scpblock
;common scpblock,s_scp,bad_scp,rms_scp,n_scp,kpt_scp,spt_scp,fpt_scp,$
;from_scp,to_scp,time_scp,vig_scp,nplat_scp
;
@common_map3dref
;common map3dref,map3dref,map3drefrms,map3drefd,map3drefy,map3drefz,map3drefn
;
@common_steerin
;common steerin,array,nptin,npt,ninramp,imeas,osamp
;
loadct,2 ; GRN-RED-BLU-WHT
;
print,'DG2 for pointing range ',ifine1,ifine2 
print,'... and pixarray =',byte(pixarray)
print,'... iviguse,ireplace,iusefine,ibias  =',byte(iviguse),byte(ireplace),$
byte(iusefine),byte(ibias)
print,'... fractbadscp,tglitch              =',fractbadscp,tglitch
print,'... thresherd1,thresherd2            =',thresherd1,thresherd2
print,'... implementerd,implementscp        =',implementerd,implementscp
;
idg2badvalue1=3 ; value written to BAD array for plateaus found to be bad
idg2badvalue2=100 ; value written to BAD array for first instance of a 
;                   pointing direction is found to be bad
;
; find unexpanded ERD sample range I1ERD to I2ERD of fine pointing data to be 
; deglitched i1erd:i2erd (NB if slews are used the first instance of a 
; pointing direction may not be on a fine pointing - actually not true
; since we use SPT_SCP, the s/c fine pointing indicator, in the WHERE 
; statements below and SPT_SCP is set by INITSCP to be -1 on slews). 
; Also find SCP range I1SCP to I2SCP, starting and ending with the first and 
; last plateau with all consituent ERD data points pointing towards the 
; specified s/c pointing.
;
if(iusefine eq 1)then ii=where(spt_scp eq ifine1 and fpt_scp eq 1,nii)
if(iusefine eq 0)then ii=where(spt_scp eq ifine1,nii)
i1scp=ii(0)
i1erd=from_scp(i1scp)
if(iusefine eq 1)then ii=where(spt_scp eq ifine2 and fpt_scp eq 1,nii)
if(iusefine eq 0)then ii=where(spt_scp eq ifine2,nii)
i2scp=ii(nii-1)
i2erd=to_scp(i2scp)
;
; find array of chopper sweep boundaries in ERD coordinates
steptest=step(i1erd+1:i2erd)-step(i1erd:i2erd-1)
ichsweep=where(abs(steptest) gt 3,nichsweep) ; covers C200 (chop position 
;      switches from 7 to 1) and C100 (chop position switches from 13 to 1).
ichsweep=i1erd+ichsweep
;
; set up MAPREF refernce brightness map needed in the IBIAS option
;
ny=n_elements(map3dref(0,*,0))
nz=n_elements(map3dref(0,0,*))
mapref=fltarr(ny,nz)
npix=n_elements(map3dref(*,0,0))
medallmap=median(map3dref)
for iy=0,ny-1 do begin
for iz=0,nz-1 do begin
itot=0
;
for ipix=0,npix-1 do begin
if(map3drefn(ipix,iy,iz) gt 0)then begin
mapref(iy,iz)=mapref(iy,iz)+map3dref(ipix,iy,iz)
itot=itot+1
endif ; map3drefn(ipix,iy,iz) gt 0
endfor ; ipix
if(itot le 0)then mapref(iy,iz)=medallmap
if(itot ge 1)then mapref(iy,iz)=mapref(iy,iz)/float(itot)
endfor ; iz
endfor ; iy
;
for pix=0,n_elements(pixarray)-1 do begin ; *** main detector pixel loop
;
ipix=pixarray(pix)
print,'DG2: processing data for pixel',ipix
;
; set up data dependent arrays in SCPBLOCK (plateau structure already set up
; for all detector pixels in call to INITSCP in ERDTOMAP) from unexpanded data
; in SIGNAL common block:
;
erdtoscp,ipix
;
; plot ERD and SCP data over range to be deglitched
;
iwindow=10+ipix
if(ipix eq 0)then tit=tdtnam+' '+iseq+' PIXEL 1'
if(ipix eq 1)then tit=tdtnam+' '+iseq+' PIXEL 2'
if(ipix eq 2)then tit=tdtnam+' '+iseq+' PIXEL 3'
if(ipix eq 3)then tit=tdtnam+' '+iseq+' PIXEL 4'
if(ipix eq 4)then tit=tdtnam+' '+iseq+' PIXEL 5'
if(ipix eq 5)then tit=tdtnam+' '+iseq+' PIXEL 6'
if(ipix eq 6)then tit=tdtnam+' '+iseq+' PIXEL 7'
if(ipix eq 7)then tit=tdtnam+' '+iseq+' PIXEL 8'
if(ipix eq 8)then tit=tdtnam+' '+iseq+' PIXEL 9'
window,iwindow,tit=tit,xs=800,ys=400,xpos=470,ypos=570
;
; plot good ERD data in red
igooderd=where(bad(ipix,i1erd:i2erd) eq 0,nigooderd)
ymin=min(s(ipix,i1erd+igooderd))
ymin=min([ymin,0.0]) ; make plot extend at least down to zero in Y
ymaxdata=max(s(ipix,i1erd+igooderd))
ymax=ymaxdata+0.3*(ymaxdata-ymin) ; upper margin for ancillary plots
color=37 ; red for colour table 2
plot,i1erd+igooderd,s(ipix,i1erd+igooderd),yran=[ymin,ymax],xst=1,yst=1,$
color=color
;
; overplot good SCP plateaus in green, including error bars, in ERD coordinates
for i=i1scp,i2scp do begin
if(bad_scp(ipix,i) eq 0)then begin
color=15 ; green for colour table 2
oplot,[from_scp(i),to_scp(i)],[s_scp(ipix,i),s_scp(ipix,i)],color=color
xmid=(from_scp(i)+to_scp(i))/2
oplot,[xmid,xmid],$
[s_scp(ipix,i)-rms_scp(ipix,i),s_scp(ipix,i)+rms_scp(ipix,i)],color=color
endif ; bad_scp(ipix,i) eq 0
endfor ; i=i1scp,i2scp
;
; overplot chopper sweep boundaries as dotted white line in ERD coordinates
for i=0,nichsweep-1 do begin
oplot,[ichsweep(i),ichsweep(i)],[ymin,ymax],linestyle=1
endfor ; i=0,nichsweep-1
;
; overplot fine pointing flag at top of plotting area
oplot,i1erd+findgen(i2erd-i1erd+1),$
ymax-0.35*(ymax-ymaxdata)+0.2*(ymax-ymaxdata)*fpt(i1erd:i2erd)
;
; find and overplot glitches found in ERD. Use whole ERD range to 
; allow inclusion of tails from ERD glitches found prior to the requested 
; range. The simple median is a reasonable estimate, as we are working on 
; non-expanded data.
;
;rmserdmed=median(rms(ipix,igooderd))
rmserdmed=median(rms(ipix,*)) ; median rms of ERD data
nn=n_elements(glitch(0,*))
glitcherd=fltarr(nn)
glitcherd(*)=abs(glitch(ipix,*)/rmserdmed) ; in units of sigma
glitcherdthresh=thresherd1
inoglitcherd=where(glitcherd lt glitcherdthresh,ninoglitcherd)
if(ninoglitcherd gt 0)then glitcherd(inoglitcherd)=0.0
scale=1.0/100.0 ; when multiplied by scale a glitch of 100 sigma
;                             will reach top of plot
oplot,i1erd+findgen(i2erd-i1erd+1),$
ymax-1.0*(ymax-ymaxdata)+(ymax-ymaxdata)*glitcherd(i1erd:i2erd)*scale
;
; overplot in blue the presumed glitch tails as a function of the glitch
; time constant TGLITCH (input parameter)
;
if(array eq 'c100')then xdum=13.0 ; number of plateaus in sweep
if(array eq 'c200')then xdum=7.0
xntail=xdum*float(n_elements(glitch(0,*)))/float(nplat_scp)
ntail=long(xntail) ; average number of ERD samples in a chopper sweep,
                   ; which is set to the length of the exponential tail
ntail=2*ntail ; two chopper sweeps is maximum length
;tglitch=1.0
xndecay=tglitch/dtime ; number of samples in glitch time constant 
ndecay=long(xndecay)
tail=fltarr(ntail)
tail=exp(0.0-findgen(ntail)/xndecay)
iglitcherd=where(glitcherd ge glitcherdthresh,niglitcherd)
glerd=fltarr(nn)
if(niglitcherd gt 0)then begin
glitcherd=glitcherd(iglitcherd)
;
for i=0,niglitcherd-1 do begin
i1=iglitcherd(i)
i2=i1+ntail-1
;
if(i2 gt nn-1)then begin
i2=nn-1
ndum=i2-i1+1
dum=fltarr(ndum)
dum=exp(0.0-findgen(ndum)/xndecay)
glerd(i1:i2)=glerd(i1:i2)+glitcherd(i)*dum
endif ; i2 gt nn-1
;
if(i2 le nn-1)then glerd(i1:i2)=glerd(i1:i2)+glitcherd(i)*tail
;
endfor ; i=0,niglitcherd-1 
;
color=100 ; violet
oplot,i1erd+findgen(i2erd-i1erd+1),$
ymax-1.0*(ymax-ymaxdata)+(ymax-ymaxdata)*glerd(i1erd:i2erd)*scale,$
color=color
; also replot the ERD spikes in white
for i=0,niglitcherd-1 do begin ; second loop
dum1=ymax-1.0*(ymax-ymaxdata)
dum2=dum1+(ymax-ymaxdata)*glitcherd(i)*scale
oplot,[iglitcherd(i),iglitcherd(i)],[dum1,dum2]
oplot,[0,nn-1],[dum1,dum1] ; baseline for cosmetics
endfor ; i=0,niglitcherd-1 second loop
;
endif ; niglitcherd gt 0
;
; set up GLSCP(nplat_scp), the average value of GLERD on each plateau:
;
glscp=fltarr(nplat_scp)
for iplat=0,nplat_scp-1 do begin
ninplat=to_scp(iplat)-from_scp(iplat)+1
glscp(iplat)=total(glerd(from_scp(iplat):to_scp(iplat)))/float(ninplat)
endfor ; iplat=0,nplat_scp-1
;
; prior to deglitch precondition (relativise, apply vignetting to output data
; if specified) the data and pass quantity to be deglitched into array XDG
;
kmin=min(kpt_scp(ipix,i1scp:i2scp))
kmax=max(kpt_scp(ipix,i1scp:i2scp))
istart=0
for kp=kmin,kmax do begin
;
;itwomethod=1 ; for just two samples per sky direction per s/c direction 
             ; apply offset to the statistic such that the point with the
             ; lowest overall signal is zero
itwomethod=2  ; If the ERD level of a proton hit is greater in one plateau 
              ; than the other, then take the glitch to be in the plateau
              ; with the highest value of GLSCP. I there are no glitches in
              ; either plateau then procede as for itwomethod=1.
;
preglitch2,kp,i1scp,i2scp,glscp,thresherd2,iusefine,iviguse,itwomethod,ipix,$
x,g,xminus,xplus,spt,igood,nigood,badprev,badpost,kprev,kpost,nrel
;
;
; bias xdg to lower values for larger fluxes if ibias is 1, using map
; template from all map pixels in common MAP3REF
;
if(ibias ge 1)then begin
ktoyz,kp,iyy,izz
x(*)=x(*)/(mapref(iyy,izz)^float(ibias))
endif ; ibias eq 1
;
if(nigood gt 0)then begin
;
if(istart gt 0) then begin
;
xdg=[xdg,x] ; quantity derived from SCP signal for deglitching
igooddg=[igooddg,igood+i1scp] ; original SCP identifiers for XDG
gdg=[gdg,g]                   ; value of average ERD glitch tail
xminusdg=[xminusdg,xminus]    ; prior signal for XDG
xplusdg=[xplusdg,xplus]       ; post signal for XDG
badprevdg=[badprevdg,badprev] ; mask for prior signal
badpostdg=[badpostdg,badpost] ; mask for post signal
kprevdg=[kprevdg,kprev]       ; pointing direction of prior signal
kpostdg=[kpostdg,kpost]       ; pointing direction of post signal
sptdg=[sptdg,spt]             ; s/c pointing
nreldg=[nreldg,nrel]          ; number of plateaus for which relativisation in 
;                               PREGLITCH2 was performed on each s/c 
;                               fine pointing
;
endif ; istart gt 0
;
if(istart eq 0)then begin
;
xdg=x
igooddg=igood+i1scp
xminusdg=xminus
xplusdg=xplus
gdg=g
badprevdg=badprev
badpostdg=badpost
kprevdg=kprev
kpostdg=kpost
sptdg=spt
nreldg=nrel
;
endif ; istart eq 0
;
istart=istart+1
;
endif ; nigood gt 0
;
endfor ;  kp=0,kmin,kmax
;
ntot=n_elements(xdg)
ii=where(abs(xdg) gt 0.001,nii) ; xdg has had an abs applied in preparedg2
ntotnonzero=nii
avdg=median(xdg(ii))
;print,'**** avdg,nii = ',avdg,nii
;
; next look for, and correct for, trends in level of XDG as a function of
; NRELDG (relativisation statistics) for often occuring values of nreldg
;
nrelmin=min(nreldg)
nrelmax=max(nreldg)
relcalc=fltarr(nrelmax-nrelmin+1)
nrelcalc=lonarr(nrelmax-nrelmin+1)
irelcalc=nrelcalc
;
; we ammend spread in errors for values of nrel accounting for more
; than 20 percent of the data:
;
relfractlim=0.20
nrelfractlim=relfractlim*ntotnonzero
;
for irel=nrelmin,nrelmax do begin
; statistics are restricted to the non-zero data points since the
; shifts introduced in PREPAREGLITCH2 may define a certain fraction of 
; points to be zero
ii=where(nreldg eq irel and abs(xdg) gt 0.0001,nii)
relcalc(irel-nrelmin)=0.0
nrelcalc(irel-nrelmin)=0
irelcalc(irel-nrelmin)=-1
if(nii gt nrelfractlim)then begin
relcalc(irel-nrelmin)=median(xdg(ii))
nrelcalc(irel-nrelmin)=nii
irelcalc(irel-nrelmin)=irel
xdg(ii)=xdg(ii)*avdg/relcalc(irel-nrelmin)
endif ; nii gt nrelfractlim
;
;print,'irel,relcalc,nrelcal',irel,relcalc(irel-nrelmin),nrelcalc(irel-nrelmin)
;
endfor ; irel=nrelmin,nrelmax
;
; next look for, and correct for, trends in level of XDG as a function of
; abs of flux step (ie for scatter induced by drift artifacts)
; We don't do this since glitches also count as flux steps!
;
;window,1
;plot,abs(xminusdg-s_scp(ipix,igooddg)),xdg,psym=1
;oplot,(from_scp(igooddg)+to_scp(igooddg))/2.0,xdg,psym=1
;oplot,(from_scp(igooddg)+to_scp(igooddg))/2.0,xdg*2.0,psym=1
;
; next deglitch either sky direction by sky direction (where relativisation
; imperfect due to drift effects, as is to be expected particularly when
; running prior to drift correction on bright sources) or over the whole
; dataset. This is controlled by the input parameter IGLITCHALL being
; 1 (for whole time line) or 0 (for specific directions). For the moment we
; only deglitch over the whole dataset.
;
; fill in an internal BD array for current pixel according to required
; fraction of data to flag bad FRACTBADSCP. 
;
isort=sort(xdg)
xdg=xdg(isort)
gdg=gdg(isort)
igooddg=igooddg(isort)
xminusdg=xminusdg(isort)
xplusdg=xplusdg(isort)
badprevdg=badprevdg(isort)
badpostdg=badpostdg(isort)
kprevdg=kprevdg(isort)
kpostdg=kpostdg(isort)
sptdg=sptdg(isort)
nreldg=nreldg(isort)
n=n_elements(xdg)
xnthresh=(1.0-fractbadscp)*float(n)
nthresh=long(xnthresh)
;print,'n,nthresh,fractbadscp = ',n,nthresh,fractbadscp 
bd=lonarr(n)
bd(nthresh:n-1)=1
;
; overplot bad SCP plateaus in white, including error bars, in ERD coordinates
;
print,'DG2: overplotting SCP mask for    ',n-nthresh,' out of',nplat_scp,$
' SCP plateaus'
if(implementscp eq 1)then $
print,'DG2: writing SCP mask to ERD for  ',n-nthresh,' SCP plateaus'
for i=nthresh,n-1 do begin
ii=igooddg(i) ; plot only last fraction of deviating points
oplot,[from_scp(ii),to_scp(ii)],[s_scp(ipix,ii),s_scp(ipix,ii)],color=color
xmid=(from_scp(ii)+to_scp(ii))/2
oplot,[xmid,xmid],$
[s_scp(ipix,ii)-rms_scp(ipix,ii),s_scp(ipix,ii)+rms_scp(ipix,ii)]
; implement deglitch to ERD mask arrays, if IMPLEMENTSCP is 1. 
if(implementscp eq 1)then begin
bad(ipix,from_scp(ii):to_scp(ii))=$
bad(ipix,from_scp(ii):to_scp(ii))+idg2badvalue1
endif ; implementscp eq 1
endfor ; i=nthresh,n
;
; additionally, overplot plateaus where average level of ERD glitch tail
; exceeds THRESHERD2
;
iexceederd=where(gdg gt thresherd2,niexceederd)
print,'DG2: overplotting ERD mask for    ',niexceederd,' out of',nplat_scp,$
' SCP plateaus'
ininall=where(gdg(0:nthresh-1) gt thresherd2,ninall)
print,'DG2: total overplotted plateaus = ',ninall+n-nthresh,' out of',$
nplat_scp,' SCP plateaus'
if(implementerd eq 1)then $
print,'DG2: writing ERD mask to ERD for  ',niexceederd,' SCP plateaus'
if(niexceederd gt 0)then begin
for i=0,niexceederd-1 do begin
iii=iexceederd(i)
ii=igooddg(iii)
oplot,[from_scp(ii),to_scp(ii)],[s_scp(ipix,ii),s_scp(ipix,ii)],color=color
xmid=(from_scp(ii)+to_scp(ii))/2
oplot,[xmid,xmid],$
[s_scp(ipix,ii)-rms_scp(ipix,ii),s_scp(ipix,ii)+rms_scp(ipix,ii)]
; implement deglitch to ERD mask arrays, if IMPLEMENTERD is 1. 
if(implementerd eq 1)then begin
bad(ipix,from_scp(ii):to_scp(ii))=$     
bad(ipix,from_scp(ii):to_scp(ii))+idg2badvalue1 ; increment to preserve ability
endif ; implementerd eq 1                       ; to plot
endfor ;  i=0,niexceed-1
endif ; niexceederd gt 0
; 
; Rerun ERDTOSCP to redefine common_scpblock for flagged data and
; prepare for patching if requested. Note this won't change anything
; if both implementscp and implementerd were zero
; 
if(implementerd ne 0 or implementscp ne 0)then erdtoscp,ipix
; 
; patching and overwrite of ERD data in blue and scp plateaus in white
;
if(implementscp eq 0 and implementerd eq 0)then goto, c575
if(ireplace ge 1)then begin
npatch=0
dg2bad(ipix,i1erd:i2erd)=0 ; re-initialise range of DG2BAD array for which
                           ; patches may be applied.
kmin=min(kpt_scp(ipix,*))
kmax=max(kpt_scp(ipix,*))
;
for k=kmin,kmax do begin
;
ii=where(kpt_scp(ipix,*) eq k and fpt_scp eq 1,nii)
if(nii gt 0)then begin
;
if(bad_scp(ipix,ii(0)) eq 0)then goto, c277 ; first occurance on fine pointing 
                                            ; is good so no patch
; case first occurrance of direction K is BAD
spk=spt_scp(ii(0)) ; s/c pointing for first view of K 
fmk0=from_scp(ii(0)) ; erd start for first view of K
tok0=to_scp(ii(0)) ; erd end for first view of K
bd=bad_scp(ipix,ii) ; scp mask for fine pointing plateaus in direction K
sp=spt_scp(ii) ; scp s/c pointing for fine pointing plateaus in direction K
fmk=from_scp(ii) ; erd starting sample for scp fine pointing plateaus towards K
tok=to_scp(ii) ; erd ending sample for scp fine pointing plateaus towards K
iii=where(bd eq 0 and sp eq spk,niii) ; (subsequent) good scp plateaus in 
;                                        same s/c pointing viewing K
iiii=where(bd eq 0,niiii) ; (subsequent) good scp plateaus over all 
;                                        s/c pointings viewing K
iiiii=where(spt_scp eq sp(0),niiiii) ; plateau idents on s/c pointing at which
;                                      first occurance of k has no good data 
if(niii le 0 and ireplace eq 1)then begin 
; no possibility for substitution from same plateau
print,'DG2: no patch: from,k,sc,ipix =',fmk(0),k,spk,ipix
;
; oplot ERD data in white from this plateau which is bad but can't be patched
igerd=where(bad(ipix,fmk0:tok0) eq idg2badvalue1,nigerd)
if(nigerd gt 0)then oplot,fmk0+igerd,s(ipix,fmk0+igerd)
; (note that where bad was previously 0 it now takes value idg2badvalue1)
;
endif ; niii le 0 and ireplace eq 1
if(niii le 0 and ireplace eq 2)then begin
; try to patch from same s/c pointing on other plateaus or if this
; is not possible from a median of all data on current s/c pointing
if(niiii le 0)then begin ; no data from this pointing direction on any s/c
; pointing
;print,'DG2: no patch: from,k,sc,ipix =',fmk(0),k,spk,ipix
; oplot ERD data in white from this plateau which is bad but can't be patched
;igerd=where(bad(ipix,fmk0:tok0) eq idg2badvalue1,nigerd)
;if(nigerd gt 0)then oplot,fmk0+igerd,s(ipix,fmk0+igerd)
; (note that where bad was previously 0 it now takes value idg2badvalue1)
;
; patch from all timeline on pointing
print,'DG2: patching from whole pointing: from,k,sc,ipix =',fmk(0),k,spk,ipix
fmkk=from_scp(iiiii)
tokk=to_scp(iiiii)
patcher,ipix,fmk0,tok0,fmkk,tokk 
;
; oplot patched ERD data in blue from this plateau
igerd=where(bad(ipix,fmk0:tok0) eq idg2badvalue2,nigerd)
color=100 ; violet
if(nigerd gt 0)then oplot,fmk0+igerd,s(ipix,fmk0+igerd),color=color
;
endif ; niiii le 0
if(niiii gt 0)then begin
; patch from same s/c pointing on other plateaus
npatch=npatch+1
fmkk=fmk(iiii)
tokk=tok(iiii)
if(ireplace eq 2)then begin
patcher,ipix,fmk0,tok0,fmkk,tokk 
; patcher overwrites S and RMS with a composite and increments BAD with 
; idg2badvalue2 and sets DG2BAD to idg2badvalue2
endif ; ireplace eq 2
;
; oplot patched ERD data in blue from this plateau
igerd=where(bad(ipix,fmk0:tok0) eq idg2badvalue2,nigerd)
color=100 ; violet
if(nigerd gt 0)then oplot,fmk0+igerd,s(ipix,fmk0+igerd),color=color
;
endif ; niiii gt 0
;
endif ; niii le 0 and ireplace eq 2
;
if(niii gt 0)then begin ; can patch from same s/c pointing
npatch=npatch+1
fmkk=fmk(iii)
tokk=tok(iii)
if(ireplace ge 1)then begin
patcher,ipix,fmk0,tok0,fmkk,tokk 
; patcher overwrites S and RMS with a composite and increments BAD with 
; idg2badvalue2 and sets DG2BAD to idg2badvalue2
endif ; ireplace ge 1
;
; oplot patched ERD data in blue from this plateau
igerd=where(bad(ipix,fmk0:tok0) eq idg2badvalue2,nigerd)
color=100 ; violet
if(nigerd gt 0)then oplot,fmk0+igerd,s(ipix,fmk0+igerd),color=color
;
endif ; niii gt 0
;
c277:
endif ; nii gt 0
;
endfor ;  k=kmin,kmax
;
print,'DG2: number of plateaus patched =    ',npatch,' for pixel ',byte(ipix)
;
endif ; ireplace ge 1
;
c575:
;
endfor ; pix=0,n_elements(pixarray)-1 ; *** end of main detector pixel loop
;
c999:
;
return
end








