;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+ 
; NAME: 
;       PICKMEAS 
; 
; PURPOSE: 
;       This function allows the user to interactively pick a measurement
;	from the PIA internal buffers.  A file 
;       selection tool with a simple graphical user interface is created.
; 
; CATEGORY: 
;       PIA - graphics
; 
; CALLING SEQUENCE: 
;       Result = PICKMEAS() 
; 
; KEYWORD PARAMETERS: 
; 
;       GROUP:  The widget ID of the widget that calls PICKFILE.  When this 
;               ID is specified, a death of the caller results in the death of 
;               the PICKFILE widget application. 
; 
;       FTYPE:  The structure type to be searched.  
; 
;       TITLE:  A scalar string to be used for the window title.  If it is 
;               not specified, the default title is "Select Measurement" 
; 
;       NOCONFIRM: Return immediately upon selection of a file.  The default 
;               behavior is to display the selection and then return the 
;               file when the user uses the "ok" button. 
;
;	MULTI: 	When set, several strings can be selected
;
;       POSI:   Position of every selected element 
;
;	COMMENTS: To be printed as text info within widget
; 
; OUTPUTS: 
;       PICKMEAS returns a string that contains the name of the selected 
;       measurement. If no one is selected, PICKMEAS returns a null string. 
; 
; COMMON BLOCKS: 
;       PICKMEAS: COMMON block that maintains state for the widget. 
; 
; SIDE EFFECTS: 
;       This function initiates the XMANAGER if it is not already running. 
; 
; RESTRICTIONS: 
;       Only one instance of the PICKMEAS widget can be running at one time. 
; 
; PROCEDURE: 
;       Create and register the widget and then exit, returning the measurement 
;       name that was picked. 
; 
; EXAMPLE: 
; 
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               adm
;               phtaap
;               phtcla
;               phterd
;               phtscp
;               phtsmc
;               phtspd
;               phtsrd
;       PIA Procedure(s):
;               decode_admn
;
; MODIFICATION HISTORY: 
;       Written by: Carlos GABRIEL
;	Oct 95 Version 3.5
;       Modified by J. Acosta to include new buffer type (OTHER) - Jan. 96
;       Modified by J. Acosta to select within a set of indices  - Mar. 96
;	Keyword added to print a text info (JA)			   Jan. 98
;	V7.0  
;	/Modal use according to IDL >= 5.0 (CG)
;	V7.3
;- 
; 
 
;------------------------------------------------------------------------------ 
;       procedure erd2map_Pickmeas_ev 
;------------------------------------------------------------------------------ 
; This procedure processes the events being sent by the xMANAGER,No_BLOCK=1. 
;------------------------------------------------------------------------------ 
PRO erd2map_Pickmeas_ev, event 
 

COMMON pickm_cmn, themeas, select, files, list, multflg, multsel, pos

	WIDGET_CONTROL, event.id, GET_UVALUE=uvalue
		
	CASE uvalue OF 
 
	'cancel': BEGIN 
      		themeas = "" 
		multsel = ""
		pos = 0
      		WIDGET_CONTROL, event.top, /DESTROY 
    	END 
 

  	'ALL': BEGIN 
		themeas = files 
		multsel = themeas 
                pos = indgen(N_ELEMENTS(themeas))

		decode_admn, multsel, measures
      		WIDGET_CONTROL, select, set_VALUE = measures
    	END 
 

  	'files': BEGIN 
		themeas = files(event.index) 
                position = event.index 
		IF multflg THEN BEGIN
		  IF multsel(0) EQ '' THEN BEGIN 
		        multsel = themeas 
                        pos = position
		  ENDIF ELSE BEGIN
			multsel = [multsel, themeas]
                        pos = [pos,position]
                  ENDELSE
                ENDIF ELSE BEGIN
                  multsel = themeas
                  pos = position
                ENDELSE

		decode_admn, multsel, measures
      		WIDGET_CONTROL, select, set_VALUE = measures
    	END 
 
  	'ok': BEGIN
              print,'enter ok'
              GOTO, checkfile 
        END
 
  	'help': PIA_DISP_PRINT_FILE, "", $ 
                GROUP = event.top, /MODAL, $ 
                TITLE = "Measurements Selection Help", $ 
                WIDTH = 50, $ 
                HEIGHT = 12, $ 
                TEXT = [" This file selection widget lets you pick a sky", $ 
                        "measurement.  Files are shown on the right.  You can", $ 
                        "select a file by clicking on it with the mouse.", $ 
                        "Pressing the 'OK' button will accept the choice", $ 
                        "and the Cancel button will not."] 
 
ENDCASE 
RETURN 

  checkfile:
    print,'meas=[',themeas,']', strlen(themeas)
    IF strlen(themeas) EQ 0 THEN BEGIN
       XDISPLAYFILE, '', height=3, $
        Text= 'Please select a Measurement'
       RETURN
    ENDIF

    WIDGET_CONTROL, event.top, /DESTROY
    RETURN 
 
 
END ;============= end of Pickmeas event handling routine task ================ 
 
 
 
;------------------------------------------------------------------------------ 
;       procedure erd2map_Pickmeas 
;------------------------------------------------------------------------------ 
;  This is the actual routine that creates the widget and registers it with the 
;  Xmanager.  It also determines the operating system and sets the specific 
;  file designators for that operating system. 
;------------------------------------------------------------------------------ 
Function erd2map_Pickmeas, GROUP = GROUP, FTYPE = FTYPE, DTYPE = DTYPE, INP_FILES=INP_FILES, $ 
              	   NOCONFIRM = NOCONFIRM, TITLE = TITLE, MULTI = MULTI, $
              	   SEL_IND=SEL_IND, POSI = posi, COMMENTS= comments
 
@adm
@phterd
@phtsrd
@phtscp
@phtspd
@phtdyncal
@phtdrift
@phtdynresp
@phtaap
@phtaar
@phtcla
@phtsmc

COMMON pickm_cmn, themeas, select, files, list, multflg, multsel, pos
COMMON pickmeas_results, pickmeas_res

COMMON pickmeas_otherstr, otherstr
 
themeas = ''
multsel = ''

IF(XRegistered("erd2map_Pickmeas")) THEN RETURN, themeas 
 
existflag = 0 
 
IF (KEYWORD_SET(NOCONFIRM))     THEN auto_exit = 1      ELSE auto_exit = 0 
IF (N_ELEMENTS(FILE) EQ 0)      THEN FILE = "" 
 

 
        ; Widget pickfile for the rest of IDL 
 
        IF (KEYWORD_SET(DTYPE))     THEN dettyp = DTYPE ELSE dettyp = "ALL" 

        IF (KEYWORD_SET(FTYPE))     THEN ftyp = FTYPE ELSE ftyp = "ERD" 

        IF (KEYWORD_SET(TITLE))      THEN TITLE = TITLE ELSE $  
			  TITLE = "Please Select a Measurement"               
        
        IF (KEYWORD_SET(MULTI))      THEN multflg = 1 ELSE multflg = 0
         
        IF (KEYWORD_SET(SEL_IND))    THEN presel  = 1 ELSE presel  = 0

	CASE ftyp of 
		'ERD': BEGIN
			if presel eq 0 then buffer = phterd $ 
			  else buffer = phterd(sel_ind)		
			dettyp = STRMID(dettyp,0,2)
                        help,dettyp
			IF STRMID(dettyp,1,1) EQ '#' OR $
			   STRMID(dettyp,1,1) EQ '%' THEN BEGIN
			 detty = STRMID(dettyp,0,1)
        		 indices=where(STRMID(buffer.admn,0,1) eq detty,count)
			ENDIF ELSE $  
        		indices=where(STRMID(buffer.admn,0,2) eq dettyp,count)  
                END
		'SRD': BEGIN
			if presel eq 0 then buffer = phtsrd $
			  else buffer = phtsrd(sel_ind)		
			dettyp = STRMID(dettyp,0,2)
			IF STRMID(dettyp,1,1) EQ '#' OR $
			   STRMID(dettyp,1,1) EQ '%' THEN BEGIN
			 detty = STRMID(dettyp,0,1)
        		 indices=where(STRMID(buffer.admn,0,1) eq detty,count)
			ENDIF ELSE $  
        		indices=where(STRMID(buffer.admn,0,2) eq dettyp,count)  
                END
		'DYNR': BEGIN
			if presel eq 0 then buffer = phtdynresp $
			  else buffer = phtdynresp(sel_ind)		
			dettyp = STRMID(dettyp,0,2)
			IF STRMID(dettyp,1,1) EQ '#' OR $
			   STRMID(dettyp,1,1) EQ '%' THEN BEGIN
			 detty = STRMID(dettyp,0,1)
        		 indices=where(STRMID(buffer.admn,0,1) eq detty,count)
			ENDIF ELSE $  
        		indices=where(STRMID(buffer.admn,0,2) eq dettyp,count)  
                END
                'SMC': BEGIN
			if presel eq 0 then buffer = phtsmc  $
			  else buffer = phtsmc(sel_ind)		
                        dettyp = STRMID(dettyp,0,2)
                        IF STRMID(dettyp,1,1) EQ '#' OR $
                           STRMID(dettyp,1,1) EQ '%' THEN BEGIN
                         detty = STRMID(dettyp,0,1)
                         indices=where(STRMID(buffer.admn,0,1) eq detty,count)
                        ENDIF ELSE $
                        indices=where(STRMID(buffer.admn,0,2) eq dettyp,count)
                END
		'SCP': BEGIN   
			if presel eq 0 then buffer = phtscp  $		
			  else buffer = phtscp(sel_ind)		
			dettyp = STRMID(dettyp,0,2)
			IF STRMID(dettyp,1,1) EQ '#' OR $
			   STRMID(dettyp,1,1) EQ '%' THEN BEGIN
			 detty = STRMID(dettyp,0,1)
        		 indices=where(STRMID(buffer.admn,0,1) eq detty,count)
			ENDIF ELSE $  
        		indices=where(STRMID(buffer.admn,0,2) eq dettyp,count)  
                END
		'DRIFT': BEGIN   
			if presel eq 0 then buffer = phtdrift  $		
			  else buffer = phtdrift(sel_ind)		
			dettyp = STRMID(dettyp,0,2)
			IF STRMID(dettyp,1,1) EQ '#' OR $
			   STRMID(dettyp,1,1) EQ '%' THEN BEGIN
			 detty = STRMID(dettyp,0,1)
        		 indices=where(STRMID(buffer.admn,0,1) eq detty,count)
			ENDIF ELSE $  
        		indices=where(STRMID(buffer.admn,0,2) eq dettyp,count)  
                END
		'DYNCAL': BEGIN   
			if presel eq 0 then buffer = phtdyncal  $		
			  else buffer = phtdyncal(sel_ind)		
			dettyp = STRMID(dettyp,0,2)
			IF STRMID(dettyp,1,1) EQ '#' OR $
			   STRMID(dettyp,1,1) EQ '%' THEN BEGIN
			 detty = STRMID(dettyp,0,1)
        		 indices=where(STRMID(buffer.admn,0,1) eq detty,count)
			ENDIF ELSE $  
        		indices=where(STRMID(buffer.admn,0,2) eq dettyp,count)  
                END
		'SPD': BEGIN   
			if presel eq 0 then buffer = phtspd  $		
			  else buffer = phtspd(sel_ind)		
			IF dettyp EQ 'SL' or dettyp EQ 'SS' THEN BEGIN
			 dettyp = STRMID(dettyp,0,1)
        		 indices=where(STRMID(buffer.admn,0,1) eq dettyp,count) 
			ENDIF ELSE BEGIN
			 dettyp = STRMID(dettyp,0,2)
        		 indices=where(STRMID(buffer.admn,0,2) eq dettyp,count) 
			ENDELSE
                END
		'AAP': BEGIN   
			if presel eq 0 then buffer = phtaap  $		
			  else buffer = phtaap(sel_ind)		
                        IF dettyp EQ 'SL' or dettyp EQ 'SS' THEN BEGIN
                         dettyp = STRMID(dettyp,0,1)
                         indices=where(STRMID(buffer.admn,0,1) eq dettyp,count) 
                        ENDIF ELSE BEGIN
                         dettyp = STRMID(dettyp,0,2)
                         indices=where(STRMID(buffer.admn,0,2) eq dettyp,count) 
                        ENDELSE
                END
		'AAR': BEGIN   
			if presel eq 0 then buffer = phtaar  $		
			  else buffer = phtaar(sel_ind)		
                        IF dettyp EQ 'SL' or dettyp EQ 'SS' THEN BEGIN
                         dettyp = STRMID(dettyp,0,1)
                         indices=where(STRMID(buffer.admn,0,1) eq dettyp,count) 
                        ENDIF ELSE BEGIN
                         dettyp = STRMID(dettyp,0,2)
                         indices=where(STRMID(buffer.admn,0,2) eq dettyp,count) 
                        ENDELSE
                END
		'CLA': BEGIN   
			if presel eq 0 then buffer = phtcla  $		
			  else buffer = phtcla(sel_ind)		
			dettyp = STRMID(dettyp,0,2)
        		indices=where(STRMID(buffer.admn,0,2) eq dettyp,count)  
                END
                'OTHER': BEGIN
			if presel eq 0 then buffer = otherstr $
			  else buffer = otherstr(sel_ind)		
                        dettyp = STRMID(dettyp,0,2)
                        indices = where(STRMID(buffer.admn,0,2) eq dettyp,count)
                END        
		ELSE: MESSAGE, /INFO, 'Unknown buffer type'
        ENDCASE

	
	
        IF KEYWORD_SET(INP_FILES) THEN $
          files = INP_FILES $
        ELSE BEGIN
          print,"count",count
	  IF count EQ 0 then files = '' ELSE files = buffer(indices).admn  
          if dettyp eq 'AL' AND N_ELEMENTS(buffer.admn) GT 1 then $
		  files = buffer(sel_ind).admn 
        ENDELSE

	IF not KEYWORD_SET(group) THEN group = !pia_top_wid
        Pickmeasbase =  WIDGET_BASE(TITLE = TITLE, /COLUMN, /MODAL, Group=group) 
        
        IF KEYWORD_SET(COMMENTS) THEN BEGIN 
          id_0 = WIDGET_BASE(pickmeasbase,/COLUMN)
          id_1 = WIDGET_LABEL(id_0,value='Comments')
          ys = 2*N_ELEMENTS(comments)
          id_1 = WIDGET_TEXT(id_0,value=COMMENTS,/scroll,ys=ys)   
        ENDIF  

        id_0 =	WIDGET_BASE(Pickmeasbase, /ROW) 
        id_1 =  WIDGET_LABEL(id_0, VALUE = "Data Type:") 
	id_1 =	WIDGET_TEXT(id_0, VALUE = ftyp)

        id_0 =  WIDGET_BASE(Pickmeasbase, /ROW) 
        id_1 =  WIDGET_LABEL(id_0, VALUE = "Detector:") 
        id_1 =  WIDGET_TEXT(id_0, VALUE = dettyp, XS = 10, UVALUE= 'dtyp') 

        id_0 =  WIDGET_BASE(Pickmeasbase, /COLUMN) 
        id_1 =  WIDGET_LABEL(id_0, VALUE = "Measurements") 

	numero = N_ELEMENTS(files)
        print,'pick',files
	strlist = strarr(numero)
        FOR i=0,numero-1 DO BEGIN 
		decode_admn,files(i),strl
		strlist(i) = strl
	ENDFOR
        list =  WIDGET_LIST(id_0, VALUE = strlist, YSIZE = 8, $ 
                        UVALUE = 'files') 

        id_0 =  WIDGET_LABEL(Pickmeasbase, VALUE = "Selection:") 
	IF multflg THEN $ 
         select =  WIDGET_TEXT(Pickmeasbase, VALUE = themeas, XS = 20, YS=4, $
			/scroll) ELSE $ 
         select =  WIDGET_TEXT(Pickmeasbase, VALUE = themeas, XS = 20)  

	IF KEYWORD_SET(multi) THEN BEGIN
         id_0 =  WIDGET_BASE(Pickmeasbase, SPACE = 15, /ROW) 
         id_1 =  WIDGET_BUTTON(id_0, VALUE = "  Select all  ", UVALUE = 'ALL')
        ENDIF 

        id_0 =  WIDGET_BASE(Pickmeasbase, SPACE = 15, /ROW) 
        id_1 =  WIDGET_BUTTON(id_0, VALUE = "   Ok   ", UVALUE = 'ok') 
        id_1 =  WIDGET_BUTTON(id_0, VALUE = " Cancel ", UVALUE = 'cancel') 
        id_1 =  WIDGET_BUTTON(id_0, VALUE = "  Help  ", UVALUE = 'help') 
 

        WIDGET_CONTROL, Pickmeasbase, /REALIZE 
 
        xMANAGER,No_BLOCK=1, "erd2map_Pickmeas", Pickmeasbase, $ 
             EVENT_HANDLER = "erd2map_Pickmeas_ev", GROUP_LEADER = GROUP

        FOR I=0,N_ELEMENTS(multsel)-1 DO BEGIN
          ind = WHERE(files EQ multsel(i),n)
          IF n GT 1 THEN position=WHERE(ind EQ pos(i)) ELSE position=0
          pos(i) = position(0)
        ENDFOR
 
        pickmeas_res = {names:multsel, position:pos}
        posi = pos
RETURN, multsel 
 
END ;====================== end of Pickfile routine =========================== 
 
