pro expshoot,beta,tau,signalprev,signalprevshoot,sillumprev,sillum,t,$
signal,signalslow,signalshoot
;
; This is the varient EXPSHOOT_ALTERNATIVEEMPIRICAL.PRO
;
; calculates signal response SIGNAL (real array or real single number)
; to set of illuminations or single illumination SILLUM, at a set of times or
; single time T since an illumination change from SILLUMPREV to SILLUM at time
; zero. SIGNALPREV is input as the best estimate of signal response
; of the detector at time zero minus a small fraction of the read interval
; (ie at time 0 but before the flux step). T represents the time at the end of
; each elementary read interval.
;
; input:  BETA detector model constant
;         BETAEXP    exponent of illumination in flux dependence of BETA (fixed
;                    by external analysis o FCS - used only by EXPSHOOT)
;         BETACOEFF  coefficient of flux in flux dependence of BETA (fixed by 
;                    external analysis of FCS - used only by EXPSHOOT)
;         TAU (VOLTS) detector model constant
;         TAUOFFSET  DC offset in tau (fixed by external analysis o FCS -
;                    used only by EXPSHOOT)
;         GAMMAEXP   (common MOREPAR) detector model constant. For GAMMAEXP=1
;                    the Foukes function is exactly reproduced. For
;                    GAMMAEXP < 1 the detector changes more rapidly at first,
;                    thus mimicing the observed tendency for short response
;                    timescales immediately after the flux step, during
;                    (eg) P32 chopper plateaus of ca. 1/2 second, and long
;                    timescales thereafter; (eg) on FCS plateaus of ca. 32s.
;                    scales thereafter
;         BETASHOOT,BETACOEFFSHOOT,BETAEXPSHOOT,
;         TAUOFFSETSHOOT,TAUSHOOT,GAMMAEXPSHOOT (common MOREPAR): 
;                    parameters for shooting function.
;         ZEROBIAS   (common MOREPAR) illumination level about which noise
;                    fluctuations
;                    should have a symetrical relaxation times for positive
;                    and negative fluctuations. This is normally the estimated
;                    background level on the map - the minimum real sky
;                    illumination encountered
;         SIGNALPREV (single number in V/s): long detector response at T(0)=0
;                     immediately prior to the initial flux step from
;                     SILLUMPREV to SILLUM(0).
;         SIGNALPREVSHOOT  (single number in V/s):
;                     shooting response prior to flux step
;         SILLUMPREV  (single number in V/s): illumination immediately prior
;                     to illumination step at T(0)=0. Must be positive.
;         SILLUM      (array in V/s): array of new
;                      illuminations at time T. There are no restrictions
;                      on the values SILLUM can be take, so long as they are
;                      positive. Thus, SILLUM(0) need not differ from
;                      SILLUMPREV, and any adjacent elements can be identical.
;         T           (array in s) time since flux step
;                      at time 0. T refers to the time at which the detector
;                      is actually read. Note this is a non-PIA standard, and
;                      follows the time assignment in the SIGNALCREATE.PRO
;                      routine - ie T(0) should actually be the first read
;                      interval.
;
; output: SIGNALSLOW  (array in V/s) slow detector response at times in T 
;                      since initial step from SILLUMPREV to SILLUM(0).
;         SIGNALSHOOT (array in V/s) short detector response at times in T 
;                      since initial step from SILLUMPREV to SILLUM(0).
;         SIGNAL      (array in V/s) total detector response at times in T 
;                      since initial step from SILLUMPREV to SILLUM(0).
;                      SIGNAL=SIGNAL
;
;
@common_morepar
;common morepar,gammaexp,zerobias,betashoot,taushoot,gammaexpshoot,$
;betacoeffshoot,betaexpshoot,tauoffsetshoot,tauoffset,betaexp,betacoeff
;
;save,beta,tau,signalprev,signalprevshoot,sillumprev,sillum,t,$
;file='expshoot.xdr'
;
n=n_elements(t)
if(n_elements(sillum) ne n)then $
print,'EXPSHOOT warning: n_elements of sillum,t = ',n_elements(sillum),n
signal=dblarr(n)
signalslow=signal
signalshoot=signal
sf=signalprev ; slow response just prior to flux step at time zero
sfshoot=signalprevshoot ; shooting response just prior to flux step at zero
sillump=sillumprev ; illumination just prior to flux step at time zero
setshootbeta_model3,beta,sillum(0),betawk,betash
setshootbeta_model3,beta,sillump,betawkprev,betashprev
setshoottime_model3,tau,sillump,tlongprev,tshootprev
setshoottime_model3,tau,sillum(0),tlong,tshoot  
;
if(min(sillum) eq max(sillum) and n gt 1)then goto, c555 ; vector calculation
;
; point by point calculation (for arbitrary illumination sequences and for
; treatment of singularity
;
for i=0L,n-1 do begin ; i=0 corresponds to first signal
;
if(sillum(i) ne sillump or i eq 0)then begin ; we have changed illumination
;
if(i gt 0)then begin
sf=signalslow(i-1)
sfshoot=signalshoot(i-1)
setshootbeta_model3,beta,sillum(i),betawk,betash
setshoottime_model3,tau,sillum(i),tlong,tshoot  
endif ; i gt 0
setjump_model3,sillum(i),sillump,sf,sfshoot,tlong,tlongprev,$
betawk,betawkprev,betash,betashprev,tshoot,tshootprev,s0,s0shoot
sillump=sillum(i) ; update
betawkprev=betawk
betashprev=betash
tlongprev=tlong
tshootprev=tshoot
;b=s0-sillum(i)
;b=s0-2.0*sillum(i)
b=s0-sillum(i)+betash*sillum(i)
;bshoot=s0shoot-sillum(i)
bshoot=s0shoot-betash*sillum(i)
e=1.0/tlong
eshoot=1.0/tshoot
;
ii=i-1 ; reference read for when illumination actually changed
if(i gt 0)then tchange=t(ii) ; assumes time zero for first flux change
if(i eq 0)then tchange=0.0
;
endif ; sillum(i) ne sillump
;
dum=t(i)-tchange
ee=dum*e
eeshoot=dum*eshoot
;
;signalslow(i)=sillum(i)+b*exp(-ee)
;signalslow(i)=2.0*sillum(i)+b*exp(-ee)
signalslow(i)=(1.0-betash)*sillum(i)+b*exp(-ee)
;signalshoot(i)=s0shoot*exp(-eeshoot)
;signalshoot(i)=sillum(i)+bshoot*exp(-eeshoot)
signalshoot(i)=betash*sillum(i)+bshoot*exp(-eeshoot)
;signal(i)=signalshoot(i)+signalslow(i)
;signal(i)=signalslow(i)-signalshoot(i)
signal(i)=signalslow(i)+signalshoot(i)
;
endfor ; i
;
goto, c999
;
c555: ; VECTOR PROCESSING STARTS (no loops)
;
setjump_model3,sillum(0),sillump,sf,sfshoot,tlong,tlongprev,$
betawk,betawkprev,betash,betashprev,tshoot,tshootprev,s0,s0shoot
;b=s0-sillum(0) ; scalar constants for plateau
;b=s0-2.0*sillum(0)
b=s0-sillum(0)+betash*sillum(0)
;bshoot=s0shoot-sillum(0)
bshoot=s0shoot-betash*sillum(0)
e=1.0/tlong
eshoot=1.0/tshoot
;
tchange=0.0
dum=t-tchange ; vector
ee=dum*e
eeshoot=dum*eshoot
;
;signalslow=sillum+b*exp(-ee)
;signalslow=2.0*sillum+b*exp(-ee)
signalslow=(1.0-betash)*sillum+b*exp(-ee)
;signalshoot=s0shoot*exp(-eeshoot)
;signalshoot=sillum+bshoot*exp(-eeshoot)
signalshoot=betash*sillum+bshoot*exp(-eeshoot)
;signal=signalshoot+signalslow
;signal=signalslow-signalshoot
signal=signalslow+signalshoot
;
c999:
;
return
end






