pro expshoot,beta,tau,signalprev,signalprevshoot,sillumprev,sillum,t,$
signal,signalslow,signalshoot
;
; calculates signal response SIGNAL (real array or real single number)
; to set of illuminations or single illumination SILLUM, at a set of times or
; single time T since an illumination change from SILLUMPREV to SILLUM at time
; zero. SIGNALPREV is input as the best estimate of signal response
; of the detector at time zero minus a small fraction of the read interval
; (ie at time 0 but before the flux step). T represents the time at the end of
; each elementary read interval.
;
; input:  BETA detector model constant
;         BETAEXP    exponent of illumination in flux dependence of BETA (fixed
;                    by external analysis o FCS - used only by EXPSHOOT)
;         BETACOEFF  coefficient of flux in flux dependence of BETA (fixed by 
;                    external analysis of FCS - used only by EXPSHOOT)
;         TAU (VOLTS) detector model constant
;         TAUOFFSET  DC offset in tau (fixed by external analysis o FCS -
;                    used only by EXPSHOOT)
;         GAMMAEXP   (common MOREPAR) detector model constant. For GAMMAEXP=1
;                    the Foukes function is exactly reproduced. For
;                    GAMMAEXP < 1 the detector changes more rapidly at first,
;                    thus mimicing the observed tendency for short response
;                    timescales immediately after the flux step, during
;                    (eg) P32 chopper plateaus of ca. 1/2 second, and long
;                    timescales thereafter; (eg) on FCS plateaus of ca. 32s.
;                    scales thereafter
;         BETASHOOT,BETACOEFFSHOOT,BETAEXPSHOOT,
;         TAUOFFSETSHOOT,TAUSHOOT,GAMMAEXPSHOOT (common MOREPAR): 
;                    parameters for shooting function.
;         ZEROBIAS   (common MOREPAR) illumination level about which noise
;                    fluctuations
;                    should have a symetrical relaxation times for positive
;                    and negative fluctuations. This is normally the estimated
;                    background level on the map - the minimum real sky
;                    illumination encountered
;         SIGNALPREV (single number in V/s): long detector response at T(0)=0
;                     immediately prior to the initial flux step from
;                     SILLUMPREV to SILLUM(0).
;         SIGNALPREVSHOOT  (single number in V/s):
;                     shooting response prior to flux step
;         SILLUMPREV  (single number in V/s): illumination immediately prior
;                     to illumination step at T(0)=0. Must be positive.
;         SILLUM      (array in V/s): array of new
;                      illuminations at time T. There are no restrictions
;                      on the values SILLUM can be take, so long as they are
;                      positive. Thus, SILLUM(0) need not differ from
;                      SILLUMPREV, and any adjacent elements can be identical.
;         T           (array in s) time since flux step
;                      at time 0. T refers to the time at which the detector
;                      is actually read. Note this is a non-PIA standard, and
;                      follows the time assignment in the SIGNALCREATE.PRO
;                      routine - ie T(0) should actually be the first read
;                      interval.
;
; output: SIGNALSLOW  (array in V/s) slow detector response at times in T 
;                      since initial step from SILLUMPREV to SILLUM(0).
;         SIGNALSHOOT (array in V/s) short detector response at times in T 
;                      since initial step from SILLUMPREV to SILLUM(0).
;         SIGNAL      (array in V/s) total detector response at times in T 
;                      since initial step from SILLUMPREV to SILLUM(0).
;                      SIGNAL=SIGNAL
;
;print,'EXPSHOOT INPUT PARAMETERS beta,tau,signalprev,sillumprev'
;print,beta,tau,signalprev,sillumprev
;help,sillum
;print,sillum(0:10)
;help,t
;print,t(0:10)
;
@common_morepar
;common morepar,gammaexp,zerobias,betashoot,taushoot,gammaexpshoot,$
;betacoeffshoot,betaexpshoot,tauoffsetshoot,tauoffset,betaexp,betacoeff
;
n=n_elements(t)
if(n_elements(sillum) ne n)then $
print,'EXPSHOOT warning: n_elements of sillum,t = ',n_elements(sillum),n
signal=dblarr(n)
signalslow=signal
signalshoot=signal
sf=signalprev ; slow response just prior to flux step at time zero
sfshoot=signalprevshoot ; shooting response just prior to flux step at zero
sillump=sillumprev ; illumination just prior to flux step at time zero
setshootbeta_model3,beta,sillum(0),betawk,betash
;
if(min(sillum) eq max(sillum) and n gt 1)then goto, c555 ; vector calculation
;
; point by point calculation (for arbitrary illumination sequences and for
; treatment of singularity
;
for i=0L,n-1 do begin ; i=0 corresponds to first signal
;
if(sillum(i) ne sillump or i eq 0)then begin ; we have changed illumination
;
if(i gt 0)then begin
;betawk=beta+betacoeff*(sillum(i)^(betaexp))
setshootbeta_model3,beta,sillum(i),betawk,betash
sf=signalslow(i-1)
sfshoot=signalshoot(i-1)
endif ; i gt 0
dum=sillum(i)-sillump
s0=betawk*dum+sf
;betash=betashoot+betacoeffshoot*(sillum(i)^(betaexpshoot))
s0shoot=betash*dum+sfshoot
sillump=sillum(i) ; update
b=s0-sillum(i)
;
;em=abs(sillum(i)-zerobias)+zerobias
;eg=em^gammaexp
;egs=em^gammaexpshoot
;e=1.0/(tauoffset+(tau/eg)) ; this is an inverse timescale
;eshoot=1.0/(tauoffsetshoot+(taushoot/egs))
;
setshoottime_model3,tau,sillum(i),tlong,tshoot ; finds long and shoot time 
e=1.0/tlong                                    ; constants for sillum(i)
eshoot=1.0/tshoot
;
ii=i-1 ; reference read for when illumination actually changed
if(i gt 0)then tchange=t(ii) ; assumes time zero for first flux change
if(i eq 0)then tchange=0.0
;
endif ; sillum(i) ne sillump
;
dum=t(i)-tchange
ee=dum*e
eeshoot=dum*eshoot
;
signalslow(i)=sillum(i)+b*exp(-ee)
signalshoot(i)=s0shoot*exp(-eeshoot)
signal(i)=signalshoot(i)+signalslow(i)
;
endfor ; i
;
goto, c999
;
c555: ; VECTOR PROCESSING STARTS (no loops)
;
dum=sillum(0)-sillump
setshootbeta_model3,beta,sillum(0),betawk,betash
;betawk=beta+betacoeff*(sillum(0)^(betaexp))
s0=betawk*dum+sf
;betash=betashoot+betacoeffshoot*(sillum(0)^(betaexpshoot))
s0shoot=betash*dum+sfshoot
b=s0-sillum(0) ; scalar constants for plateau
;
;em=abs(sillum(0)-zerobias)+zerobias
;eg=(em^gammaexp)
;egs=(em^gammaexpshoot)
;e=1.0/(tauoffset+(tau/eg)) ; this is an inverse timescale
;eshoot=1.0/(tauoffsetshoot+(taushoot/egs))
;
setshoottime_model3,tau,sillum(0),tlong,tshoot ; finds long and shoot time 
e=1.0/tlong                                    ; constants for sillum(i)
eshoot=1.0/tshoot
;
tchange=0.0
dum=t-tchange ; vector
ee=dum*e
eeshoot=dum*eshoot
;
signalslow=sillum+b*exp(-ee)
signalshoot=s0shoot*exp(-eeshoot)
signal=signalshoot+signalslow
;
c999:
;
return
end






