pro findplateau,fpoint,keypoint,vig,nplat,nplateau,from,to,$
vigp,keyp,finep,pident,iverbose
;
; find pointing plateaus from FPOINT and KEYPOINT arrays. In general this 
; program has to be run separately for different pixels. There are several 
; possibilities, since the chopper motion is not synchronised with the
; attainment of a fine pointing. The following treatment gives a general
; treatment which should also be robust in the case that pointing jitter
; takes the pointing in and out of a nominal fine pointing position
; (eg if the pointing precision is set too fine in the calculation of the 
; fine pointing flag in the ERDTOMAP routine POINT. ii is the identifier of a 
; sample point (including non-transmitted samples) prior to i: 
;
; The possibilities are as follows:
;
; (i)   fpoint(i) = fpoint(ii) = 0 Midslew. Just have one point for the plateau
; (ii)  fpoint(i) = 0 fpoint(ii) = 1 Start of slew. Current plateau is
;       self contained with one point. Previous plateau must be wound up with
;       ii the last point. ii need not coincide with the end of a chopper 
;       plateau
; (iii) fpoint(i)=1 fpoint(ii)=0 Start of fine pointing and start of pointing 
;       plateau (note the pointing plateau may only be one point long if
;       the start of the fine pointing happend to coincide with the
;       end of a chopper plateau.
; (iv)  fpoint(ii)=fpoint(i)=1 and keypoint(ii)=keypoint(i)
;       middle of pointing plateau
; (v)   fpoint(ii)=fpoint(i)=1 and keypoint(ii) NE keypoint(i)
;       end of pointing plateau.
;
; input
;         KEYPOINT   (1D array INTEGER). For each time sample 
;                    (including data taken on board but not transmitted 
;                     in fast readout modes) this contains the integer key K
;                     to the nearest direction on the P32 natural grid.
;                     K= iy + iz * nygrid,  where iy,iz are the 2D
;                     pointing positions and the grid has dimensions NYGRID,
;                     NZGRID, though this information is not needed, and not
;                     used by this routine.
;         FPOINT     (1D array INTEGER) For each time sample 
;                    (including data taken on board but not transmitted 
;                     in fast readout modes) this contains 1 if the 
;                     on target flag (OTF) was set (as defined by ERDTOMAP)
;                     and zero if the OTF is not set - ie when the data
;                     is taken on the fly.
;         VIG        (1D floating array) This is set to the attenuation
;                     due to vignetting (VIG <= 1) as a function of chopper
;                     position in the sense detector signal = vig * sky
;                     As the same sky direction is viewed by the same pixel
;                     in different chopper offsets, the detector pixel
;                     illumination can vary even if the same sky direction is
;                     viewed.
;         IVERBOSE    = 1 for diagnostic prints
;
; output
;         NPLAT    - number of plateaus
;         NPLATEAU - an integer array (NPLAT) containing number of samples
;                    on each plateau 
;         TO       - (NPLAT) identifier of first point on a plateau
;         FROM     - (NPLAT) identifier of last point on a plateau
;         VIGP     - (NPLAT) value of vigneting factor (normally <=1) on a 
;                     plateau
;         KEYP     - (NPLAT) value of natural grid identifier on a plateau
;         FINEP    - (NPLAT) = 1 for fine pointing plateau = 0 for on the fly
;         PIDENT   - (integer array of dimension numberof elementary points)
;                     containing plateau identifier (not to be confused with 
;                     the identifier of the P32 fine pointing)
;
npt=n_elements(keypoint)
i=0L
ninplateau=1L ; first time sample must be first point of first pointing plateau
istart=0L ; set to 0 on first pointing plateau
ifrom=0L
c100:
;
i=i+1 ; first point to be tested is i=1 (second time sample)
ii=i-1
;
if(i ge npt)then begin
if(ninplateau gt 0)then begin
nplateau=[nplateau,ninplateau]
ito=npt-1
from=[from,ifrom]
to=[to,ito]
vp=vig(i-1)
kp=keypoint(i-1)
fp=fpoint(i-1)
vigp=[vigp,vp]
keyp=[keyp,kp]
finep=[finep,fp]
endif ; ninplateau gt 0
goto,c200
endif ; i ge npt
;
if(fpoint(i) eq 1 and fpoint(ii) eq 1 and $
keypoint(ii) eq keypoint(i))then begin 
; we are on middle of pointing plateau
if(iverbose eq 1)then $
print,'middle of pointing plateau within s/c fine pointing: i = ',i
ninplateau=ninplateau+1
goto, c100
endif ; middle of pointing plateau
;
if(fpoint(i) eq 1 and fpoint(ii) eq 1 and $
keypoint(ii) ne keypoint(i))then begin 
; we have started the next pointing plateau within a fine pointing
if(iverbose eq 1)then $
print,'start of next pointing plateau within s/c fine pointing: i = ',i
ito=i-1 ; last of previous plateau
if(istart eq 0)then begin 
nplateau=[ninplateau]
from=[ifrom]
to=[ito]
vp=vig(i-1)
kp=keypoint(i-1)
fp=fpoint(i-1)
vigp=[vp]
keyp=[kp]
finep=[fp]
if(iverbose eq 1)then $
help,nplateau
if(iverbose eq 1)then $
print,'first pointing plateau has ninplateau = ',ninplateau
endif ; istart eq 0
if(istart gt 0)then begin
nplateau=[nplateau,ninplateau]
from=[from,ifrom]
to=[to,ito]
vp=vig(i-1)
kp=keypoint(i-1)
fp=fpoint(i-1)
vigp=[vigp,vp]
keyp=[keyp,kp]
finep=[finep,fp]
endif ; istart gt 0
istart=1
ninplateau=1
ifrom=i
goto, c100
endif ; we have started the next pointing plateau within s/c fine pointing
;
if(fpoint(i) eq 0 and fpoint(ii) eq 0)then begin ; we are in mid-slew
if(iverbose eq 1)then $
print,'mid slew: i =  ',i
;help,nplateau
ito=i-1 ; last of previous pointing plateau (single point in slew)
if(istart eq 0)then begin
from=[ifrom]
to=[ito]
nplateau=[ninplateau]
vp=vig(i-1)
kp=keypoint(i-1)
fp=fpoint(i-1)
vigp=[vp]
keyp=[kp]
finep=[fp]
; sets up first pointing plateau, found on a slew
;help,nplateau
if(iverbose eq 1)then $
print,'first pointing plateau has ninplateau = ',ninplateau
endif ; istart eq 0
if(istart gt 0)then begin
nplateau=[nplateau,ninplateau]
from=[from,ifrom]
to=[to,ito]
vp=vig(i-1)
kp=keypoint(i-1)
fp=fpoint(i-1)
vigp=[vigp,vp]
keyp=[keyp,kp]
finep=[finep,fp]
endif ; istart gt 0
; sets up previous pointing plateau of slew
;print,'mid slew: ' 
;help,nplateau
ninplateau=1 ; for current pointing plateau in slew
istart=1
ifrom=i
goto, c100
endif ; we are in mid-slew
;
if(fpoint(i) eq 0 and fpoint(ii) eq 1)then begin 
; we are on first point of slew
if(iverbose eq 1)then $
print,'first point of slew: i = ',i
;help,nplateau
ito=i-1
if(istart eq 0)then begin
nplateau=[ninplateau]
from=[ifrom]
to=[ito]
vp=vig(i-1)
kp=keypoint(i-1)
fp=fpoint(i-1)
vigp=[vp]
keyp=[kp]
finep=[fp]
; sets up first pointing plateau, found as the first and last point of a slew
;help,nplateau
if(iverbose eq 1)then $
print,'first pointing plateau has ninplateau = ',ninplateau
endif ; istart eq 0
if(istart gt 0)then begin
nplateau=[nplateau,ninplateau] 
; sets up previous pointing plateau = last of slew
from=[from,ifrom]
to=[to,ito]
vp=vig(i-1)
kp=keypoint(i-1)
fp=fpoint(i-1)
vigp=[vigp,vp]
keyp=[keyp,kp]
finep=[finep,fp]
endif ; istart gt 0
ninplateau=1 ; for current pointing plateau in slew
istart=1
ifrom=i
goto, c100
endif ; we are on first point of slew
;
if(fpoint(i) eq 1 and fpoint(ii) eq 0)then begin 
; we are on first point of spacecraft fine pointing
if(iverbose eq 1)then $
print,'first point of s/c fine pointing: i = ',i
;help,nplateau
ito=i-1
if(istart eq 0)then begin
nplateau=[ninplateau]
from=[ifrom]
to=[ito]
vp=vig(i-1)
kp=keypoint(i-1)
fp=fpoint(i-1)
vigp=[vp]
keyp=[kp]
finep=[fp]
; sets up first pointing plateau, found as the first and last point of a s/c
; fine pointing
;help,nplateau
if(iverbose eq 1)then $
print,'first pointing plateau has ninplateau = ',ninplateau
endif ; istart eq 0
if(istart gt 0)then begin
nplateau=[nplateau,ninplateau]
from=[from,ifrom]
to=[to,ito]
vp=vig(i-1)
kp=keypoint(i-1)
fp=fpoint(i-1)
vigp=[vigp,vp]
keyp=[keyp,kp]
finep=[finep,fp]
; sets up previous pointing plateau = last of slew
endif ; istart gt 0
ninplateau=1 ; for current pointing plateau on s/c fine pointing
istart=1
ifrom=i
goto,c100
endif ; we are on first point of spacecraft fine pointing
;
c200:
;
nplat=n_elements(nplateau)
;
print,'FINDPLATEAU: nplat = ',nplat ; number of plateaus
;
; set up PIDENT
;
pident=keypoint
pident(*)=-1L
;
for iplat=0L,nplat-1L do begin
;
pident(from(iplat):to(iplat))=iplat
;
endfor ; iplat=0,nplat-1
;
itest=where(pident eq -1,nitest)
if(nitest gt 0)then begin
print,'FINDPLATEAU: warning some data points '
print,'not assigned to a plateau'
endif ; nitest gt 0
;
;print,'nplateau = ',nplateau ; array containing number of points per plateau
;print,'from = ',from
;print,'to = ',to
;
;i1=4500
;i2=5300
;plot,i1+findgen(i2-i1+1),keypoint(i1:i2),yran=[-10,130],xst=1
;oplot,i1+findgen(i2-i1+1),fpoint(i1:i2)*10
;oplot,(from+to)/2.0,keyp,psym=1
;oplot,(from+to)/2.0,finep*10,psym=1
;
return
end

