; modified by RJT from July 1998 version of FOOTPR_MAT
;+
; NAME:
;	FOOTpr_mat
;
; PURPOSE:
;	Obtain original footprint matrix and reduced matrix to 
;	a certain pixel size resolution 
;
; CATEGORY:
;	PIA - extras
;
; CALLING SEQUENCE:
;	FOOT, detector, filter, apert, pixsize, fp, conv_fp, fpred, 
;                   caldir=cdir
;
;  EXAMPLE: IDL> footpr_mat, 'C1', 'C1_60', 0, [15.,23.], fp, cfp, fpr, $
;           IDL> caldir='/d12/vlk/cgabriel/pia_7.2/calg/'
;
; INPUTS:
;	detector	One of: "P1", "P2", "P3", "C1", "C2"
;
;	filter		One of:  P1_3.29 P1_3.60 P1_4.85 P1_7.30 P1_7.70 
;				 P1_10.0 P1_11.3 P1_12.8 P1_16.0 P1_11.5 
;				 P2_20 P2_25 
;				 P3_60 P3_100 
;				 C1_90b C1_50 C1_60 C1_70 C1_100 C1_105 C1_90
;				 C2_160 C2_200 C2_180 C2_135 C2_120
;	
;	apert		Aperture position (as in ERD files)
;
;	pixsize		Map pixel size - Two dimensional, in arcsec
;			Example: [15.,23.] 
; INPUT KEYWORDS:
;	caldir		Directory containing the footprint FITS files. If
;			not given, actual directory is searched for.
;
; OUTPUTS:
;	fp		Footprint matrix with full resolution, as in FITSfile
;	conv_fp		Interpolated footprint matrix to the pixsize given
;	fpred		Reduced footprint matrix to same dimensions as image
;
; PROCEDURES USED:
;	PIA Procedure(s):
;		phtfilter_init
;
; COMMON BLOCKS:
;	phtfilter		filter id's
;	phtfilter_prop		filter properties
;
; MODIFICATION HISTORY:
;	Written by:		C. Gabriel, ESA/VILSPA		July 1998
;	corresponding to PIA V7.2
;-


PRO PHTFILTER_INIT


COMMON PHTFILTER, P1filters, P2filters, P3filters, C1filters, C2filters


ON_ERROR, 2

	p1filters = ['P1_3.29', 'P1_3.60', 'P1_4.85', 'P1_7.30', 'P1_7.70', $
			'P1_10.0', 'P1_11.3', 'P1_12.8', 'P1_16.0', 'P1_11.5']
	p2filters = ['P2_20', 'P2_25']
	p3filters = ['P3_60', 'P3_100']
	c1filters = ['C1_90b', 'C1_50', 'C1_60', 'C1_70', $
			'C1_100', 'C1_105', 'C1_90']
	c2filters = ['C2_160', 'C2_200', 'C2_180', 'C2_135','C2_120']

END


PRO foot, detector, filter, apert, pixsize, fp, conv_fp, fpred, $
	caldir=caldir

COMMON PHTFILTER, P1filters, P2filters, P3filters, C1filters, C2filters
;@phtfilter_prop

        IF N_PARAMS() LT 4 THEN BEGIN
            PRINT,$
 'Syntax: footpr_mat, det, filt, apert, pixs, fp, conv_fp, fpred, caldir=cdir'
            RETURN
        ENDIF
;
	IF not KEYWORD_SET(caldir) THEN caldir = '' 

	IF N_ELEMENTS(c1filters) EQ 0 THEN PHTFILTER_INIT

	filts = [p1filters, p2filters, p3filters, c1filters, c2filters]

		 
	CASE detector of
	
	'C1': BEGIN
		fxread, caldir + 'PC1FOOTP.FITS', fp, fp_hdr
		ind = WHERE(c1filters EQ filter, count)
		IF count EQ 0 THEN BEGIN
		   MESSAGE, /INFO, 'Filter not found, returning'
		   RETURN
		ENDIF
		fp = REFORM(fp(ind,*,*))
		fp = fp / TOTAL(fp)
		fpsize = [7.5,7.5]
		steps = 35
	END
	
	'C2': BEGIN
		fxread, caldir + 'PC2FOOTP.FITS', fp, fp_hdr
		ind = WHERE(c2filters EQ filter, count)
		IF count EQ 0 THEN BEGIN
		   MESSAGE, /INFO, 'Filter not found, returning'
		   RETURN
		ENDIF
		fp = REFORM(fp(ind,*,*))
		fp = fp / TOTAL(fp)
		fpsize = [15.,15.]
		steps = 35
	END
	
	'P2': BEGIN
		aper = apert - 1
				
		fxread, caldir + 'PP2FOOTP.FITS', fp, fp_hdr
		ind = WHERE(p2filters EQ filter, count)
		IF count EQ 0 THEN BEGIN
		   MESSAGE, /INFO, 'Filter not found, returning'
		   RETURN
		ENDIF

		fp = REFORM(fp(ind,aper,*,*))
		fp = fp / TOTAL(fp)
		IF aper LT 6 OR aper GT 10 THEN $
		 fpsize = [3.,3.] ELSE fpsize = [2.,2.]
		steps = 87
	END

	'P3': BEGIN
		aper = apert - 1
				
		fxread, caldir + 'PP3FOOTP.FITS', fp, fp_hdr
		ind = WHERE(p3filters EQ filter, count)
		IF count EQ 0 THEN BEGIN
		   MESSAGE, /INFO, 'Filter not found, returning'
		   RETURN
		ENDIF

		fp = REFORM(fp(ind,aper,*,*))
		fp = fp / TOTAL(fp)
		IF aper LT 6 THEN $
		fpsize = [10.,10.] ELSE fpsize = [2.,2.]
		steps = 41
	END
	
	ELSE: BEGIN
	 MESSAGE, /INFO, 'Not yet implemented, returning
	 RETURN
	END
	
	ENDCASE
	
	pixs = pixsize
	fcts = pixs / fpsize

; added rjt - 0 appears to be bad pixel
nbad=0
scan=fltarr(steps)
for iscan=0,steps-1 do begin
scan(*)=fp(*,iscan)
newscan=scan
ii=where(scan eq 0,nii)
niibad=0
;
if(nii gt 0)then begin ; linear interpolation of bad pixels if surrounded by
; good pixels
;
for k=0,nii-1 do begin
i0=ii(k)
iplus=i0+1
iminus=i0-1
if(scan(iplus) gt 0. and scan(iminus) gt 0.)then begin
newscan(i0)=(scan(iplus)+scan(iminus))/2.0
nbad=nbad+1
niibad=niibad+1
endif ; scan(iplus) gt 0. and scan(iminus) gt 0.
endfor ; k
;
print,'FOOTPR_MAT: warning there are bad pixels in original scan ',iscan
print,'            nii,niibad = ',nii,niibad 
;
endif ; nii gt 0
;
endfor ; iscan
;
; convolve first to image pixel size
	unos = [round(fcts(0)),round(fcts(1))] > [1,1]
	kernel = fltarr(unos(0)+2,unos(1)+2)
	kernel(1:unos(0),1:unos(1)) = 1
	
	conv_fp = convol(fp,kernel, total(kernel))
; rjt   
        conv_fp=fp	
; get always an odd number of pixels for the reduced matrix
	nx = fix((steps)/fcts(0)/2)* 2 + 1
	ny = fix((steps)/fcts(1)/2)* 2 + 1

	tot = (steps-1) * fpsize
	totx = (nx-1) * pixs(0)
	toty = (ny-1) * pixs(1)	
	
	int_posx = (indgen(nx) * pixs(0) + (tot(0)-totx)/2) / fpsize(0)
	int_posy = (indgen(ny) * pixs(1) + (tot(1)-toty)/2) / fpsize(1)

	fpred = $
;rjt	 interpolate(conv_fp, int_posx, int_posy, /grid)
        interpolate(conv_fp, int_posx, int_posy, /grid, /cubic)
	fpred = fpred / total(fpred)	
		

	RETURN
	END	












