pro foukes,beta,tau,signalprev,sillumprev,sillum,t,signal
;
; NB Both FOUKES (general case where there may be arbitrary illumination
;    steps in SILLUM array and FOUKESFAST (where SILLUM is constant)
;    are called for imodel=1. They are both in this routine - the FOUKESFAST
;    is called if SILLUM is a constant, by going to c555. 
;    If changes are made to FOUKES, the same changes must be made to 
;    FOUKESFAST.
;
; calculates signal response SIGNAL (real array or real single number) 
; to set of illuminations or single illumination SILLUM, at a set of times or
; single time T since an illumination change from SILLUMPREV to SILLUM at time
; zero. SIGNALPREV is input as the best estimate of signal response
; of the detector at time zero minus a small fraction of the read interval
; (ie at time 0 but before the flux step). T represents the time at the end of
; each elementary read interval.
; 
; input:  BETA detector model constant
;         TAU (VOLTS) detector model constant
;         GAMMAEXP   (common MOREPAR) detector model constant. For GAMMAEXP=1
;                    the Foukes function is exactly reproduced. For
;                    GAMMAEXP < 1 the detector changes more rapidly at first, 
;                    thus mimicing the observed tendency for short response
;                    timescales immediately after the flux step, during 
;                    (eg) P32 chopper plateaus of ca. 1/2 second, and long 
;                    timescales thereafter; (eg) on FCS plateaus of ca. 32s.
;                    scales thereafter 
;         SIGNALPREV (single number in V/s): actual detector response at T(0)=0
;                     immediately prior to the initial flux step from 
;                     SILLUMPREV to SILLUM(0). Note this number is not normally
;                     actually measured due to the destructive read at T(0)=0 
;                     so has to be a model estimate. SIGNALPREV must be 
;                     positive.
;         SILLUMPREV  (single number in V/s): illumination immediately prior
;                     to illumination step at T(0)=0. Must be positive.
;         SILLUM      (array in V/s): array of new 
;                      illuminations at time T. There are no restrictions
;                      on the values SILLUM can be take, so long as they are
;                      positive. Thus, SILLUM(0) need not differ from 
;                      SILLUMPREV, and any adjacent elements can be identical.
;         T           (array in s) time since flux step
;                      at time 0. T refers to the time at which the detector
;                      is actually read. Note this is a non-PIA standard, and
;                      follows the time assignment in the SIGNALCREATE.PRO 
;                      routine - ie T(0) should actually be the first read 
;                      interval.
;
; output: SIGNAL      (array in V/s) detector response at times in T since 
;                      initial illumination step from SILLUMPREV to SILLUM(0).
;                       
;
;print,'FOUKES INPURT PARAMETERS beta,tau,signalprev,sillumprev'
;print,beta,tau,signalprev,sillumprev
;help,sillum
;print,sillum(0:10)
;help,t
;print,t(0:10)
;
@common_morepar
;common morepar,gammaexp
;
iwarn=0
n=n_elements(t)
if(n_elements(sillum) ne n)then $
print,'FOUKES warning: n_elements of sillum,t = ',n_elements(sillum),n 
;print,'n=',n
signal=dblarr(n)
sf=signalprev ; detector response just prior to flux step at time zero
sillump=sillumprev ; illumination just prior to flux step at time zero
;
if(min(sillum) eq max(sillum) and n gt 1)then goto, c555 ; vector calculation
;
; point by point calculation (for arbitrary illumination sequences and for
; treatment of singularity
;
c850:
;
for i=0L,n-1 do begin ; i=0 corresponds to first signal
;
if(sillum(i) ne sillump or i eq 0)then begin ; we have changed illumination
;
if(i gt 0)then sf=signal(i-1)
s0=beta*(sillum(i)-sillump)+sf
sillump=sillum(i) ; update
b=(s0-beta*sillum(i)) ; constants for each 'plateau'
a=(1.0-beta)*b*sillum(i)
c=sillum(i)-s0
d=beta*sillum(i)
e=(sillum(i)^gammaexp)/tau ; foukes-schubert modification type I 
;                            pure foukes-schubert for gammaexp=1            
;           
ii=i-1 ; reference read for when illumination actually changed
if(i gt 0)then tchange=t(ii) ; assumes time zero for first flux change
if(i eq 0)then tchange=0.0
;
endif ; sillum(i) ne sillump
;
;print,'i,ii',i,ii
ee=(t(i)-tchange)*e
;
;signal(i)=a/(b+c*exp(-ee))
;signal(i)=signal(i)+d
;
signal(i)=(a/(b+c*exp(-ee)))-b ; vector starts from value 0 at start
;                                of each ramp
;    
if(b lt 0.0 and c gt 0.0)then begin ; avoid worst effects of divergence to 
;                                     singularity. Note that if one hits the
;                                     b lt 0.0 and c gt 0.0 condition probably
;                                     foukes schubert is not giving an 
;                                     adequate description of the detector
;
signal(i)=abs(signal(i))
if((i-ii) eq 1)then signal(i)=0.0 ; ensure that ramp starts from s0
tcrit=(1.0/e)*alog(abs(c/b))      ; even when singularity encountered
;                                 ; otherwise fits to detector parameters fail
if(t(i) gt tcrit)then begin ; we've hit the singularity
iwarn=iwarn+1 ; to prevent too many error messages
;
;if(iwarn eq 1)then begin
;print,'FOUKES: warning - detector singularity met'
;print,'... tcrit = ',tcrit
;print,'... ii,i = ',ii,i
;endif ; iwarn eq 1
;
if(i-ii gt 1)then begin ; singularity hit after first data point of plateau
sadiff=abs(c-signal(i))
si2=c-sadiff
si1=signal(i-1)-s0
smid=(si1+si2)/2.0
if(smid gt c or smid lt 0.0)then smid=c/2
if((i-ii) gt 2)then signal(i-1)=smid+s0
signal(i)=smid
endif ; i-ii gt 1
;
endif ; t(i) gt tcrit
;
endif ; b lt 0.0 and c gt 0.0
;    
signal(i)=signal(i)+s0
;if(i gt 0)then print,'i,signal(i),signal(i-1) = ',i,signal(i),signal(i-1)
;signal=signal+d
;
endfor ; i
;
goto, c999
;
c555: ; FOUKESFAST STARTS (no loops)
;
;for i=0L,n-1 do begin ; i=0 corresponds to first signal
;
;if(sillum(i) ne sillump or i eq 0)then begin ; we have changed illumination
;
;if(i gt 0)then sf=signal(i-1)
s0=beta*(sillum(0)-sillump)+sf 
;sillump=sillum(i) ; update
b=(s0-beta*sillum(0)) ; scalar constants for plateau
a=(1.0-beta)*b*sillum(0) 
c=sillum(0)-s0
d=beta*sillum(0)
e=(sillum(0)^gammaexp)/tau ; foukes-schubert modification type I 
;                            pure foukes-schubert for gammaexp=1  
;
;ii=i-1 ; reference read for when illumination actually changed.
;if(i gt 0)then tchange=t(ii) ; assumes time zero for first flux change
;if(i eq 0)then tchange=0.0
tchange=0.0
;endif ; sillum(i) ne sillump
;
;print,'i,ii',i,ii
ee=(t-tchange)*e ; vector
;
if(b lt 0.0 and c gt 0.0)then goto, c850 ; special treatment of singularity
; to avoid detector diverging from a stable condition
;
signal=(a/(b+c*exp(-ee)))-b ; vector
;
signal=signal+s0
;signal=signal+d
;
;endfor ; i
;
c999:
;
return
end






