;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PIA P32 Processing Tools Software.              !
;! Designed by: Nanyao Lu at IPAC (lu@ipac.caltech.edu)                        !
;!              Bernhard Schulz at Vilspa (bschulz@iso.vilspa.esa.es)          !
;! Developed by: Iffat Khan at IPAC (irk@ipac.caltech.edu)                     !
;!                                                                             !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       GET_CONV_S2B      
;       (originated from ERD2MAP initial interface written by 
;        Carlos Gabriel @Vilspa)
;
; PURPOSE:
;       This routine converts signal into calibrated flux.
;
; DESCRIPTION:
;       The routine computes the flux calibration factor either using default
;       responsivity or using FCS1/FCS2 measurements. The output flux may be
;       surface brightness in MJy/sr or point source (Jy/beam).
;
; CATEGORY:
;       PIA - P32     
;
; CALLING SEQUENCE:
;       get_conv_s2b, imeas, itype, conv, FCSS=fcss, TMPERD=tmperd,
;                     CNV_TYPE=cnv_type
;
; INPUTS:
;       imeas: Selected measurement
;       itype: calibration method
;              itype = 0 -> default responsivity
;              itype = 1 -> responsivity calculated with the fcs1/fcs2 measurements.
;
; OPTIONAL INPUTS:
;       None.
;
; OUTPUTS:
;          
;       conv: conversion factor to get the desired flux values  
;
; KEYWORDS:
;       FCSS: ids of FCS1 and/or FCS2 measurements. meaningful only if itype = 1
;       TMPERD: temporary PIA structure. 
;       CNV_TYPE: Converted flux units
;                 cnv_type = 1 -> MJy/sr
;                 cnv_type = 2 -> Jy/beam
;
; CALLED ROUTINES:
;       DEFAULT_RESPONSE
;       DER_RESP_2FCS
;       FLUX_PC
;       GET_WHEELS
;       PIA_LOAD_IERD
;
; CALLED FROM:    
;       MJTOVS
;       PIA_CALIB
;       PREPAREDRIFT
;       PSFVS
;
; COMMON BLOCKS:
;
; PROCEDURE:
;       @phterd
;       @phtffield
;       @phtcap
;       @phtresact
;
; SIDE EFFECTS:
;       None.
;
; RESTRICTIONS:
;
;
; MODIFICATION HISTORY:
;	Modified design, Version 1.1 IRK        Jan 08, 2001
;-
;-----------------------------------------------------------------------
;
PRO get_conv_s2b, imeas, itype, conv, fcss=fcss, tmperd=tmperd, cnv_type=cnv_type

@phterd
@phtffield
@phtcap
@phtresact

        ctype = 1
        IF KEYWORD_SET(cnv_type) THEN BEGIN
          IF cnv_type EQ 1 THEN ctype=1 $
          ELSE IF cnv_type EQ 2 THEN ctype=0
        ENDIF

	IF not KEYWORD_SET(tmperd) THEN $
	 pia_load_ierd, phterd(imeas), tmperd
	
	IF itype THEN BEGIN
	
; calculate default response
	   defres = default_response(tmperd)
	   defres = defres(*,*,0)
; reduce to one value per pixel
	   the_pixels = N_ELEMENTS(defres(*,0))
	   respis = FLTARR(the_pixels)
	   FOR pixnr=0,the_pixels-1 DO $
	     respis(pixnr) = AVG(defres(pixnr,*))
; Actual responsivity
	ENDIF ELSE BEGIN
	   der_resp_2fcs, tmperd.det, fcss=fcss, error=error
           ;print,'error=',error
	   IF NOT error THEN BEGIN
	      IF tmperd.det EQ 'C1' THEN respis = c1respact ELSE $
	   			      respis = c2respact
	   ENDIF ELSE BEGIN
              XDISPLAYFILE, '', height=4, $
                     Text= ['Warning: No FCS measurements are found in SCP.', $
                          'Taking default responsivity.']
; calculate default response
	    defres = default_response(tmperd)
	    defres = defres(*,*,0)
; reduce to one value per pixel
	    the_pixels = N_ELEMENTS(defres(*,0))
	    respis = FLTARR(the_pixels)
	    FOR pixnr=0,the_pixels-1 DO $
	     respis(pixnr) = AVG(defres(pixnr,*))
	    MESSAGE, /INFO, 'Taking default responsivity !'
	   ENDELSE  	    
	ENDELSE
	   	   
	
; If phtffield not yet filled, read it from the FITS file
	IF n_elements(c1flatf) EQ 0 THEN read_flatfield
;  get wheel positions + ...
	get_wheels, tmperd.admn, wheels
	filt = wheels(1)
	det = tmperd.det
	IF det EQ 'C1' THEN BEGIN
	   	capac=c1cap 
	    	flatf = c1flatf(*,filt-2) 
	   ENDIF ELSE BEGIN
	   	capac=c2cap
	    	flatf = c2flatf(*,filt-9) 
	ENDELSE

	npix = pixel_det(tmperd.det)
	unitsig = fltarr(npix) + 1.
	conv = capac / respis / flatf

	
        ;print,'ctype = ',ctype 
	flux_pc, conv, tmperd.det, filt, filt, ctype, out
	IF ctype  EQ 1 THEN conv = out * 1.0E-06 $
        ELSE conv = out

       ;print,'ctype',ctype,conv

	
	RETURN
	END
