pro makemap,$
nmappix,mappix,mapbright,maprms,mapdisp,mapoffy,mapoffz,mapnum,iweight,iotf
;
; makes map from any combination of detector pixels (nmappix pixels in array 
; mappix(0,nmappix-1). Data in s with positions(arcsec) in y,z ytrue,ztrue
; is gridded onto a rectangular nygrid x nzgrid grid with pixels centred at 
; positions (arcsec) ygrid(0:nygrid-1) zgrid(0:nzgrid-1) and with sizes 
; (arcsec)
; sgridy x sgridz. Output is MAPBRIGHT(nygrid,nzgrid) for brightness 
; (input units of s), MAPRMS (nygrid,nzgrid) containing propagated input
; 1 sigma error calculated from RMS, MAPDISP (containing rms dispersion of
; values on each grid point/sqrt(number of contributing points). Also output 
; are the weighted mean positions of the data contributing to each map pixel 
; in y and z (mapoffy, mapoffz).
;
; if IWEIGHT is 0 then each input data point is given equal weight, and the 
; noise values in rms are ignored. This option is useful when comparing
; responsivities between detector pixels if the systematic calibration errors
; are unrelated to the detector noise. 
;
; If IWEIGHT is 1 then weighting is according to (1/rms)^2 of each individual
; input data point. If the input errors are accurate, this will give the
; best sensitivity. However it will be more prone to artifacts due to 
; any systematic errors in the input data.
;
; Only good data points (ie BAD zero) are used.
; Other data selection is by MAPPIX for origin of pixels
; and by iotf:
; if iotf = 1 only data on target (defined by fpt) are plotted
; if iotf = 0 only data off target (defined by fpt)are plotted
; if iotf = -1 all data are plotted
;
; to select according to iotf the fpt array is used:
;      FPT(0:nptin-1) = 1 on fine pointing =0 otherwise
;
; gridding is by simple averaging of points which land within area of pixel.
; This is expected not to introduce error for the iotf=1 option if the grid
; is chosen as the natural grid to coincide with the fine pointings of each
; detector pixel. However there will be errors for iotf=0 and iotf=-1 
; since the data will often not fall at the centre of a map pixel even if
; the natural grid is chosen. A future option here would be to do a quadratic
; surface fit from neighboring pixels.
;
; MAPNUM is output as the number of elementry data contributing to each pixel
;
; If no data points land within a map pixel then MAPRMS and MAPDISP is
; set to -99999.0 and MAPBRIGHT is set to the median of the nearest GRIDNEAR
; map pixels with data (to help display) for the pixel selection in MAPPIX.
; GRIDNEAR is an internal parameter.
;
; the errflag parameter(-99999.9) is also output
;
;common steerin,array,nptin,npt,ninramp,imeas,osamp
;common mapper,sgridy,sgridz,nygrid,nzgrid,ygrid,zgrid,errflag
;common signal,rs,cs,nch,s_in,s,badin,bad,rel,rms,glitch,jf,njf,ninchmax,rd,$
;nkf,kf,nkfmax,irelstart,nchopmed,gbad,npolyrms,nrmsmed,sigmaxfract
;common plotter,otfstart,otfstop,rdstat,chop,otf,rpid,yrel,zrel,$
;ytrue,ztrue,yzrpidcen,nyraster,nzraster
;common wherepoint,idy,idz
;common pinfo,fpt
;
@common_steerin
@common_mapper
@common_signal
@common_plotter
@common_wherepoint
@common_pinfo
;
if(n_elements(nmappix) le 0)then begin
print,'makemap,nmappix,mappix,mapbright,maprms,mapdisp,mapoffy,mapoffz,mapnum,iweight,iotf'
print,'eg: makemap,npix,lindgen(npix),mapb,mapr,mapd,mapy,mapz,mapn,0,1'
print,'    for no weighting, using only data from fine pointings'
print,'eg: makemap,npix,lindgen(npix),mapb,mapr,mapd,mapy,mapz,mapn,1,-1'
print,'    for weighting, using only data from fine pointings and slews'
goto,c999
endif
;
if(nmappix ne n_elements(mappix))then begin
print,'MAKEMAP: aborting - error in specification of detector pixels'
goto,c999
endif ; nmappix ne n_elements(mappix)
;
gridnear=8 ; = 8 for nearest pixels around a single hole
gridnear=3 ; .... but 8 may give too much smearing
errflag=-99999.0
;
;print,'MAKEMAP: s(4,1000:1025) = ',s(4,1000:1025) ; diagnostic
;print,'MAKEMAP: bad(4,1000:1025) = ',bad(4,1000:1025) ; diagnostic
;print,'MAKEMAP: ytrue(4,1000:1025) = ',ytrue(4,1000:1025) ; diagnostic
;print,'MAKEMAP: ztrue(4,1000:1025) = ',ztrue(4,1000:1025) ; diagnostic
;
mapbright=fltarr(nygrid,nzgrid) ; initialised 0.0
maprms=mapbright 
mapnum=mapbright
maprms(*,*)=errflag ; initialised with errflag
mapdisp=maprms
mapoffy=maprms
mapoffz=mapoffy
;
; concatenate data rms pointing information for all detector pixels in MAPPIX
;
szz=dblarr(nmappix*nptin)
rmszz=szz
yzz=szz
zzz=szz
fzz=szz
badz=lonarr(nmappix*nptin)
for ip = 0L, nmappix-1 do begin
ipix = mappix(ip)
i1=ip*nptin
i2=i1+nptin-1
badz(i1:i2)=bad(ipix,*)
szz(i1:i2)=s(ipix,*)
rmszz(i1:i2)=rms(ipix,*)
yzz(i1:i2)=ytrue(ipix,*)
zzz(i1:i2)=ztrue(ipix,*)
fzz(i1:i2)=fpt(*)
;if(ip eq 4)then print,'MAKEMAP: i1+1000, i2+1025', i1+1000,i2+1025 ; diagnostic
endfor ; ip
;
; data selection according to bad flag and pointing
;
if(iotf eq 1)then igrid=where(badz eq 0 and fzz eq 1,ngrid) 
if(iotf eq 0)then igrid=where(badz eq 0 and fzz eq 0,ngrid) 
if(iotf eq -1)then igrid=where(badz eq 0,ngrid) 
print,'MAKEMAP: number of data points gridded = ',ngrid
sz=szz(igrid)
yz=yzz(igrid)
zz=zzz(igrid)
rmsz=rmszz(igrid)
;
; calculate which map pixel each data point belongs to
;
evenodd,nygrid,iyodd
if(iyodd eq 1)then begin ; find map pixel identifiers iyz for map columns
iyz=round(yz/sgridy)
iyz=iyz+nygrid/2
endif ; iyodd
if(iyodd ne 1)then begin ; even number of map columns
iyz=round((yz/sgridy)-0.5)
iyz=iyz+nygrid/2
endif ; iyodd
evenodd,nzgrid,izodd
if(izodd eq 1)then begin ; find map pixel identifiers izz for map rows
izz=round(zz/sgridz)
izz=izz+nzgrid/2
endif ; izodd
if(izodd ne 1)then begin ; even number of map rows
izz=round((zz/sgridz)-0.5)
izz=izz+nzgrid/2
endif ; izodd
;print,'MAKEMAP: min(iyz),max(iyz) = ',min(iyz),max(iyz) ; diagnostic
;print,'MAKEMAP: min(izz),max(izz) = ',min(izz),max(izz) ; diagnostic
if(min(iyz) lt 0 or max(iyz) gt nygrid-1)then begin
print,'MAKEMAP: warning - data outside grid in Y'
print,' sgridy,nygrid,min(iyz),max(iyz) = ',sgridy,nygrid,$
min(iyz),max(iyz) 
endif ; min(iyz)
if(min(izz) lt 0 or max(izz) gt nzgrid-1)then begin
print,'MAKEMAP: warning - data outside grid in Z'
print,' sgridz,nzgrid,min(izz),max(izz) = ',sgridz,nzgrid,$
min(izz),max(izz) 
endif ; min(izz)
;
; now grid data
;
for iy=0L,nygrid-1 do begin
for iz=0L,nzgrid-1 do begin
;
iw=where(iyz eq iy and izz eq iz,niw)
if(niw le 0)then goto, c333
;
sw=sz(iw)
sw=double(sw)
yw=yz(iw)
yw=double(yw)
zw=zz(iw)
zw=double(zw)
rmsw=rmsz(iw)
rmsw=double(rmsw)
wt=rmsw ; initialisation
if(iweight eq 0) then wt(*)=1.0 ; no weighting if this option chosen
if(iweight eq 1) then wt=(1.0/rmsw)*(1.0/rmsw)
wt=wt/total(wt) ; normalised weights of points contributing to pixel iy,iz
mapnum(iy,iz)=niw
;
rmsprop,sw,rmsw,smean,ssig,ssigmean,iweight
;
;mapbright(iy,iz)=total(wt*sw)
mapoffy(iy,iz)=total(wt*yw)
mapoffz(iy,iz)=total(wt*zw)
mapbright(iy,iz)=smean
maprms(iy,iz)=ssigmean ; propagation of weighted rmsw to give error in mean
mapdisp(iy,iz)=ssig    ; propagation of weighted sw-smean to give error in mean
;maprms(iy,iz)=sqrt(total(wt*wt*rmsw*rmsw))
;av=total(sw)/float(niw)
;dummy=total(sw*sw)-(float(niw)*av*av)
;if(niw gt 1)then mapdisp(iy,iz)=sqrt(dummy/float(niw))/sqrt(niw-1) 
;if(niw eq 1)then mapdisp(iy,iz)=sqrt(dummy/float(niw))/sqrt(niw) 
;
c333:
;
endfor ; iz
endfor ; iy
;
;print,'MAKEMAP: data gridded'
;
; now deal with points which have no data
;
r=fltarr(nygrid,nzgrid) ; radius of other pixels from test pixel
y=r ; y offset for test pixel ...
z=r ; z offset
br=r ; brightness
rr=r ; rms 
for iy=0L,nygrid-1 do begin
for iz=0L,nzgrid-1 do begin
;
if(maprms(iy,iz) ne errflag)then goto, c334 ; data exists so go directly to next point
;
for iiz=0L,nzgrid-1 do begin
y(*,iiz)=(findgen(nygrid)-float(iy))*sgridy
endfor ; iiz
for iiy=0L,nygrid-1 do begin
z(iiy,*)=(findgen(nzgrid)-float(iz))*sgridz
endfor ; iiy
r=sqrt(y*y+z*z)
irw=where(maprms ne errflag,nrw)
rw=r(irw)
bw=mapbright(irw)
isrw=sort(rw)
rw=rw(isrw)
bw=bw(isrw)
mapbright(iy,iz)=total(bw(0:gridnear-1))/float(gridnear)
;
;if(print,'MAPBRIGHT
;
c334:
;
endfor ; iz
endfor ; iy
;
c999:
;
return
end

