;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PIA P32 Processing Tools Software.              !
;! Designed by: Nanyao Lu at IPAC (lu@ipac.caltech.edu)                        !
;!              Bernhard Schulz at Vilspa (bschulz@iso.vilspa.esa.es)          !
;! Developed by: Iffat Khan at IPAC (irk@ipac.caltech.edu)                     !
;!                                                                             !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_CALIB        
;
; PURPOSE:
;       This routine performs the flux calibration.
;
; DESCRIPTION:
;       The routine computes the calibrated flux output as follows:
;       1. Obtain the linearized signal if the user has chosen the option 
;
;       2. Depending on the selected calibration method, the flux correction
;          factor is computed for the desired flux units
;
;       3. Multiply the signal array by the correction factor to obtain the
;          calibrated flux.
;
; CATEGORY:
;       PIA - P32     
;
; CALLING SEQUENCE:
;       pia_calib, pid, CNV_TYPE=cnv_type 
;
; INPUTS:
;       pid:      pointer variable to global parameters structure
;
; OPTIONAL INPUTS:
;       None.
;
; OUTPUTS:
;       None.
;
; KEYWORDS:
;       CNV_TYPE: The calibrated flux type                
;                 cnv_type = 1  Surface Brightness (MJy/sr)        
;                 cnv_type = 2  Flux Density (Jy/beam) 
;       If the keyword is not defined, cnv_type is set to 1
;
; CALLED ROUTINES:
;       GET_CONV_S2B
;       GET_RLIN
;       PIA_LOAD_IERD
;
; CALLED FROM:    
;       PIA_ERD2MAP_MAIN      
;
; COMMON BLOCKS:
;       DRIFT_STRUCT
;
; PROCEDURE:
;       @common_all
;       @common_detpars
;       @phterd
;       @phtfilter
;
; SIDE EFFECTS:
;       None.
;
; RESTRICTIONS:
;
;
; MODIFICATION HISTORY:
;	original design, Version 1.1 IRK        Jan 08, 2001
;       Version 1.35                            Mar 08, 2001
;       Version 1.40                            Mar 16, 2001
;-
;-----------------------------------------------------------------------
;
PRO pia_calib, pid, cnv_type=cnv_type           

@phterd
@common_all
@phtfilter
COMMON drift_struct, drift_struct

	 	
        IF NOT KEYWORD_SET(cnv_type) THEN cnv_type = 1

; linearize signals first, if required
        IF (*pid).lin_sig EQ 1 THEN BEGIN
 	   pia_load_ierd, phterd((*pid).inderd), tmperd
   	   GET_rlin, tmperd, s, sig_cor
   	   s = sig_cor
	ENDIF
        
        IF  (*pid).cal_type EQ 0 THEN BEGIN  ;Default
  	      get_conv_s2b, imeas, 1, result1, cnv_type=cnv_type
	      FOR i=0,N_ELEMENTS(result1)-1 DO s(i,*) = s(i,*) * result1(i)
	      (*pid).bright = 1
		
	 ENDIF ELSE BEGIN                 ;Actual	 		
		    
              CASE (*pid).cal_type OF
                1: fcss = [(*pid).fcs1, (*pid).fcs2]
                2: fcss = (*pid).fcs1
                3: fcss = (*pid).fcs2
              ENDCASE
             ;print, 'fcss',fcss
       
              get_conv_s2b, imeas, 0, result1, fcss=fcss, cnv_type=cnv_type
              FOR i=0,N_ELEMENTS(result1)-1 DO s(i,*) = s(i,*) * result1(i)
		
	 ENDELSE		

         (*pid).bright = 1
	
	RETURN
	END
	
	
	
	
	
