;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PIA P32 Processing Tools Software.              !
;! Designed by: Nanyao Lu at IPAC (lu@ipac.caltech.edu)                        !
;!              Bernhard Schulz at Vilspa (bschulz@iso.vilspa.esa.es)          !
;! Developed by: Iffat Khan at IPAC (irk@ipac.caltech.edu)                     !
;!                                                                             !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_ERD2MAP_INIT
;
; PURPOSE:
;       This routine starts a graphical user interface to P32 Data 
;       initialization.
;
; DESCRIPTION:
;       The GUI buttons on the widget let the user to select options for
;       ramps linearization, dark current subtraction, and reset interval        
;       correction. 'erdtomap' processing is started with a click to 'OK'.  
;       'Cancel' will takes you back to the measurement selection widget.         
;
;       once ERDTOMAP processing is complete, data is saved in the virtual
;       memory by copying from local buffer (COMMON SIGNAL). This data is
;       kept frozen and can only be changed by deglitcher. This data is      
;       copied into the local buffer before performing the drift correction.
;
;       The new keywords written in fits file header during the processing
;       are as follows:
;       PR_LINC : if Linearize Ramps is selected
;
;       if reset interval correction is applied:
;       PR_RIN0 : a0 of polynomial    
;       PR_RIN1 : a1 of polynomial
;       PRS_R0_S : Systematic relative uncertainty in a0
;       PRS_R1_S : Systematic relative uncertainty in a1
;
;       PRS_DARK : if dark current subtraction is applied
;
; CATEGORY:
;       PIA - P32     
;
; CALLING SEQUENCE:
;       pia_erd2map_init, pid, GROUP=group
;
; INPUTS:
;       pid:      pointer variable to global parameters structure
;
; OPTIONAL INPUTS:
;       None.
;
; OUTPUTS:
;       None.
;
; KEYWORDS:
;       GROUP : The group leader of the Data Initialization widget. 
;               If none defined then PIA top widget is taken as the
;               group leader.
;
; CALLED ROUTINES:
;       ERDTOMAP
;       FILL_ERD
;       LIN_VOLTAGES
;       OSAMP_FCT
;       PIA_ERD2MAP_INIT_EVENT (Internal procedure of Event handler)
;       PIA_ERD2MAP_MAIN
;       PIA_LOAD_IERD
;
; CALLED FROM:    
;       PIA_ERD2MAP      
;
; COMMON BLOCKS:
;       
;
; PROCEDURE:
;       @common_all
;       @phterd     
;       @phtfilter  
;
; SIDE EFFECTS:
;       None.
;
; RESTRICTIONS:
;
;
; MODIFICATION HISTORY:
;	original design,             IRK  Version 1.0      Jan, 2001
;       add keywords to FITS header  IRK  Version 1.3      Feb, 2001
;-
;-----------------------------------------------------------------------
;


PRO PIA_erd2map_init_EVENT,event
@phterd
@common_all
@phtfilter


	WIDGET_CONTROL, event.id, GET_UVALUE=action
	WIDGET_CONTROL, event.top, GET_UVALUE=pid
        debug = 0
	
	CASE action of 
	

	 'CORR': BEGIN
	  	CASE event.value OF
		  0: (*pid).lin_volt = event.select
		  1: (*pid).res_int_cor = event.select
		  2: (*pid).darks = event.select
		ENDCASE
                IF debug THEN BEGIN
                   print,(*pid).lin_volt
                   print,(*pid).darks
                   print,(*pid).res_int_cor
                ENDIF
	 END
	 
	 "OK": BEGIN
                WIDGET_CONTROL, event.top, /hourglass
                phdr = (*pid).hdr
                tmphdr = *phdr

                IF ((*pid).lin_volt EQ 1) THEN BEGIN

	 	  pia_load_ierd, phterd((*pid).inderd), tmp
		  lin_done = fxpar(tmp.hdr,'PR_LINC',count=count)
		  newhdr = tmp.hdr
		  IF count EQ 0 THEN BEGIN
		    lin_voltages, tmp, newhdr, /silent 
		    fill_erd, tmp, newhdr, replace=(*pid).inderd
; use replace now		  (*pid).inderd = N_ELEMENTS(phterd)-1
		  ENDIF ELSE $
          	    PRINT,' Linearization already performed, skipped'
                ENDIF
		n1 =  (*pid).n1
		n2 =  (*pid).n2
		pix =  (*pid).pix
		n1 = LONG(n1(0)) & n2 = LONG(n2(0)) & pix = FIX(pix(0))
		arrayin = STRLOWCASE(STRMID(phterd((*pid).inderd).admn,0,2))+$
				'00'
	 	handle_value, phterd((*pid).inderd).rpid, rpid 
                ny=max(rpid(0,*)) & nz=max(rpid(1,*))
                IF debug THEN print,'ny nz=',ny,nz

	 	handle_value, phterd((*pid).inderd).hdr, hdr
		osamp = osamp_fct(arrayin,nz,hdr) 

; what it needs, apparently
		dirnam=!PIA_TOP + '/'
		tdtnam=STRMID(phterd((*pid).inderd).admn, 4, 8)
		iseq = '01'
		caldir = !CALG
		
	 	erdtomap, arrayin, osamp, 'n', 0, 0, 1,PROC_FCS=(*pid).proc_fcs,$
		 n1,n2,pix,'x', tmphdr, measpos = (*pid).inderd, /VERBOSE, $
		 reset_int_corr=(*pid).res_int_cor, $
		 dark_cur_subt=(*pid).darks, slin_cor=0, $
		 MROLL=mroll, /NO_PLOT
                 ;save,s,bad,time, file= "/stage/toolsd/pia/ia/erd2map_save"
		(*pid).roll = mroll
		(*pid).bright = 0
		(*pid).dr_fr = 1
		(*pid).dr_to = nyraster*nzraster
		(*pid).drift_cor=0				
                 e2map_struc = {time:time, s:s, rms:rms, bad:bad, ytrue:ytrue, ztrue:ztrue}
                 dptr = ptr_new(e2map_struc)
                 IF debug THEN help,dptr

                 IF phdr NE ptr_new() THEN ptr_free, phdr
                 (*pid).hdr = ptr_new(tmphdr)

                 betalongconmin=0.01
                 betalongconmax=0.99
                 betashortconmin=0.0
                 betashortconmax=0.5
                 taulongconmin=0.01
                 taulongconmax=10000.
                 taushortconmin=0.001
                 taushortconmax=5.0
                 illcon1=0.5  ; reference brightnesses in V/sec for constraints
                 illcon2=20.0  ; These should be interactively varied within
                              ; the actual brightness range.

                  
                  WIDGET_CONTROL, event.top, /DESTROY

	 END

	 "CANCEL": BEGIN
                 (*pid).map_can = 1
                  WIDGET_CONTROL, event.top, /DESTROY
	 END

         "HELP": BEGIN
           IF !VERSION.OS NE 'vms' THEN $
             spawn, "$w3net -remote  'openURL('$PIA_ERD2MAP_HTML'/p32_erd2map.html)'"  $
           ELSE $
             spawn, "w3net -remote  'openURL('$PIA_ERD2MAP_HTML'/p32_erd2map.html)'",/nowait
         END


	 ELSE:

	ENDCASE

        IF debug THEN print,'ready to return'
 	RETURN
	END
	

	
PRO PIA_erd2map_init, pid, GROUP=group

  debug =0

  IF not KEYWORD_SET(group) THEN group=!PIA_TOP_WID
  id_base= WIDGET_BASE(TITLE="Data Initialization Options ", /COLUMN, $
		  /MODAL, GROUP=group, XSIZE=300)
	
  id_info = WIDGET_BASE( id_base, /COLUMN , /ALIGN_CENTER)

  id_0 = WIDGET_BASE( id_info,  /ALIGN_CENTER, /COL, /FRAME)


  otfval = ((*pid).otf - 1) * (-1)

  object =  (*pid).object
  junk  = WIDGET_LABEL( id_0, VALUE= object) 

  admn =  (*pid).admn
  dumstr = "Measurement:          " + admn
  junk  = WIDGET_LABEL( id_0, VALUE= dumstr)


  plid = (*pid).plid
  dumstr = "Proposal ID:          " + plid
  junk  = WIDGET_LABEL( id_0, VALUE= dumstr, /align_left)


  filter = (*pid).filter
  dumstr = "Filter:               " + filter
  junk  = WIDGET_LABEL( id_0, VALUE= dumstr, /align_left)

  raster = (*pid).raster
  dumstr = "Map Size:             " + raster
  junk  = WIDGET_LABEL( id_0, VALUE= dumstr, /align_left)

  (*pid).map_can = 0

  id_0 = WIDGET_BASE(id_base, /ROW, /FRAME, /ALIGN_CENTER)
  stdbut = cw_bgroup(id_0,  $ 
		/NONEXCLUSIVE, ROW=3, $
		SET_VALUE = [(*pid).lin_volt, $
	        (*pid).darks, $
                (*pid).res_int_cor],  $
		UVALUE = 'CORR', $
		["Linearize ramps",  $
		 "Dark current subtraction",   $
                 "Reset interval correction"] )  

  id_0 = WIDGET_BASE(id_base, /ROW)
  junk = WIDGET_BUTTON( id_0, VAL="     Ok      ", UV="OK" )
  junk = WIDGET_BUTTON( id_0, VAL="   Cancel    ", UV="CANCEL" )
  junk = WIDGET_BUTTON( id_0, VAL="    Help     ", UV="HELP" )

	
  WIDGET_CONTROL, id_base, /realize, set_UVALUE=pid	
  Xmanager, /NO_BLOCK, "PIA_ERD2MAP_INIT", id_base, GROUP=group
  IF debug THEN print,'out event init', (*pid).map_can
  IF (*pid).map_can EQ 1 THEN $
    pia_erd2map $ 
  ELSE $ 
    pia_erd2map_main, pid

  RETURN
  END
	
