;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PIA P32 Processing Tools Software.              !
;! Designed by: Nanyao Lu at IPAC (lu@ipac.caltech.edu)                        !
;!              Bernhard Schulz at Vilspa (bschulz@iso.vilspa.esa.es)          !
;! Developed by: Iffat Khan at IPAC (irk@ipac.caltech.edu)                     !
;!                                                                             !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_ERD2MAP_MAIN
;
; PURPOSE:
;       This routine starts a main P32 processing Tools graphical user
;       interface. 
;
; DESCRIPTION:
;       The GUI consists of the following major sections.
;       1. Display info on the selected measurement
;
;       2. GUI buttons to show FITS header, Compact Status, to set the map 
;          display options. and to display Pixel map, Grid and Map.
;
;       3. Method of Error determination by selecting one of the 4 radio buttons.
;
;       4. Selection of Slew Data type by selecting one of the 3 radio buttons.
;          (Note: this option links to Richard's integer ISLEWMETHOD:
;            Excluded   -> ISLEWMETHOD = 0 
;            Coadded    -> ISLEWMETHOD = 1 
;            Sequential -> ISLEWMETHOD = 2  
;       5. Text buttons to enter the desired number of iterations.
;
;       6. Buttons to save/restore Drift fiiting parameters from a file or
;          to reset to defaults.
;
;       7. Processing buttons
;          CALCULATE_ON_ALL_PXIELS: calculates drift correction for all pixels 
;                                   using the current drift paramters.
;          INSPECT_SINGLE_PIXEL:    Starts the GUI for single pixel detailed 
;                                   processing.
;          SAVE/RESTORE        :    Save the local data buffer (COMMON SIGNAL)
;                                   onto a Disk file. Restore the local data
;                                   buffer (COMMON SIGNAL) from a Disk file.
;          EXPORT_RESULTS_TO_SCP:   Export data in V/s to SCP level in PIA.
;                                   2 new keywords are written into FITS header
;                                   before exporting. These are:    
;                                   PRS_VIGN : if Vignetting correction was applied  
;                                   PRS_DEGL : if Deglitching correction was applied
;
; CATEGORY:
;       PIA - P32     
;
; CALLING SEQUENCE:
;       pia_erd2map_main, pid, GROUP=group
;
; INPUTS:
;       pid:      pointer variable to global parameters structure
;
; OPTIONAL INPUTS:
;       None.
;
; OUTPUTS:
;       None.
;
; KEYWORDS:
;       GROUP : The group leader of the P32 Tools widget. 
;               If none defined then PIA top widget is taken as the
;               group leader.
;
; CALLED ROUTINES:
;       CONVERT_E2M_PIA
;       CREATE_MAPPAR_FT
;       DRIFT32
;       FINDP32GRID
;       GRIDDISP
;       MAKEMAP
;       MAKEMAPMULTI
;       MAPCONTOURMULT
;       PIA_CALIB   
;       PIA_DISP_CS
;       PIA_DISP_HDR
;       PIA_DISP_MAP
;       PIA_DISP_PRINT_FILE
;       PIA_ERD2MAP_MAIN_EVENT (Internal procedure of Event handler)
;       PIA_ERD2MAP_MAPDRAW 
;       PIA_ERD2MAP_SPIX
;       PIA_MKTRANS     
;       PIA_PICKFILE 
;       RESTORE_PARAMS (Internal procedure)
;
; CALLED FROM:    
;       PIA_ERD2MAP_INIT      
;
; COMMON BLOCKS:
;       DRIFT_STRUCT
;       PIXPTR
;       E2M_MAIN_LOCIDS
;
; PROCEDURE:
;       @common_all
;       @common_detpars
;       @phterd
;
; SIDE EFFECTS:
;       None.
;
; RESTRICTIONS:
;
;
; MODIFICATION HISTORY:
;	original design, Version 1.1 IRK        Jan 08, 2001
;       Version 1.35                            Mar 08, 2001
;       Version 1.40                            Mar 16, 2001
;-
;-----------------------------------------------------------------------
;

PRO PIA_erd2map_main_EVENT,event
@phterd
@common_all
COMMON e2m_main_locids, lid
COMMON drift_struct, drift_struct
common pixptr, map_all, map_pix

        debug = 0
	WIDGET_CONTROL, event.id, GET_UVALUE=action
	WIDGET_CONTROL, event.top, GET_UVALUE=pid
	
        IF debug THEN  help,event,/str
	CASE action of 
	
		
	 'RMS': BEGIN
	 	(*pid).irmscorr = event.select
	 END


	 'SLEW': BEGIN
                 islewmethod =  event.value 
                 (*pid).slew = islewmethod
                 IF debug THEN print,'slew=',(*pid).slew
         END

	 'SAVE_BUF': BEGIN
                 text="_DRIFT"
                 is = STRTRIM( (*pid).meas, 2)
                 if(array eq 'c100') then tfile= 'C1_'+tdtnam+'_'+is+text+'.dat'
                 if(array eq 'c200')then tfile ='C2_'+tdtnam+'_'+is+text+'.dat'

                 filter = '*'
                 name = PIA_pickfile(file=tfile, filter=filter, path=!INTDIR,  /overwrite)
                 IF name EQ  '' THEN BEGIN
                   MESSAGE, /INFO, 'No file chosen'
                   RETURN
                 ENDIF
                 IF debug THEN print,name
                 save,s,bad,time,ytrue,ztrue,rms,ndriftsolve,drift_struct,file=name

         END

	 'REST_BUF': BEGIN
                 text="_DRIFT"
                 is = STRTRIM((*pid).meas, 2)
                 if(array eq 'c100') then tfile= 'C1_'+tdtnam+'_'+is+text+'.dat'
                 if(array eq 'c200')then tfile ='C2_'+tdtnam+'_'+is+text+'.dat'

                 filter = '*'
                 name = PIA_pickfile(file=tfile, filter=filter, path=!INTDIR )
                 IF name EQ  '' THEN BEGIN
                   MESSAGE, /INFO, 'No file chosen'
                   RETURN
                 ENDIF
                 OPENR, 1, name, ERROR=err
                 IF debug THEN print,name, err
                 IF err NE 0 THEN BEGIN
                  XDISPLAYFILE, '', height=3, $
                     Text= 'Error: File does not exist'
                  RETURN
                ENDIF
                CLOSE, /all
                 restore,name
                 drindx = where(ndriftsolve gt 0, ndrcnt)
                 IF ndrcnt GT 0 THEN BEGIN
                   (*pid).dr_init = 0
                   (*pid).drift_cor = 1
                 ENDIF
                 IF debug THEN print,(*pid).dr_init,(*pid).drift_cor

         END

         'MMAP': BEGIN
                 WIDGET_CONTROL, event.top, /hourglass
                
                mappix = indgen(npix)
                makemapmulti,npix,mappix,mapall,mapallrms,mmapd,mmapy,mmapz,$
                 mmapn,1,(*pid).otf
                mapcontourmult,1,10,1.0,0,mapall,mapallrms,'x'
         END

         'GRID': BEGIN
                WIDGET_CONTROL, event.top, /hourglass
                IF debug THEN print,'pix',(*pid).pix
                pix = (*pid).pix 
                handle_value, phterd((*pid).inderd).rpid, rpid
                ny=max(rpid(0,*)) & nz=max(rpid(1,*))
                findp32grid,ny,nz
                griddisp,ny,nz,pix-1,-1,0,'x'
         END

         'CS': PIA_disp_cs, phterd((*pid).inderd).admn

         'HDR': BEGIN
                handle_value, phterd((*pid).inderd).hdr, hdr
                pia_disp_hdr, hdr, phterd((*pid).inderd).admn, 'PHTERD'
         END

         'MDISP': BEGIN
                  pia_disp_map, pid
                  IF debug THEN print,'out unit',(*pid).map_unit
                  IF debug THEN print,'out opt ',map_all, map_pix
         END

         'SAVE': BEGIN            
                 text="_DRIFT"
                 is = STRTRIM((*pid).meas, 2)
                 if(array eq 'c100') then tfile= 'C1_'+tdtnam+'_'+is+text+'.pars'
                 if(array eq 'c200')then tfile ='C2_'+tdtnam+'_'+is+text+'.pars'
                 filter = '*'
                 name = PIA_pickfile(file=tfile, filter=filter, path=!INTDIR, /overwrite)
                 IF name EQ  '' THEN BEGIN
                   MESSAGE, /INFO, 'No file chosen'
                   RETURN
                 ENDIF
                 varp = (*pid).dr_wps
                 vstr = *varp
                 vars = vstr.vars
                 IF debug THEN  help,vars
                 SAVE, vstr, filename=name
         END

         'LOAD': BEGIN 
                 text="_DRIFT"
                 is = STRTRIM((*pid).meas, 2)
                 if(array eq 'c100') then tfile= 'C1_'+tdtnam+'_'+is+text+'.pars'
                 if(array eq 'c200')then tfile ='C2_'+tdtnam+'_'+is+text+'.pars'
                 filter = '*'
                 name = PIA_pickfile(file=tfile, filter=filter,  path=!INTDIR)
                 IF name EQ  '' THEN BEGIN
                   MESSAGE, /INFO, 'No file chosen'
                   RETURN
                 ENDIF
                 IF debug THEN print,name
                 openr, 1, name, ERROR=err
                 IF debug THEN print,name, err
                 IF err NE 0 THEN BEGIN
                  XDISPLAYFILE, '', height=3, $
                     Text= 'Error: File does not exist'
                  RETURN
                ENDIF
                CLOSE, /all
                RESTORE, name       
                 (*pid).dr_wps = ptr_new(vstr)
         END

         'RESET': BEGIN
            vars = lid.vars
            flags = lid.flags
            (*pid).dr_wps = ptr_new({vars:vars, flags:flags})
            (*pid).cal = 0
         END

         'CUST_PROC': BEGIN
            pia_cust_drift_proc, pid, GROUP=event.top
         END


         'DRIFT': BEGIN
                WIDGET_CONTROL, event.top, /hourglass
; get general not changing values
                debug = 0
                if debug then goto, debug
                wptr = (*pid).dr_wps
                vars = (*wptr).vars
                pars = (*wptr).flags
                restore_params, vars, pars     
                IF debug THEN print,'drift vars',vars
                dr_param = intarr(12)
                IF debug THEN print,'param',dr_param
                speedfactor = 2.
                zerobias = 0.
                istartnewsol = 1
                e2map_str = *dptr
                IF debug THEN help,e2map_str,/str
                time = e2map_str.time
                ytrue = e2map_str.ytrue
                ztrue = e2map_str.ztrue
                s = e2map_str.s
                rms = e2map_str.rms
                bad = e2map_str.bad
                bad_in = bad
                IF debug THEN help,s,rms,bad
                idefcal = 1
                islewmethod = (*pid).slew
                PRINT, 'ATTENTION, ATTENTION: using islewmethod: ',islewmethod
                tplatback = 0. & tplatforward = 0.
                icompositecorr = 0
                nlookback = 1
                iverbose = 0

                imodel = (*pid).drift_model
                icalmethod = 4

           

; implement solution?
                implementdrift = 1

; processing from where to where
                 processfrom = (*pid).dr_fr - 1
                 processto = (*pid).dr_to - 1

                IF (*pid).dr_allpix THEN $
                        pixarray = (*pid).dr_pix - 1 ELSE $
                        pixarray = indgen(npix)

; self calibration
                IF icalmethod EQ 2 THEN BEGIN
                    self_cal = (*pid).self_cal
                        IF N_ELEMENTS(processfrom) GT 1 THEN BEGIN
                           MESSaGE, /INFO, $
                            'Self calibration requires a single pointing range'
                           RETURN
                        ENDIF
; full range     
                    ical1 = intarr(npix)
                    ical2 = ical1
                        ical1(pixarray) = processfrom
                        ical2(pixarray) = processto
                    IF self_cal EQ 0 THEN BEGIN
                        icompositecorr = 0
; composite: just central raster - ical2=ical1
                    ENDIF ELSE IF self_cal EQ 1 THEN BEGIN
                        icompositecorr = 1
                    ENDIF
; for the case it is absolutely needed
                ENDIF ELSE BEGIN
                    ical1 = processfrom
                    ical2 = ical1
                ENDELSE

                (*pid).bright = 0
                compositeweight = [1.0, 0.0, 0.0]
                WIDGET_CONTROL, lid.ndetinit, GET_VALUE=val
                (*pid).ndetinit = val(0)
                WIDGET_CONTROL, lid.niter, GET_VALUE=val
                (*pid).niter = val(0)

                ndetinit = (*pid).ndetinit
                niter = (*pid).niter
                inegmethod = (*pid).inegmethod
                backpred = (*pid).backpred
                irmscorr = (*pid).irmscorr

                isolveselfcal = dr_param

                illcon1 = 0.5
                illcon2 = 20.0

           IF debug THEN print,'dr_init', (*pid).dr_init
; Process finally
                drift32,processfrom,processto,idefcal,$
                iverbose,pixarray,niter,$
        icalmethod,tplatback,tplatforward,nlookback,ical1,ical2,isolveselfcal,$
        irmscorr,implementdrift,istartnewsol,ndetinit,icompositecorr, $
        /use_buffer, dr_init=(*pid).dr_init
debug:
        bad = bad_in
        (*pid).drift_cor=1
        (*pid).dr_init = 0
           IF debug THEN print,'dr_init', (*pid).dr_init

         END   

         'SPIX': BEGIN
                 IF debug THEN print,'npix=',npix
                 vptr = (*pid).dr_wps
                 tmpvars = *vptr
                 IF debug THEN  help,vptr, tmpvars 

                 IF debug THEN help, (*pid).dr_tps
                 (*pid).dr_tps = ptr_new(tmpvars)
		 WIDGET_CONTROL, lid.ndetinit, GET_VALUE=val
		 (*pid).ndetinit = val(0)
		 WIDGET_CONTROL, lid.niter, GET_VALUE=val
		 (*pid).niter = val(0)
                 pia_erd2map_spix, pid
 
         END   

         'MAP': BEGIN
                IF debug THEN print,'map opts', map_all, map_pix

                if map_all then begin
                   allpix=npix
                   mappix = indgen(npix)
                ENDIF ELSE BEGIN
                   spix = where(map_pix GT 0, allpix)
                   IF allpix GT 0 THEN mappix=spix  
                ENDELSE 
                IF allpix EQ 0 THEN BEGIN
                  XDISPLAYFILE, '', height=3, $
                     Text= 'Warning: None of the pixels selected'   
                   RETURN
                ENDIF
                IF debug THEN  help,allpix,mappix
                WIDGET_CONTROL, event.top, /hourglass
                s_in  = s
                bad_in = bad
                IF debug THEN print,'bright', (*pid).bright
                FOR i = 0,allpix-1 DO BEGIN
                    ipix = mappix(i)
                    print,'pix drift',ipix,ndriftsolve(ipix)
                    IF ndriftsolve(ipix) GT 0 THEN BEGIN
                      itrans = WHERE(drift_struct.itrans EQ 1, ntrans)
                      print,ntrans,n_elements(bad(ipix,*))
                      IF ntrans EQ N_ELEMENTS(bad(ipix,*)) THEN BEGIN
                        l_maskout = drift_struct.maskout(ipix, itrans)
                        mskind = WHERE(l_maskout GT 0, nmask)
                        print,'nmask',nmask
                        IF nmask GT 0 THEN BEGIN
                            bdum = where(bad(ipix,mskind) gt 0, bcnt)
                            print,'bcnt',bcnt
                            bad(ipix,mskind) = 1
                       ENDIF
                      ENDIF
                    ENDIF
                ENDFOR

                unit = (*pid).map_unit
                type = 0
                datatype = 0
                IF unit EQ 0 THEN BEGIN
                     pia_calib, pid,  cnv_type=1
                     IF debug THEN print,'mjy',min(s),max(s)
                ENDIF ELSE IF unit EQ 1 THEN BEGIN   ; Jy/Beam
                     datatype = 4
                     pia_calib, pid, cnv_type=2
                     IF debug THEN print,'jy',min(s),max(s)

                ENDIF ELSE IF unit EQ 2  THEN BEGIN
                   type = -2
                ENDIF
                IF debug THEN print,mappix
                erd2map_ff = (*pid).erd2map_ff
                handle_value, phterd((*pid).inderd).rpid, rpid
                ny=max(rpid(0,*)) & nz=max(rpid(1,*))
                findp32grid,ny,nz ; define grid
                s_orig=s
                IF erd2map_ff THEN BEGIN
                   IF allpix EQ 1 THEN BEGIN
                     XDISPLAYFILE, '', height=3, $
                     Text= 'Warning: Flat fielding was not performed on single pixel selection.'
                   ENDIF ELSE BEGIN
; use first quartile flatfielding
                     ff = fltarr(npix)   
                     IF debug THEN print,'flat fld=',ff
                     FOR i = 0,allpix-1 DO BEGIN
                        ipix = mappix(i)
                        ind  = WHERE(bad(ipix,*) EQ 0,n)
                        fmed = MEDIAN(s(ipix,ind))
                        indind = WHERE(s(ipix,ind) LT fmed)
                        ff(ipix) = MEDIAN(s(ipix,ind(indind)))
                     ENDFOR
                     mff = avg(ff)
                     ff = ff / mff
                     IF debug THEN print,'flat fld=',ff, mff
                     FOR i=0,allpix-1 DO BEGIN
                        ipix = mappix(i)
                        s(ipix,*) = s(ipix,*) / ff(ipix)
                     ENDFOR
                        PIA_DISP_PRINT_FILE, '',TITLE='Flat field calculation',$
                                TEXT=['The calculated flat field parameters:',$
                                'Pixel '+$
                                STRTRIM(STRING(indgen(npix)+1),2)+':' +  $
                                STRING(ff)]

                    ENDELSE
                ENDIF
                   

                makemap,allpix,mappix,mapbright,maprms,mapdisp,mapoffy,mapoffz,$
                        mapnum,1,(*pid).otf
; but put original s array back
                s = s_orig
; mirror the map
                xele = N_ELEMENTS(mapbright(*,0))
                mtmp = mapbright
                FOR ix=0,xele-1 DO mtmp(xele-1-ix,*) = mapbright(ix,*)
                mapbright = mtmp

; we assume a perfect raster with ra and dec as in hdr and
; a correct centering of the map (for the moment)
                handle_value, phterd((*pid).inderd).hdr, hdr
                siz = size(mapbright)
                refpix = (siz(1:2) -1) / 2.
                ra = fxpar(hdr,'ATTRA')
                dec = fxpar(hdr,'ATTDEC')
                mroll = (*pid).roll
                trans = pia_mktrans([ra,dec],$
                                refpix, [sgridy,sgridz], mroll)
                create_mappar_ft, trans, par
                par.aapin=1
                par.detector=phterd((*pid).inderd).det
                par.admn=phterd((*pid).inderd).admn
                par.admn_decoded = dec_admn(par.admn)
                par.rota = mroll
                par.ra = ra
                par.dec = dec
                par.npts = fxpar(hdr,'ATTRNPTS')
                par.nlns = fxpar(hdr,'ATTRNLNS')
                par.dpts = fxpar(hdr,'ATTRDPTS')
                par.dlns = fxpar(hdr,'ATTRDLNS')
                par.trans_ra = ra & par.trans_dec = dec
                par.pixsize = [sgridy,sgridz]
                par.datatype = datatype
                hdline = 'ERD2MAP of '+fxpar(hdr,'OBJECT')

                pia_erd2map_mapdraw, mapbright, trans, par, hdr, type, $
                   TITLE='ERD2MAP Special map', $
                   Headline=hdline
                s = s_in
                bad = bad_in
                IF (*pid).drift_cor EQ 0 THEN (*pid).bright = 0
         END

	 "SCP": BEGIN
               IF (*pid).drift_cor EQ 0 THEN BEGIN
                  XDISPLAYFILE, '', height=3, $
                     Text= 'Warning: No drift correction was applied yet'   
                   RETURN
                ENDIF

                 
               unit = phterd((*pid).inderd).unit
               det = phterd((*pid).inderd).det
               admn = phterd((*pid).inderd).admn
               ordr = phterd((*pid).inderd).ordr
               hdr = (*pid).hdr
               vers= !P32TOOLS_VERSION 
               newhdr = *hdr
	       fxaddpar, newhdr, 'P32_VERS', $
	        vers, ' P32TOOLS version used', $
		  BEFORE='XTENSION'

	       true = 1
	       fxaddpar, newhdr, 'PRS_VIGN', $
	        true, ' Vignetting corr. applied', $
		  BEFORE='XTENSION'
	       s_orig = s
;	       IF (*pid).lin_sig EQ 1 THEN BEGIN
		  ;pia_load_ierd, phterd((*pid).inderd), tmperd
		  ;GET_rlin, tmperd, s, sig_cor
	          ;s = sig_cor

	          ;fxaddpar, newhdr, 'PRS_SLIN', $
		  ;true, ' Signal linearity correction performed', $
		  ;BEFORE='XTENSION'
               ;ENDIF
               IF (*pid).imple EQ 1 THEN BEGIN
	          fxaddpar, newhdr, 'PRS_DEGL', $
		  true, ' Deglitching correction applied', $
		  BEFORE='XTENSION'
               ENDIF


               convert_e2m_pia, unit, det, admn, ordr, newhdr
	       s = s_orig
               is = STRTRIM( (*pid).meas, 2)
               text = ' has been created in PIA SCP buffer'
               if(array eq 'c100') then msg= 'C1-'+tdtnam+'-'+is+text
               if(array eq 'c200')then msg ='C2-'+tdtnam+'-'+is+text
               XDISPLAYFILE, '', height=3, $
                  Text= msg

	 END
	
	 "QUIT": BEGIN
                journal
	 	WIDGET_CONTROL, event.top, /DESTROY
	 END

         "HELP": BEGIN
           IF !VERSION.OS NE 'vms' THEN $
             spawn, "$w3net -remote 'openURL('$PIA_ERD2MAP_HTML'/p32_main.html)' &"  $
           ELSE $
             spawn, "w3net -remote 'openURL('$PIA_ERD2MAP_HTML'/p32_main.html)'",/nowait
         END

	
	 ELSE:

	ENDCASE

 	RETURN
	END
	

	
PRO PIA_erd2map_main, pid, GROUP=group

COMMON e2m_main_locids, lid
@common_names
@common_steerin
common pixptr, map_all, map_pix


        debug = 0
	IF not KEYWORD_SET(group) THEN group=!PIA_TOP_WID
        vers = !P32TOOLS_VERSION
        main_tit = "PIA P32 TOOLS - " + vers
	id_base= WIDGET_BASE(TITLE=main_tit, /COLUMN)

        id_info = WIDGET_BASE( id_base, /COLUMN , /ALIGN_CENTER)
        id_0 = WIDGET_BASE( id_info,  /ALIGN_CENTER, /COL, /FRAME)

        object =  (*pid).object
        junk  = WIDGET_LABEL( id_0, VALUE= object) 

        admn =  (*pid).admn
        dumstr = "Measurement:          " + admn
        junk  = WIDGET_LABEL( id_0, VALUE= dumstr)

        plid = (*pid).plid
        dumstr = "Proposal ID:          " + plid
        junk  = WIDGET_LABEL( id_0, VALUE= dumstr, /align_left)


        filter = (*pid).filter
        dumstr = "Filter:               " + filter
        junk  = WIDGET_LABEL( id_0, VALUE= dumstr, /align_left)

        raster = (*pid).raster
        dumstr = "Map Size:             " + raster
        junk  = WIDGET_LABEL( id_0, VALUE= dumstr, /align_left)
        vptr = (*pid).dr_wps
        vars = (*vptr).vars
        flags = (*vptr).flags       
        det = (*pid).det
       ;filename=STRMID(STRUPCASE(det),0,2)+iseq+tdtnam+'.E2M_PARS'
        IF debug THEN print,n_elements(vars)
        IF debug THEN print,vars(0:11)   
        map_pix(*) = 1


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	id_0  =  WIDGET_BASE(id_base, /COL, /FRAME, /align_center)
	lab = WIDGET_LABEL(id_0, VAL=' SHOW', /align_center, $
                           FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')
	id_1  =  WIDGET_BASE(id_0, ROW=1, /align_center)
        junk = WIDGET_BUTTON(id_1, VALUE="  Pixel Map  ",UVAL="MMAP")
        junk = WIDGET_BUTTON(id_1, VALUE="   Grid   ",UVAL="GRID")
        junk = WIDGET_BUTTON(id_1, VALUE="   Header   ",UVAL="HDR")
        junk = WIDGET_BUTTON(id_1, VALUE="  Com. Status  ",UVAL="CS")
	id_2  =  WIDGET_BASE(id_0, ROW=1,/align_center)
        junk = WIDGET_BUTTON(id_2, VALUE="Map Display options",UVAL="MDISP")
	junk = WIDGET_BUTTON(id_2, VAL="  Display Map  ", UV="MAP" )

	
	pbase= WIDGET_BASE(id_base, /COL, /FRAME)
        lab = WIDGET_LABEL(pbase,VAL='DRIFT processing', /align_center,  $
                           FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')
	id_0 = WIDGET_BASE(pbase, /COL)
        lab = WIDGET_LABEL(id_0 ,VAL='Determine the errors in the result by:', /align_left,    $
                           FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')
	id_1 = WIDGET_BASE(id_0, /COL, /align_center)
        stdbut = cw_bgroup(id_1,   $
                        /EXCLUSIVE, COLUMN=2, $
                        SET_VALUE = (*pid).impl_drft, $
                        UVALUE = 'RMS', $
                        ["Measurement errors", $
                         "Responsivity scaling ", $
                         "Internal dispersion ", $
                         "Flux step dispersion" ])

	id_0 = WIDGET_BASE(pbase, /COL)
        lab = WIDGET_LABEL(id_0 ,VAL='Options of Using Slew Data:', /align_left,  $
                           FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')
	id_1 = WIDGET_BASE(id_0, /COL)
        stdbut = cw_bgroup(id_1,   $
                        /EXCLUSIVE, COLUMN=3, $
                        SET_VALUE = (*pid).slew, $
                        UVALUE = 'SLEW', $
                        ["Excluded", $
                         "Coadded", $
                         "Sequence" ])

	id_0 = WIDGET_BASE(pbase, /COL)
        lab = WIDGET_LABEL(id_0 ,VAL='Number of Iterations:', /align_left,  $
                           FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')
	id_1 = WIDGET_BASE(id_0, /ROW)
        ndetinit = WIDGET_TEXT( id_1, VAL=STRTRIM((*pid).ndetinit,2), $
                UV="JUNK", /editable, xsize=4 )
        junk =  WIDGET_LABEL(id_1, VAL='# of iter. for Start')
        niter = WIDGET_TEXT( id_1, VAL=STRTRIM((*pid).niter,2), UV="JUNK", $
                /editable, xsize=4  )
        junk = WIDGET_LABEL(id_1, VAL='# of major iterations')


	id_0 = WIDGET_BASE(pbase, /COL)
        lab = WIDGET_LABEL(id_0 ,VAL='Drift Fitting Parameters:', /align_left,  $
                           FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')
	id_1 = WIDGET_BASE(id_0, /ROW,space=45)
        junk = WIDGET_BUTTON( id_1, VALUE=' Save to File ', UVALUE='SAVE')
        junk = WIDGET_BUTTON( id_1, VALUE=' Load from File ', UVALUE='LOAD')
        junk = WIDGET_BUTTON( id_1, VALUE=' Reset to Default ', UVALUE='RESET')


	lid = {ndetinit:ndetinit, niter:niter, siglin:1, slew:0, $
               vars:vars, flags:flags}


	id_0 = WIDGET_BASE(id_base, ROW=2, /FRAME,space=15)
	junk = WIDGET_BUTTON(id_0, VAL=" Calculate on All Pixels ", UV="DRIFT" )
	junk = WIDGET_BUTTON(id_0, VAL="  Inspect Single Pixel   ", UV="SPIX" )
	srid = WIDGET_BUTTON(id_0, VAL="  Save/Restore ", /menu)
	junk = WIDGET_BUTTON(srid, VAL="  Save Data Buffer into a Disk File ", UVAL="SAVE_BUF")
	junk = WIDGET_BUTTON(srid, VAL="  Restore Data Buffer from a Disk File ", UVAL="REST_BUF")
	junk = WIDGET_BUTTON(id_0, VAL=" Export Results to SCP Level ", uv="SCP") 
	junk = WIDGET_BUTTON(id_0, VAL="  Display Map  ", UV="MAP" )
	junk = WIDGET_BUTTON(id_0, VAL=" Custom Processing ", UV="CUST_PROC" )


	id_4 = WIDGET_BASE(id_base, /ROW)
	id0 = WIDGET_BUTTON( id_4, VAL="  Close   ", /menu)      
	id1 = WIDGET_BUTTON( id0, VAL="Are you Sure?", UV="QUIT" )
	id2 = WIDGET_BUTTON( id_4, VAL="   Help   ", UV="HELP" )

	WIDGET_CONTROL, id_base, /realize, set_UVALUE=pid	
	Xmanager, /NO_BLOCK, "PIA_ERD2MAP_MAIN", id_base, GROUP=group
        IF debug THEN print,'return from main event'

	RETURN
	END
	
	
