;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PIA P32 Processing Tools Software.              !
;! Designed by: Nanyao Lu at IPAC (lu@ipac.caltech.edu)                        !
;!              Bernhard Schulz at Vilspa (bschulz@iso.vilspa.esa.es)          !
;! Developed by: Iffat Khan at IPAC (irk@ipac.caltech.edu)                     !
;!                                                                             !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_ERD2MAP_SPIX
;
; PURPOSE:
;       This routine starts a P32 single pixel processing graphical
;       user interface. 
;
; DESCRIPTION:
;       The GUI consists of the following major sections.
;
;       1. Slider widget for pixel selection       
;
;       2. Deglitching tool (see PIA_ERD2MAP_DG.PRO)
;
;       3. Display of selected pixel data                                        
;          either the whole data without rebining, or
;          rebin per ramp: by taking average of NNDR (read from COMP.STAT)
;          data points
;          rebin per chopper: by taking average of data points between 2     
;          chopper sweeps.
;          The data is plotted in red color, slew data in blue, and BAD data
;          in yellow colors. The chopper sweeps are plotted as white dotted
;          lines.
;
;       4. Self Calibration of 12 detector parameters 
;
;          12 text buttons are shown to enter the desired value of parameters.  
;          The values are entered in column matrices of 'A', 'B', and 'C'.
;          and later copied into 12 modal parameters used in the processing.
;
;          Initially, they contain the values of model parameters saved in
;          Working Parameter Set (WPS). A copy of WPS is generated at the
;          start of the gui. This is called Temporary parameter set(TPS).
;          After drift processing, TPS is updated with the recalculated values
;          of free parameters which are also written into their text widgets.
;
;          Also, 12 toggle buttons are shown to specify which parameters will 
;          be self calibrated using the initial guesses read from the text
;          buttons. Buttons 'on' means self calibration is applied,
;          'off' keeps the parameter value fixed during fitting. 
;
;          Buttons to reset the parameter values to the ones at the start 
;          of this GUI or to the default values.
;
;          Raster range selection text buttons
;
;          Options for self calibration: perform on composite swep or full range
;
;          Button to recalculate parameters using above input and apply corrections
;          to signal and rms.
;
;       5. Control buttons    
;
;          ACCEPT: TPS is copied to WPS and the widget is closed. The new
;                  parameter values will be used for any subsequent
;                  drift processing.
;          CLOSE: Ignore the current parameter values in TPS and close the widget.
;                 On return, the parameter values in WPS will be the same as at
;                 the start of this GUI.
;          HELP:  Provides help on the processing in this GUI.
;
; CATEGORY:
;       PIA - P32     
;
; CALLING SEQUENCE:
;       pia_erd2map_spix, pid, GROUP=group
;
; INPUTS:
;       pid:      pointer variable to global parameters structure
;
; OPTIONAL INPUTS:
;       None.
;
; OUTPUTS:
;       None.
;
; KEYWORDS:
;       GROUP : The group leader of the P32 Tools widget. 
;               If none defined then PIA top widget is taken as the
;               group leader.
;
; CALLED ROUTINES:
;       CONV_CS
;       DRIFT32
;       PIA_ERD2MAP_SPIX_EVENT (Internal procedure of Event handler)
;       PIA_ERD2MAP_DG    
;       PIA_ERD2MAP_XPLOT
;       PIA_SET_PARS
;       PIA_SET_WPS 
;       PIA_SET_TPS (Internal procedure)
;       RASTER_RANGE (Internal procedure)
;       READ_CS
;       RESTORE_TEMP_PARS (Internal procedure)
;       SET_COLOURS
;       STARTENDPOINT
;
; CALLED FROM:    
;       PIA_ERD2MAP_MAIN      
;
; COMMON BLOCKS:
;       E2M_SPIX_LOCIDS
;
; PROCEDURE:
;       @common_all
;       @common_detpars
;       @phtcs  
;       @phterd 
;       @phtfilter
;
; SIDE EFFECTS:
;       None.
;
; RESTRICTIONS:
;       A maximum of 6 free parameters is allowed for Self Calibration
;
; MODIFICATION HISTORY:
;	original design, Version 1.1 IRK        Jan 08, 2001
;       Version 1.35                            Mar 08, 2001
;       Version 1.40                            Mar 16, 2001
;       Version 1.50                            Mar 30, 2001
;-
;-----------------------------------------------------------------------
;

PRO  restore_temp_pars, lid, npixl, pvars, pflags


       debug = 0
       avar = strarr(4) & bvar = strarr(4) & cvar = strarr(4)
       afl = intarr(4) & bfl = intarr(4) & cfl = intarr(4)

       pia_set_pars, npixl, pvars, pflags, avar, bvar, cvar, afl, bfl, cfl
       IF debug THEN print,'avar',avar
       IF debug THEN print,'flags',afl,bfl,cfl
       FOR i = 0,3 DO BEGIN
          WIDGET_CONTROL, lid.a(i), SET_VALUE = avar(i)
          WIDGET_CONTROL, lid.b(i), SET_VALUE = bvar(i)
          WIDGET_CONTROL, lid.c(i), SET_VALUE = cvar(i)
          IF afl(i) EQ 1 THEN $
            WIDGET_CONTROL, lid.af(i), SET_BUTTON=1 $
           ELSE $
            WIDGET_CONTROL, lid.af(i), SET_BUTTON=0
          IF bfl(i) EQ 1 THEN $
            WIDGET_CONTROL, lid.bf(i), SET_BUTTON=1 $
           ELSE $
            WIDGET_CONTROL, lid.bf(i), SET_BUTTON=0
          IF cfl(i) EQ 1 THEN $
            WIDGET_CONTROL, lid.cf(i), SET_BUTTON=1 $
           ELSE $
            WIDGET_CONTROL, lid.cf(i), SET_BUTTON=0
       ENDFOR
       lid.avar = avar
       lid.bvar = bvar
       lid.cvar = cvar
 
       lid.varsa = afl
       lid.varsb = bfl
       lid.varsc = cfl

END

PRO pia_set_tsp, lid, npixl, vars, pars
@common_all

       debug = 0
       IF debug THEN print,'save tsp pixl',npixl

          vars(0,npixl) = beta(npixl)
          vars(1,npixl) = betacoeff
          vars(2,npixl) = betaexp

          vars(3,npixl) = tauoffset
          vars(4,npixl) = tau(npixl)
          vars(5,npixl) = gammaexp

          vars(6,npixl) = betashoot
          vars(7,npixl) = betacoeffshoot
          vars(8,npixl) = betaexpshoot

          vars(9,npixl) = tauoffsetshoot
          vars(10,npixl) = taushoot
          vars(11,npixl) = gammaexpshoot

          flags = pars       
;         flags(*, npixl) = 0

       avar = strarr(4) & bvar = strarr(4) & cvar = strarr(4)
       afl = intarr(4) & bfl = intarr(4) & cfl = intarr(4)

       pia_set_pars, npixl, vars, flags, avar, bvar, cvar, afl, bfl, cfl
       IF debug THEN print,'avar',avar
       IF debug THEN print,'flags',afl,bfl,cfl
       FOR i = 0,3 DO BEGIN
          WIDGET_CONTROL, lid.a(i), SET_VALUE = avar(i)
          WIDGET_CONTROL, lid.b(i), SET_VALUE = bvar(i)
          WIDGET_CONTROL, lid.c(i), SET_VALUE = cvar(i)
          IF afl(i) EQ 1 THEN $
            WIDGET_CONTROL, lid.af(i), SET_BUTTON=1 $
           ELSE $
            WIDGET_CONTROL, lid.af(i), SET_BUTTON=0
          IF bfl(i) EQ 1 THEN $
            WIDGET_CONTROL, lid.bf(i), SET_BUTTON=1 $
           ELSE $
            WIDGET_CONTROL, lid.bf(i), SET_BUTTON=0
          IF cfl(i) EQ 1 THEN $
            WIDGET_CONTROL, lid.cf(i), SET_BUTTON=1 $
           ELSE $
            WIDGET_CONTROL, lid.cf(i), SET_BUTTON=0
       ENDFOR
       lid.avar = avar
       lid.bvar = bvar
       lid.cvar = cvar

       lid.varsa = afl
       lid.varsb = bfl
       lid.varsc = cfl

RETURN
END
	
	
pro raster_range, pid, n1, n2
COMMON e2m_spix_locids, lid
	
   debug = 0
   WIDGET_CONTROL, lid.sinfrom, GET_VALUE=val
   n1 = FIX(val(0))

   IF n1 GT (*pid).dr_to  THEN BEGIN
      XDISPLAYFILE, '', height=5,$
        Text= ['ERROR: from raster position exceeds maximum ',  $
               'please reenter the value.']
      RETURN
   ENDIF
     
     
   IF n1 LE 0 THEN BEGIN
     n1 = 1
     WIDGET_CONTROL, lid.sinfrom, SET_VALUE=STRTRIM(n1, 2)
   ENDIF
   n1 = n1 - 1      
     
   WIDGET_CONTROL, lid.sinto, GET_VALUE=val
   n2 = FIX(val(0)) 

   IF n1 GT n2  THEN BEGIN
      XDISPLAYFILE, '', height=5,$
        Text= ['ERROR: from raster position exceeds to raster',  $
               'please reenter the values.']
      RETURN
   ENDIF
   IF n2 GT lid.rsize THEN BEGIN
     n2 = lid.rsize
     WIDGET_CONTROL, lid.sinto, SET_VALUE=STRTRIM(n2, 2)
   ENDIF
   n2 = n2 - 1      

   IF debug THEN print,'n1,n2',n1,n2

RETURN
END


PRO PIA_erd2map_spix_EVENT,event
@common_all
@phtcs
COMMON e2m_spix_locids, lid

        debug = 0
        action = ''
	WIDGET_CONTROL, event.id, GET_UVALUE=action
	WIDGET_CONTROL, event.top, GET_UVALUE=pid
        IF debug THEN help,event,/str
	
	CASE action of 
	
 	 'SELECT': BEGIN
                cpix = lid.pix
                IF debug THEN print,cpix
                tptr = (*pid).dr_tps
                vars = (*tptr).vars
                pars = (*tptr).flags
                IF debug THEN help,pars, vars
                IF debug THEN print,pars

                save_temp_pars, lid, vars, pars

                (*pid).dr_tps = ptr_new({vars:vars, flags:pars})
                IF debug THEN print,pars
                pix = event.value
                lid.pix = pix
         
                npixl = pix - 1
                IF debug THEN print,'restoring pixel #', pix
                vptr = (*pid).dr_tps
                pvars = (*vptr).vars
                pflags = (*vptr).flags
                restore_temp_pars, lid, npixl, pvars, pflags
         END
         'REBIN': BEGIN
                 lid.rebin = event.value
                IF debug THEN print,'rebin',lid.rebin 

         END

         'RESET': BEGIN

                pix = lid.pix 
                npixl = pix - 1
                tptr = (*pid).dr_tps
                vars = (*tptr).vars
                pars = (*tptr).flags
                vars(*, npixl) = lid.vars(*, npixl)
                pars(*, npixl) = lid.flags(*, npixl)
                IF debug THEN print,vars(*,*)
                restore_temp_pars, lid, npixl, vars, pars
                ptr_free, tptr
                (*pid).dr_tps = ptr_new({vars:vars, flags:pars})
         END

         'RESET_DEF': BEGIN

                pix = lid.pix 
                npixl = pix - 1
                tptr = (*pid).dr_tps
                vars = (*tptr).vars
                pars = (*tptr).flags
                IF debug THEN help,vars,npixl
                pia_set_wps, vars, pixl=npixl
                IF debug THEN print,vars(*,*)
                restore_temp_pars, lid, npixl, vars, pars
                ptr_free, tptr
                (*pid).dr_tps = ptr_new({vars:vars, flags:pars})
         END
         'ACCEPT': BEGIN

                tptr = (*pid).dr_tps
                vars = (*tptr).vars
                pars = (*tptr).flags
                save_temp_pars, lid, vars, pars
                ptr_free, tptr
                (*pid).dr_tps = ptr_new({vars:vars, flags:pars})
                IF debug THEN print,tptr    
                ind = WHERE(pars EQ 1, pcnt)
                IF debug THEN print,pcnt,ind
                IF pcnt GT 0 THEN BEGIN
                      XDISPLAYFILE, '', height=5,$
                     Text= ['Warning: free drift parameters still present',  $
                            'please fix these before leaving.']
                   RETURN
                ENDIF
                IF debug THEN print,vars(*,*)
                (*pid).dr_wps = ptr_new({vars:vars, flags:pars})

                 e2map = *dptr
                 badp = lid.save_bad
                 newbad = *badp
                 e2map.bad = newbad
                 *dptr = e2map

                 WIDGET_CONTROL, event.top, /DESTROY
         END

	 'CAL': BEGIN
	        conv_tab = [0,4,2,5]
	 	(*pid).cal = conv_tab(event.value)
		
		
	        IF (*pid).cal EQ 2 THEN $
		   WIDGET_CONTROL, lid.self_cal, sensitive=1 ELSE $
		   WIDGET_CONTROL, lid.self_cal, sensitive=0 
	        IF (*pid).cal EQ 2 THEN $
		   WIDGET_CONTROL, lid.varpar, sensitive=1 ELSE $
		   WIDGET_CONTROL, lid.varpar, sensitive=0 
         END


	 'FREE1': BEGIN
                  indx = WHERE(lid.af eq event.id, cnt)
                  IF debug THEN print,'indx',indx, event.select
                  lid.varsa(indx) = event.select
	 END
	 	  
	 'FREE2': BEGIN
                  indx = WHERE(lid.bf eq event.id, cnt)
                  IF debug THEN print,'indx',indx, event.select
                  lid.varsb(indx) = event.select
	 END
	 	  
	 'FREE3': BEGIN
                  indx = WHERE(lid.cf eq event.id, cnt)
                  IF debug THEN print,'indx',indx, event.select
                  lid.varsc(indx) = event.select
	 END
	 	  
	 'SELFCAL': BEGIN
	 	(*pid).self_cal = event.value
	 END

	 'DISP': BEGIN
               avg_pts = 0
               IF lid.rebin GT 0 THEN BEGIN
                  measname = phterd((*pid).inderd).admn
                  IF debug THEN print,'name',measname
                  psta_file='PSTA'+strmid(measname,4,8)+'.FITS'
                  read_cs,psta_file,header,csun

                  ; return to caller if compact status not found

                  if csun EQ -1 then begin
                        msg = 'Compact Status: ' + psta_file + ' not found!'
                        message, /info, msg
                        RETURN
                  endif

                  conv_cs,phtcs(csun),convcs
                  handle_value,convcs.handle,cst
;calculate index
                  detshort = STRMID(measname,0,2)
                  IF detshort EQ 'SS' THEN detshort = 'SL'

                  index_arr=where(strmid(cst.subs,0,2) eq detshort,count)

                  if count eq 0 then begin
                    MESSAGE, /INFO,'CS Not available!'
                    RETURN
                  endif else begin
                    in=index_arr(fix(strmid(measname,2,2)))
                    nndr = FIX(cst.nndr(in)) + 1    
                    ndr = FIX(cst.ndr(in))
                    rst_time = cst.intt(in)
                    IF debug THEN print,'ndr ninrmp',ndr,nndr    
                    IF lid.rebin EQ 1 THEN avg_pts = nndr $
                    ELSE IF lid.rebin EQ 2 THEN avg_pts = -1
                  endelse

                  IF debug THEN print,'enter display'
                ENDIF

                (*pid).dr_pix = lid.pix  
                IF debug THEN print,'pix=',(*pid).dr_pix
                IF (*pid).dr_pix GT npix OR (*pid).dr_pix LT 1 THEN BEGIN
                      XDISPLAYFILE, '', height=5,$
                     Text= 'ERROR: Illegal entry for Pixel.'
                   RETURN
                ENDIF

                pixplt = lid.pix - 1
                b1ind = where(bad(pixplt,*) GT 0 , bcnt)
                b1ind = where(bad(pixplt,*) eq 0, gcnt)
                IF debug THEN print,'good bad=', gcnt, bcnt
                IF debug THEN print,'ndrift',ndriftsolve(pixplt)      
                raster_range, pid, n1, n2
                set_colours 
                pia_erd2map_xplot, n1, n2, 'time', pixplt, avg_pts

	 END

         'DEGLCH': BEGIN
                raster_range, pid, n1, n2
                pixplt = lid.pix - 1
                WIDGET_CONTROL, event.top, /hourglass
                local_bad = bad(pixplt,*)
                local_s = s(pixplt,*)
                dgl_imple = 0
                startendpoint,n1, n2,  $
                irasterstart,irasterstop,istartnoe,istopnoe,$
                irasterstarte,irasterstope,istarte,istope

                e2map_str = *dptr

                time(istartnoe:istopnoe) = e2map_str.time(istartnoe:istopnoe)
                ytrue(istartnoe:istopnoe) = e2map_str.ytrue(istartnoe:istopnoe)
                ztrue(istartnoe:istopnoe) = e2map_str.ztrue(istartnoe:istopnoe)
                s(pixplt,istartnoe:istopnoe) = e2map_str.s(pixplt, istartnoe:istopnoe)
                rms(pixplt,istartnoe:istopnoe)= e2map_str.rms(pixplt, istartnoe:istopnoe)
                badp = lid.save_bad
                bads = *badp
                bad (pixplt,istartnoe:istopnoe)= bads(pixplt, istartnoe:istopnoe)
                b1ind = where(bad(pixplt,*) GT 0, bcnt)
                b1ind = where(bad(pixplt,*) eq 0, gcnt)
                IF debug THEN print,'good bad=', gcnt, bcnt
                (*pid).imple = 0
                pia_erd2map_dg, n1, n2, pixplt, pid
                IF debug THEN print,'out of glitch with imple', (*pid).imple
                b1ind = where(bad(pixplt,*) gt 0, bcnt)
                b1ind = where(bad(pixplt,*) eq 0, gcnt)
                IF debug THEN print,'good bad=', gcnt, bcnt
                IF (*pid).imple GT 0 THEN BEGIN
                   (*pid).drift_cor = 0
                    *badp = bad
                ENDIF ELSE $
          ;          bad(pixplt,*) = local_bad
          ;     b1ind = where(bad(pixplt,*) gt 0, bcnt)
          ;     b1ind = where(bad(pixplt,*) eq 0, gcnt)
          ;     IF debug THEN print,'good bad=', gcnt, bcnt
          ;     print,'good bad=', gcnt, bcnt
                lid.dg_imple(pixplt) = 1
                
	 END

         'DRIFT': BEGIN
                WIDGET_CONTROL, event.top, /hourglass
; get general not changing values
                speedfactor = 2.
                zerobias = 0.
                
                islewmethod = (*pid).slew
                idefcal = 1
                PRINT, 'ATTENTION, ATTENTION: using islewmethod: ',islewmethod
                tplatback = 0. & tplatforward = 0.
                icompositecorr = 0
                nlookback = 1
                iverbose = 0
                (*pid).bright = 0
                imodel = (*pid).drift_model

                (*pid).dr_pix = lid.pix  
                IF (*pid).dr_pix GT npix OR (*pid).dr_pix LT 1 THEN BEGIN
                      XDISPLAYFILE, '', height=5,$
                     Text= 'ERROR: Illegal entry for Pixel.'
                   RETURN
                ENDIF
IF debug THEN print,(*pid).dr_pix
                npixl = (*pid).dr_pix - 1
                istartnewsol = 1
;               IF ndriftsolve(npixl) GT 0 THEN istartnewsol = 0
                IF debug THEN print,'new sol', istartnewsol

                avar = lid. avar
                bvar = lid. bvar
                cvar = lid. cvar
                FOR i = 0,3 DO BEGIN
        
                   WIDGET_CONTROL, lid.a(i), GET_VALUE = tmp    
                   avar(i) = tmp(0)
                   WIDGET_CONTROL, lid.b(i), GET_VALUE = tmp    
                   bvar(i) = tmp(0)
                   WIDGET_CONTROL, lid.c(i), GET_VALUE = tmp    
                   cvar(i) = tmp(0)
                ENDFOR
                dr_param = [lid.varsa(0), lid.varsb(0), lid.varsc(0), $
                            lid.varsa(1), lid.varsb(1), lid.varsc(1), $
                            lid.varsa(2), lid.varsb(2), lid.varsc(2), $
                            lid.varsa(3), lid.varsb(3), lid.varsc(3)]   

               IF debug THEN print,'pix=',npixl
               IF debug THEN print,avar,bvar,cvar
               IF debug THEN print,dr_param
                cal = 0
                pia_get_pars, npixl, avar, bvar, cvar, dr_param, cal
                (*pid).dr_param = dr_param
                (*pid).cal = cal            

                pixarray = (*pid).dr_pix - 1
                IF debug THEN help,pixarray
                icalmethod = (*pid).cal
               ;icalmethod = 0
; implement solution?
                implementdrift = 1

                raster_range, pid, n1, n2
; processing from where to where
                from = n1 
                to = n2 
                processfrom = from
                processto = to
                IF islewmethod EQ 0 THEN BEGIN
                   processfrom = lindgen(to-from+1)+from
                   processto = processfrom
                ENDIF
                IF debug THEN help, processfrom, processto
                IF debug THEN help,s

;retrieve erd2map data for this pixel within desired range
                startendpoint,from, to,     $
                irasterstart,irasterstop,istartnoe,istopnoe,$
                irasterstarte,irasterstope,istarte,istope

                e2map_str = *dptr
                IF debug THEN help,e2map_str,/str

                time(istartnoe:istopnoe) = e2map_str.time(istartnoe:istopnoe)
                ytrue(istartnoe:istopnoe) = e2map_str.ytrue(istartnoe:istopnoe)
                ztrue(istartnoe:istopnoe) = e2map_str.ztrue(istartnoe:istopnoe)
                s(npixl,istartnoe:istopnoe) = e2map_str.s(npixl, istartnoe:istopnoe)
                rms(npixl,istartnoe:istopnoe)= e2map_str.rms(npixl, istartnoe:istopnoe)
                badp = lid.save_bad
                bads = *badp
                bad (npixl,istartnoe:istopnoe)= bads(npixl, istartnoe:istopnoe)
                tindx = where(time GT 193 and time lt 194, tcnt)
               ;for i=0,tcnt-1 do IF debug THEN print,time(tindx(i)),s(1,tindx(i)),bad(1,tindx(i))
                bind = where(bad(npixl,*) EQ 0, gcnt)
                IF debug THEN print,'good bad=', gcnt, bcnt
                IF gcnt GT 0 THEN BEGIN
                   illcon1 = MIN(s(npixl, bind))
                   illcon2 = MAX(s(npixl, bind))
                ENDIF 
                IF illcon1 LE 0. THEN illcon1 = 0.5
                IF illcon2 LE 0. THEN illcon2 = 20.0

; self calibration
                IF icalmethod EQ 2 THEN BEGIN
                    self_cal = (*pid).self_cal
                        IF N_ELEMENTS(processfrom) GT 1 THEN BEGIN
                           MESSaGE, /INFO, $
                            'Self calibration requires a single pointing range'
                           RETURN
                        ENDIF
; full range     
                    IF debug THEN print,'spix npix',npix
                    ical1 = intarr(npix)
                    ical2 = ical1
                        ical1(pixarray) = processfrom
                        ical2(pixarray) = processto
                    IF self_cal EQ 0 THEN BEGIN
                        icompositecorr = 0
; composite: just central raster - ical2=ical1
                    ENDIF ELSE IF self_cal EQ 1 THEN BEGIN
                        icompositecorr = 1
                    ENDIF
; for the case it is absolutely needed
                ENDIF ELSE BEGIN
                    ical1 = processfrom
                    ical2 = ical1
                ENDELSE

                compositeweight = [1.0, 0.0, 0.0]

                ndetinit = (*pid).ndetinit
                niter = (*pid).niter
                inegmethod = (*pid).inegmethod
                backpred = (*pid).backpred
                irmscorr = (*pid).irmscorr
                print,'irmscorr',irmscorr

                isolveselfcal = dr_param



            
; Process finally
                drift32,processfrom,processto,idefcal,$
                iverbose,pixarray,niter,$
        icalmethod,tplatback,tplatforward,nlookback,ical1,ical2,isolveselfcal,$
        irmscorr,implementdrift,istartnewsol,ndetinit,icompositecorr, $
        /use_buffer, dr_init=(*pid).dr_init

                b1ind = where(bad(npixl,*) GT 0, bcnt)
                b1ind = where(bad(npixl,*) eq 0, gcnt)
               ;IF debug THEN print,'good bad=', gcnt, bcnt
        bad (npixl,istartnoe:istopnoe)= bads(npixl, istartnoe:istopnoe)
               ;for i=0,tcnt-1 do IF debug THEN print,time(tindx(i)),s(1,tindx(i)),bad(1,tindx(i))
                b1ind = where(bad(npixl,*) GT 0, bcnt)
                b1ind = where(bad(npixl,*) eq 0, gcnt)
               ;IF debug THEN print,'good bad=', gcnt, bcnt
        (*pid).dr_init = 0
        (*pid).drift_cor=1
        lid.drift(npixl) = 1
        tptr = (*pid).dr_tps
        vars = (*tptr).vars
        pars = (*tptr).flags
        pars(*, npixl) = dr_param
        IF debug THEN help,vars,pars
        pia_set_tsp, lid, npixl, vars, pars
        ptr_free, tptr
        (*pid).dr_tps = ptr_new({vars:vars, flags:pars})

         END   

	 "QUIT": BEGIN
                save_ptr = lid.save_bad
                savebad = *save_ptr
                bad = savebad
	 	WIDGET_CONTROL, event.top, /DESTROY
	 END

         "HELP": BEGIN
           IF !VERSION.OS NE 'vms' THEN $
             spawn, "$w3net -remote 'openURL('$PIA_ERD2MAP_HTML'/p32_spix.html)' &" $
           ELSE $
             spawn, "w3net -remote 'openURL('$PIA_ERD2MAP_HTML'/p32_spix.html)'",/nowait
         END

	
	 ELSE:

	ENDCASE

 	RETURN
	END
	

	
PRO PIA_erd2map_spix, pid, GROUP=group

@phterd
@common_all
@phtfilter

COMMON e2m_spix_locids, lid
 
        debug = 0
        vptr = (*pid).dr_wps
        vstr = *vptr
        (*pid).dr_tps = ptr_new(vstr) 
        pvars = vstr.vars
        vars = vstr.flags
        IF debug THEN help,pvars,vars



        pix = (*pid).pix
        npixl = FIX((*pid).pix) - 1
        spix = STRTRIM(pix, 2)
        IF debug THEN print,'npix=', npixl
        IF debug THEN help,pvars

        avar = strarr(4) & bvar = strarr(4) & cvar = strarr(4)
        varsa = intarr(4) & varsb = intarr(4) & varsc = intarr(4)
        pia_set_pars, npixl, pvars, vars, avar, bvar, cvar, varsa, varsb, varsc

        (*pid).self_cal = 1
	lid = {sinfrom:0L, sinto:0L, pix:pix, rsize: (*pid).dr_to, dr_param:intarr(12),  $
               dg_imple: intarr(npix), $
	       self_cal:0L, varpar:0L, dr_pix:0L, drift:intarr(npix), rebin:0L, vars:pvars,  $
               flags: vars, a:lonarr(4), b:lonarr(4), c:lonarr(4), $
               af:lonarr(4), bf:lonarr(4), cf:lonarr(4),  $
               avar: avar, bvar:bvar, cvar:cvar, $
               varsa:varsa, varsb:varsb, varsc:varsc, save_bad:ptr_new(-1)}

        e2map = *dptr
        save_ptr = ptr_new(e2map.bad)
        lid.save_bad = save_ptr 
        
        IF debug THEN print,'enter'
        IF not KEYWORD_SET(group) THEN group=!PIA_TOP_WID
	mod_id = WIDGET_BASE(TITLE="Inspect signals per pixel",  $ 
                ;/COLUMN ,/MODAL, GROUP=group)
                 /COLUMN )

        id_base =  WIDGET_BASE(mod_id, /COLUMN)

       ;id_1 = WIDGET_BASE(id_base, /ROW, /ALIGN_CENTER )

       ;id_0 = WIDGET_BASE(id_base, COL=2, space=30, /align_center,/grid_layout)                  
        id_0 = WIDGET_BASE(id_base, /ROW , /align_center,/grid_layout)                  


        id_1 = WIDGET_BASE(id_0, /ROW)  ;COL 1                  
        IF debug THEN print,'id1=',id_1,npix
        xs = 60
        IF npix GT 4 THEN xs = 100
        junk = WIDGET_LABEL(id_1, VAL='Pixel ')
        slider = WIDGET_SLIDER(id_1, MINIMUM=1, MAXIMUM=strtrim(npix,2), $
                                UVAL="SELECT", VAL=pix, XSIZE=xs)

	
        id_2 = WIDGET_BASE(id_0, /ROW, ypad=15)            
	dg2  = WIDGET_BUTTON( id_2, VAL="  Deglitch  ", $
                              UV="DEGLCH")
;       auto = WIDGET_BUTTON(dg2, VAL="Auto", UV="AUTO_DG")
;       manual = WIDGET_BUTTON(dg2, VAL="Select area graphically", $
;                              UV="SEL_DG")
;       desel  = WIDGET_BUTTON(dg2, VAL="Deselect area graphically", $
;                              UV="REST_DG")
;       WIDGET_CONTROL,desel, sensitive=0
 
	id_1 = WIDGET_BASE(id_base, /ROW )
	lid.rebin = cw_bgroup(id_1,  $ 
			/EXCLUSIVE, /ROW, $
                        LABEL_LEFT='Display Rebin:', $
			SET_VALUE = '0', $
			UVALUE = 'REBIN', $
			["No rebin", $
			 "per ramp", $
                         "per chopper"])

	junk = WIDGET_BUTTON(id_1, VAL="Display Signals",UVAL="DISP", /align_center)
	
	id_0 = WIDGET_BASE(id_base, /COL,/FRAME)
	junk = WIDGET_LABEL(id_0, VAL='Drift Fitting Parameters', $)
                         FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')

	id_1 = WIDGET_BASE(id_0, /COL)
        junk = WIDGET_LABEL(id_1, VAL='          A       Free          B      Free       C      Free',  $
                         FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')

        id_2 = WIDGET_BASE(id_1, /ROW)
        col0 = WIDGET_BASE(id_2, /col,space=18)
        junk = WIDGET_LABEL(col0, VAL = 'Beta1',  $
                         FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')
        junk = WIDGET_LABEL(col0, VAL = 'Tau1',  $
                         FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')
        junk = WIDGET_LABEL(col0, VAL = 'Beta2',  $
                         FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')
        junk = WIDGET_LABEL(col0, VAL = 'Tau2',  $
                         FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')
        col1 = WIDGET_BASE(id_2, /col, space=0)
        lid.a(0) = WIDGET_TEXT( col1, VAL=avar(0), UVAL="JUNK", /editable, xsize=10)
        lid.a(1) = WIDGET_TEXT( col1, VAL=avar(1), UVAL="JUNK", /editable, xsize=10)
        lid.a(2) = WIDGET_TEXT( col1, VAL=avar(2), UVAL="JUNK", /editable, xsize=10)
        lid.a(3) = WIDGET_TEXT( col1, VAL=avar(3), UVAL="JUNK", /editable, xsize=10)
   
        col1_1 = WIDGET_BASE(id_2  , /col, /nonexclusive,/align_center) 
        FOR i=0, 3 DO BEGIN
            lid.af(i) = WIDGET_BUTTON(col1_1,VAL=' ', UVAL="FREE1")
            IF varsa(i) EQ 1 THEN $
              WIDGET_CONTROL, lid.af(i), /set_button   
        ENDFOR
 
        col2 = WIDGET_BASE(id_2, /col, space=0)
        lid.b(0) = WIDGET_TEXT( col2, VAL=bvar(0), UVAL="JUNK", /editable, xsize=10, ysiz=1)
        lid.b(1) = WIDGET_TEXT( col2, VAL=bvar(1), UVAL="JUNK", /editable, xsize=10, ysiz=1)
        lid.b(2) = WIDGET_TEXT( col2, VAL=bvar(2), UVAL="JUNK", /editable, xsize=10, ysiz=1)
        lid.b(3) = WIDGET_TEXT( col2, VAL=bvar(3), UVAL="JUNK", /editable, xsize=10, ysiz=1)
   
        col2_1 = WIDGET_BASE(id_2, /col, /nonexclusive,/align_center) 
        FOR i=0, 3 DO BEGIN
            lid.bf(i) = WIDGET_BUTTON(col2_1,VAL=' ', UVAL="FREE2")
            IF varsb(i) EQ 1 THEN $
              WIDGET_CONTROL, lid.bf(i), /set_button   
        ENDFOR
 
        col3 = WIDGET_BASE(id_2, /col, space=0)
        lid.c(0) = WIDGET_TEXT( col3, VAL=cvar(0), UVAL="JUNK", /editable, xsize=10)
        lid.c(1) = WIDGET_TEXT( col3, VAL=cvar(1),UVAL="JUNK",  /editable, xsize=10)
        lid.c(2) = WIDGET_TEXT( col3, VAL=cvar(2), UVAL="JUNK", /editable, xsize=10)
        lid.c(3) = WIDGET_TEXT( col3, VAL=cvar(3), UVAL="JUNK", /editable, xsize=10)
        
        col3_1 = WIDGET_BASE(id_2, /col, /nonexclusive,/align_center) 
        FOR i=0, 3 DO BEGIN
            lid.cf(i) = WIDGET_BUTTON(col3_1,VAL=' ', UVAL="FREE3")
            IF varsc(i) EQ 1 THEN $
              WIDGET_CONTROL, lid.cf(i), /set_button   
        ENDFOR

        junk = WIDGET_LABEL(id_1, /align_center, $
                 VAL=' Note: A maximum of 6 free parameters are allowed',  $
                         FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')
        id_3 = WIDGET_BASE(id_1, /ROW, /align_center, space=20)
	junk = WIDGET_BUTTON( id_3, VAL=" Reset  ",  UV="RESET")
	junk = WIDGET_BUTTON( id_3, VAL=" Reset to Defaults ",  UV="RESET_DEF")

        id_2 = WIDGET_BASE(id_0, /ROW)
        rstr = 'Raster Size: { ' + STRTRIM((*pid).dr_to, 2) + ' }  '
	junk = WIDGET_LABEL(id_2, VAL=rstr)
	junk = WIDGET_LABEL(id_2, VAL='Select from raster')
	lid.sinfrom = WIDGET_TEXT( id_2, VAL=STRTRIM((*pid).dr_fr,2), $
			UV="JUNK", /EDITABLE, XSIZE=4, YSIZE=1 )

	junk = WIDGET_LABEL(id_2, VAL=' to raster')
	lid.sinto = WIDGET_TEXT( id_2, VAL=STRTRIM((*pid).dr_to,2), $
		UV="JUNK", /EDITABLE , XSIZE=4, YSIZE=1  )	

        id_1 = WIDGET_BASE(id_0, /COL, /align_center)
        junk = WIDGET_LABEL(id_1, VAL=' Options for Self Calibration ',  $
                         FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')
	id_0 = WIDGET_BASE(id_1, /COL)
	lid.self_cal = cw_bgroup(id_0,  $ 
			/EXCLUSIVE, COLUMN=2, $
			SET_VALUE = (*pid).self_cal, $
			UVALUE = 'SELFCAL', $
			["Full range", $
			 "Composite "])
			 
	id_2 = WIDGET_BUTTON( id_0, VAL="  Re-Calculate Parameters  ",  $
                              UV="DRIFT", /align_center)


	id_0 = WIDGET_BASE(id_base, /ROW)    
	id_1 = WIDGET_BUTTON( id_0, VAL="  Accept  ", UV="ACCEPT" )

	id_1 = WIDGET_BUTTON( id_0, VAL="  Close   ", /menu)        
	id_2 = WIDGET_BUTTON( id_1, VAL="Are You Sure?", UV="QUIT" )

	id_1 = WIDGET_BUTTON( id_0, VAL="  Help    ", UV="HELP" )

	WIDGET_CONTROL, mod_id , /realize, set_UVALUE=pid	
	Xmanager, /NO_BLOCK, "PIA_ERD2MAP_SPIX", mod_id , GROUP=group

	RETURN
	END
	
