;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PIA P32 Processing Tools Software.              !
;! Designed by: Nanyao Lu at IPAC (lu@ipac.caltech.edu)                        !
;!              Bernhard Schulz at Vilspa (bschulz@iso.vilspa.esa.es)          !
;! Developed by: Iffat Khan at IPAC (irk@ipac.caltech.edu)                     !
;!                                                                             !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_ERD2MAP_XPLOT
;
; PURPOSE:
;       This routine starts a PIA graphical window to display P32  
;       processed data. 
;
; DESCRIPTION:
;       The GUI consists of the PIA specific plotting window.
;
;      1. Display of selected pixel data                                        
;          Depending upon the input option 'avg_pts', either the whole 
;          data is plotted without rebining, or the following
;          rebin per ramp: plot by taking average of NNDR (read from COMP.STAT)
;          data points
;          rebin per chopper: plot by taking average of data points between 2     
;          chopper sweeps.
;          The data is plotted in red color, slew data in blue, and BAD data
;          in yellow colors. The chopper sweeps are plotted as white dotted
;          lines.
;       2. Control buttons    
;          All widget buttons on this gui are self explanatory.
;          (Please see the routine PIA_XPLOT.PRO in PIA for more details) 
;          All data selection is done by using the left mouse button e.g.
;          'Activate zooming' : drag the left mouse button to zoom the desired area
;          'Cursor position'  : Click the left mouse button atthe desired data
;                               location. The (x,y) values will be printed in the 
;                               lower left corner.
;          
;
; CATEGORY:
;       PIA - P32     
;
; CALLING SEQUENCE:
;       pia_erd2map_xplot,ifine1,ifine2,ifine,ipix,avg_pts
;
; INPUTS:
;       ifine1  : start of the desired fine pointing range
;       ifine2  : end of the desired fine pointing range
;       ifine   : selection of plot x-axis. if ' ' then plot is versus READOUT
;                 if 'time' then plot is versus TIME.
;       ipix   :  IDL convention for desired pixel number (i.e pixel # - 1)
;       avg_pts:  option to determine the rebinning method               
;                 avg_pts = 0  No rebinning
;                 avg_pts > 0  Rebin per ramp. avg_pts is the number of data
;                              points per ramp obtained from Compact Status
;                 avg_pts < 0  Rebin per chopper  
;
; OPTIONAL INPUTS:
;       None.
;
; OUTPUTS:
;       None.
;
; KEYWORDS:
;
; CALLED ROUTINES:
;       PIA_XPLOT    
;       REBIN_CHP (Internal procedure)
;       REBIN_RAMP (Internal procedure)
;       STARTENDPOINT
;
; CALLED FROM:    
;       PIA_ERD2MAP_SPIX      
;
; COMMON BLOCKS:
;       DRIFT_STRUCT
;
; PROCEDURE:
;       @colour_indices
;       @common_driftpar
;
; SIDE EFFECTS:
;       None.
;
; RESTRICTIONS:
;       imposed by PIA_XPLOT interface. This widget cannot be created
;       as a Modal widget.     
;
; MODIFICATION HISTORY:
;	original design, Version 1.1 IRK        Jan 08, 2001
;       Version 1.35                            Mar 08, 2001
;       Version 1.40                            Mar 16, 2001
;       Version 1.50                            Mar 30, 2001
;-
;-----------------------------------------------------------------------
;

PRO rebin_ramp, avg, xtime, sig, msk, ptf



debug = 0
npts = n_elements(sig)
fout = 0
sout = 0.
tout = 0.
nt = 0

FOR i=0L, npts-1, avg DO BEGIN
  nt = nt + 1
  j = i+avg-1
  if j GT npts-1 THEN j= npts - 1
  tmps = sig(i:j)
  tmpmsk = msk(i:j)
  tmptim = xtime(i:j)
  ntim = n_elements(tmptim)
  tmpfpt = ptf(i:j)
  weight = intarr(ntim)
  weight(*) = 1
  
  bind = WHERE(tmpmsk GT 0, bcnt)
  IF bcnt GT 0 THEN weight(bind) = 0
  IF bcnt NE ntim THEN BEGIN
    sout = [sout, total(weight * tmps) / total(weight)]
    tout = [tout, (xtime(i) + xtime(j))/2.] 
    fpind = WHERE(tmpfpt EQ 1, fcnt)
    IF fcnt GT avg/2 then fout = [fout, 1] ELSE fout = [fout, 0]
  ENDIF
ENDFOR

xtime = temporary(tout(1:*)) 
sig   = temporary(sout(1:*)) 
ptf   = temporary(fout(1:*)) 

msk = intarr(n_elements(xtime))
msk(*) = 0


RETURN
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


PRO rebin_chp , chlim, xtime, echp, sig, msk, ptf



debug = 0
npts = n_elements(sig)
fout = 0
sout = 0.
tout = 0.
nt = 0

nii = n_elements(echp)
itest=lonarr(nii)   
itest(1:nii-1)=echp(1:nii-1)-echp(0:nii-2)
iii=where(abs(itest) eq 1 or abs(itest) eq chlim, niii)

IF debug THEN print,'nii',niii
avgch = iii(1:niii-1) - iii(0:niii-2)
avgch = [iii(0), avgch]

navg = n_elements(avgch)
IF debug THEN print,'navg',navg
IF debug THEN print,avgch(0), avgch(navg-1)

i = 0
FOR k = 0L, navg -1 DO BEGIN
  nt = nt + 1
  avgp = avgch(k)
  j = i+avgp-1
  if j GT npts-1 THEN j= npts - 1
  tmps = sig(i:j)
  tmpmsk = msk(i:j)
  tmptim = xtime(i:j)
  ntim = n_elements(tmptim)
  tmpfpt = ptf(i:j)
  weight = intarr(ntim)
  weight(*) = 1
  
  bind = WHERE(tmpmsk GT 0, bcnt)
  IF debug THEN print, 'i j bcnt avg',i,j,ntim,bcnt,avgp
  IF bcnt GT 0 THEN weight(bind) = 0
  IF bcnt NE ntim THEN BEGIN
    sout = [sout, total(weight * tmps) / total(weight)]
    tout = [tout, (xtime(i) + xtime(j))/2.] 
    fpind = WHERE(tmpfpt EQ 1, fcnt)
; if xtime(i) gt 30 and xtime(j) lt 40 then IF debug THEN print,'fcnt ntim',ntim,fcnt, xtime(i),xtime(j)
    IF fcnt GT avgp/2 then fout = [fout, 1] ELSE fout = [fout, 0]
  ENDIF
  i = j + 1
ENDFOR

xtime = temporary(tout(1:*)) 
sig   = temporary(sout(1:*))
ptf   = temporary(fout(1:*))

msk = intarr(n_elements(xtime))
msk(*) = 0


RETURN
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro pia_erd2map_xplot,ifine1,ifine2,ifine,ipix,avg_pts


COMMON drift_struct, drift_struct

@common_driftpar
@colour_indices

if NOT KEYWORD_SET(select) THEN select=0
xtit='READOUT'
if(ifine eq 'time')then begin
inumplot=0
xtit='TIME (s)'
endif ; ifine eq 'time'
;
@common_all

debug = 0
if(ipix eq 0)then tit=tdtnam+' '+iseq+' PIXEL 0'
if(ipix eq 1)then tit=tdtnam+' '+iseq+' PIXEL 1'
if(ipix eq 2)then tit=tdtnam+' '+iseq+' PIXEL 2'
if(ipix eq 3)then tit=tdtnam+' '+iseq+' PIXEL 3'
if(ipix eq 4)then tit=tdtnam+' '+iseq+' PIXEL 4'
if(ipix eq 5)then tit=tdtnam+' '+iseq+' PIXEL 5'
if(ipix eq 6)then tit=tdtnam+' '+iseq+' PIXEL 6'
if(ipix eq 7)then tit=tdtnam+' '+iseq+' PIXEL 7'
if(ipix eq 8)then tit=tdtnam+' '+iseq+' PIXEL 8'

if(array eq 'c100')then chswlim=12
if(array eq 'c200')then chswlim=6

orig = where(ireduce eq 0, norig) ;list of transmitted points in EXPAND32
print,'norig =',norig

if(norig ne nptin)then begin
  print,'DEGLITCHER aborting:  number of good points in expanded data not NPTIN'
  return  
endif ; norig ne nptin

ftrans = fpoint(orig)



;find start and end points in original sampling IPLOT1 IPLOT2
startendpoint,ifine1,ifine2,$
  irasterstart,irasterstop,iplot1,iplot2,$
  irasterstarte,irasterstope,ieplot1,ieplot2
xtime = time(iplot1:iplot2)
IF debug THEN help,xtime
np = n_elements(xtime)
sigin = REFORM(s(ipix, iplot1:iplot2), np)
badin = REFORM(bad(ipix,iplot1:iplot2), np)
IF debug THEN help,sigin
IF debug THEN print,'sigin ',min(sigin),max(sigin)
fpts = ftrans(iplot1:iplot2)
echp = step(iplot1:iplot2)
echtim = xtime


IF debug THEN print,'avg',avg_pts
dr_cor = ndriftsolve(ipix)
IF debug THEN print,'dr_cor',dr_cor
IF dr_cor EQ 0 THEN BEGIN
  np = n_elements(xtime)
  bad_data = REFORM(bad(ipix,iplot1:iplot2), np)
  IF avg_pts GT 0 THEN rebin_ramp, avg_pts, xtime, sigin, bad_data, fpts
  IF avg_pts LT 0 THEN rebin_chp , chswlim, xtime, echp, sigin, bad_data, fpts
  good = WHERE(bad_data EQ 0, ngood)
  badd  = WHERE(bad_data GT 0, nbad)   
  IF nbad GT 0 THEN BEGIN
   bads = sigin(badd)
   IF debug THEN print,'bads', min(bads),max(bads)
   badx = xtime(badd)
   IF ngood EQ 0 THEN $
      pia_xplot,badx, bads, psym=1, color=yellow,   $
        ytit='SIGNAL (V/s)', xtit='TIME(s)', legend='Signal in (with glitches)', $
        title='Signal - Pixel '+STRTRIM(ipix+1,2)
  ENDIF

  IF debug THEN print,'ngood',ngood
  IF ngood GT 0 THEN BEGIN
    siginf = sigin(good)
    yrtop = max(siginf)
    yrbot = min(siginf)
    IF debug THEN print,'max good', yrtop,yrbot

    xax = xtime(good)
    fps = fpts(good)    
    finep = where(fps eq 1, nfp)
    IF debug THEN print,'fine',nfp
    IF nfp GT 0 THEN BEGIN
      x2 = xax(finep)
      y2 = siginf(finep)
      pia_xplot,x2, y2, psym=1, color=red,   $
                ytit='SIGNAL (V/s)', xtit='TIME(s)', legend='Signal in (Fine pointing)', $
                title='Signal - Pixel '+STRTRIM(ipix+1,2)
    ENDIF

    IF nbad GT 0 AND avg_pts EQ 0 THEN BEGIN
      bdum = where(bads ge yrbot and bads le yrtop, bdcnt)
      IF debug THEN print,'nbad cnt',nbad, bdcnt       
      IF bdcnt GT 0 THEN BEGIN
        xbad = badx(bdum)
        sbad = bads(bdum)
        IF debug THEN print,'lim',min(xbad),max(xbad)
        pia_xplot,xbad, sbad ,psym=3,  color=yellow, /overplot,$
          legend='Signal in with Glitches'
      ENDIF
    ENDIF

    slewp = where(fps eq 0, nslew)
    IF debug THEN print,'nslew',nslew
    IF nslew GT 0 THEN BEGIN
      x1 = xax(slewp)
      y1 = siginf(slewp)
      pia_xplot,x1, y1 ,psym=1,  color=blue, /overplot,$
        legend='Signal in (Slew)'

    ENDIF
  ENDIF
  goto, plot_ech
ENDIF

;Drift Correction has been applied
IF debug THEN print,'points',nptin,neptin

xtime=drift_struct.time
fpoint=drift_struct.fpoint
msk=drift_struct.msk(ipix,*)
itrans=drift_struct.itrans
modelfit=drift_struct.modelfit(ipix,*)
respcorr=drift_struct.respcorr(ipix,*)
signal=drift_struct.signal(ipix,*)
maskout=drift_struct.maskout(ipix,*)
ech=drift_struct.ech
badpix = bad(ipix,*)
IF debug THEN help,maskout,badpix,msk

ii=where(itrans eq 1,nii)

IF debug THEN print,'no of transmitted points', nii       

;
mpfit=modelfit(ii)               ; these arrays are not IF debug THEN help in common
rpfit=respcorr(ii)*modelfit(ii)
mskbad=msk(ii)
mk = maskout(ii)
badarr = badpix
IF debug THEN help,badarr,mskbad
IF debug THEN print,'bads',nii,n_elements(badpix)
IF nii EQ N_ELEMENTS(badpix) THEN BEGIN
 ibad = WHERE(badarr NE 0, nbad)
 IF nbad GT 0 THEN $
   mskbad(ibad) = 1
ENDIF ELSE $
 PRINT, 'WARNING,WARNING, nr of transmitted points NE arrays '


fpn=fpoint(ii)
signl=signal(ii)
IF debug THEN help,fpn,mpfit

xtime = xtime(iplot1:iplot2)
mfit = mpfit(iplot1:iplot2)
rfit = rpfit(iplot1:iplot2)
fpts = fpn(iplot1:iplot2)
sigin = signl(iplot1:iplot2)
echp = ech(iplot1:iplot2)
 msk = mskbad(iplot1:iplot2)
fitmsk = mk(iplot1:iplot2)

;IF debug THEN help,sigin,xtime,echp,fpts,mfit,rfit
;sind=where(fpts eq 0, scnt)
;IF debug THEN print,'scnt',scnt
;IF debug THEN print,xtime(sind(0:150))


echtim = xtime
xsig = xtime
good = WHERE(msk EQ 0, ngood)

badd = WHERE(msk GT 0, nbad)  
IF nbad GT 0 THEN BEGIN
  bads = sigin(badd)
  badx = xtime(badd)
ENDIF

badfit = WHERE(fitmsk GT 0, nbfit)
IF debug THEN print,'nbfit=',nbfit
IF nbfit GT 0 THEN BEGIN
 badfs = sigin(badfit)
 badfx = xtime(badfit)
 IF debug THEN print,'badx',min(badfx),max(badfx)
ENDIF

mpfit = mfit(good)
rpfit = rfit(good)
xfit = xtime(good)
IF debug THEN print,'good',ngood,min(xfit),max(xfit)

;ech=step
echtim = xtime

IF avg_pts NE 0 THEN BEGIN           ;Rebin signal
  IF avg_pts GT 0 THEN rebin_ramp, avg_pts, xsig , sigin, msk, fpts
  IF avg_pts LT 0 THEN rebin_chp , chswlim, xsig, echp, sigin, msk, fpts
  good = WHERE(msk EQ 0, ngood)
  IF debug THEN print,'rebin ngood',ngood
  IF ngood GT 0 THEN BEGIN
    siginf = sigin(good)
     siginf = sigin
     fps = fpts
  ENDIF

ENDIF ELSE BEGIN
  siginf = sigin(good)
  xsig = xtime(good)
  fps = fpts(good)    
ENDELSE  
IF debug THEN help,siginf,xsig,fps
IF debug THEN print,'sigin ',min(siginf),max(siginf)
;do the actual plots

 yrtop= max([rpfit,siginf,mpfit])/(0.8)
 yrbot= min([rpfit,siginf,mpfit])

 pia_xplot,xfit,rpfit,xtitle=xtit,$
 ytitle='SIGNAL (V/s)',xstyle=1,ystyle=1, linestyle=5,   $
 title='Drift P32 results - Pixel '+STRTRIM(ipix+1,2), $
 legend='Illumination level'
;
;
  slewp = where(fps eq 0, nslew)
  IF nslew GT 0 THEN BEGIN
    x1 = xsig(slewp)
    y1 = siginf(slewp)
    pia_xplot,x1, y1,color=blue , /overplot, psym=1, $
       legend='Signal in (Slew)'
  ENDIF

  finep = where(fps eq 1, nfp)
  IF nfp GT 0 THEN BEGIN
    x2 = xsig(finep)
    y2 = siginf(finep)
    pia_xplot,x2, y2,color=red, /overplot, psym=1, $
       legend='Signal in (Fine pointing)'
  ENDIF

  pia_xplot,xfit,mpfit,color=green, /overplot, $
    legend='Model fit' ; green (modelfit)
;
    IF nbad GT 0 AND avg_pts EQ 0 THEN BEGIN
      bdum = where(bads ge yrbot and bads le yrtop, bdcnt)
      IF debug THEN print,'nbad cnt',nbad, bdcnt
      IF bdcnt GT 0 THEN BEGIN
        xbad = badx(bdum)
        sbad = bads(bdum)
        IF debug THEN print,'lim',min(xbad),max(xbad)
        pia_xplot,xbad, sbad ,psym=3,  color=yellow, /overplot,$
          legend='Signal in with Glitches'
      ENDIF
    ENDIF
    IF nbfit GT 0 AND avg_pts EQ 0 THEN BEGIN
             pia_xplot,badfx, badfs ,psym=1, symsize=.5,  color=cyan  , /overplot,$
                       legend='Signal in with No Fit'
    ENDIF

; plot vertical lines at start of each chopper sweep using the step array ECH:
; this makes use of the fact that the chopper sweeps always begin with a
; transmitted read
plot_ech:

  nii = n_elements(echtim)  
  itest=lonarr(nii)   
  itest(1:nii-1)=echp(1:nii-1)-echp(0:nii-2)
  if(array eq 'c100')then chswlim=12
  if(array eq 'c200')then chswlim=6
  iii=where(abs(itest) ge chswlim,niii)
  IF debug THEN print,'nchop',niii
  if(niii gt 0)then begin
    xpc=echtim(iii(0:niii-1)) ; plot versus time
    lin = (yrbot+yrtop) / 2
    linarr = FLTARR(niii) + lin
    pia_xplot, xpc, linarr , linarr, psym=3, yerrstyle=1, /overplot,$
      legend='Chopper sweeps' 
  endif ; niii gt 0


return
end
