pro simul32,isimul32
;
; For purposes of simulation
; overwrites data ES in E32 common block and S in SIGNAL common blocks
; with simulated data while keeping attitude data constant.
;
; Should be called before any of the manipulations are done like FINDRMS,
; DG1, RELATIVISE etc but after signal creation by SIGCREATE, after
; correction of the chopper position by RCHSET, and after POINT.
;
; The concept of SIMUL32 is to first expand the input data to the full time 
; sampling seen onboard using EXPAND32. Then to overwrite the ES array in the 
; E32 common block with the illumination. The interpolations necessary to 
; handle the illumination are done with a planar fit to the nearest grid 
; points on a model sky map created by a subroutine CREATESKY (see below). 
; Then process the illumination (sky*vignetting) through a drift simulation 
; program (including detector starting state). (We consider the short term
; drifts to be a function only of the illumination and not of the response, so
; perform this step before the flat field). As an option glitches can be added 
; beforehand for proton hits which affect responsivity. (ie the glitches with 
; tails). Then multiply by a slowly evolving time dependent flat field. Then 
; add the relativisation effects according to a template integration ramp 
; shape (which of course deals with saturation too) and maybe some less
; well understood factor specified by ripple factors RIPPLEAMP(npix,ninramp-1)
; and RIPPLECONST(npix,ninramp-1). Then add gamma (delta 
; function) glitches according to some power spectrum, white noise according 
; to the shot noise illumination (not response), a constant white
; noise due to the detector electronics, and the dark current. Then digitise 
; the signal. Then copy back a subset of the ES array to the S array in the 
; SIGNAL common block.
;
; At present only illumination, short term drift and white noise is 
; implemented.
;
; One purpose of this program is to test the various algorithms for removing
; artifacts. Another is to create realistic noiseless beams processed through 
; the P32 reduction chain for use in deconvolution applications, taking
; into account the pointing pattern and the absence of data under the
; BAD mask.
;
; input:
;
;       ISIMUL32 = 1 to simulate map of point source of unity integrated flux 
;                    density at the field centre on a constant background of 
;                    unity with no noise no vignetting no detector drift
;                = 0 to use parameters set in routine for the simulation
;
;       from NAMES   common block:
;       CFILTER (integer) = either 50,60,70,90,100,105,120,135,160,180,200
;       CALDIR       string denoting CALG directory (needed by CREATEBEAM)
;                    eg '/d12/vlk/cgabriel/pia_7.3/calg/'     
;       from STEERIN common block: ARRAY, NPTIN, NINRAMP
;       from STEER1  common block: npix,pixsize
;       from TIMER   common block: dtime
;       from MAPPER  common block: sgridy,sgridz,nygrid,nzgrid,ygrid,zgrid
;       from E32     common block: es,eytrue,eztrue,etime,echop,ebad,neptin
;       from SIGNAL  common block: rs,s,badin,rd
;
; output:
;        GAMMAEXP (common MOREPAR) 3rd detector parameter
;        ZEROBIAS (common MOREPAR) 4th detector parameter
;        S        (common block SIGNAL) overwritten with simulated data.
;                       NB all other arrays in SIGNAL, are left unchanged,
;                       even RMS, BAD.
;       ES        (common block SIMUL): 1D array of overwritten data dimension
;                 NEPTIN
;       EYTRUE,EZTRUE,ETIME,ECHOP,EBAD,NEPTIN (other arrays from common_e32)
;  NPAR    (common block CSKY) max. number of parameters for each source 
;                              = n_elements(sourceparin(0,*)) (normally 8)
;  SKY     (common CSKY)
;  SKYBEAM (common CSKY)
;  SKYPLUS (common CSKY)
;  NYGRIDSIM, NZGRIDSIM (common CSKY): dimensions of SKY and SKYBEAM
;  SGRIDYSIM,SGRIDZSIM (common CSKY) sampling interval (arcsec) for 
;                                           SKY, BEAM, SKYBEAM, SKYPLUS 
;  YGRIDSIM,ZGRIDSIM (common CSKY): offset in arcsec of positions in 
;                     each coordinate of SKY, SKYBEAM (1D arrays dimension 
;                     NYGRIDSIM or NZGRIDSIM)
;  SOURCEPAROUT (common CSKY) 2D real dimension NSOURCE,0<IPAR<=NPAR-1
;              IPAR 0: source type 
;                      = 1. for point source
;                      = 2. for 2D truncated gaussian
;                      = 3. for 2D truncated exponential
;              IPAR 1: actual Y offset in arcsec from map centre (CREATESKY
;                      shifts the source centre from the position specified in
;                      SOURCEPARIN to the centre of the nearest grid point on
;                      the simulated array. This is to speed up computations
;                      (avoid DO LOOPS) to allow use of CREATESKY and SIMUL32 
;                      in interative optimisation applications, and to avoid
;                      ambiguities when input position is on a boundary of
;                      a pixel in SKY     
;              IPAR 2: actual Z offset in arcsec from map centre
;              IPAR 3: integrated flux density of source in V/s within
;                      boundary of SKY array
;              IPAR 4: PA of major axis of source in degrees antclockwise from
;                      top 
;              IPAR 5: FWHM of major axis in arcsec
;              IPAR 6: FWHM of minor axis in arcsec 
;              IPAR 7: maximum diameter of source in both dimensions in units 
;                      of FWHM 
;
;  BEAM          (common BM)
;  NYB, NZB      (common BM): dimensions of BEAM 
;  YGRIDB,ZGRIDB (common BM): offset in arcsec of positions in each coordinate
;                             of BEAM (1D arrays dimension NYB or NZB)      
;  BEAMEXTENT    (common BM) if input BEAMEXTENTIN is larger than the 17*7.5 = ;  NPAR    (common block CSKY) max. number of parameters for each source 
;                              = n_elements(sourceparin(0,*)) (normally 8)
;  SKY     (common CSKY)
;  SKYBEAM (common CSKY)
;  SKYPLUS (common CSKY)
;  NYGRIDSIM, NZGRIDSIM (common CSKY): dimensions of SKY and SKYBEAM
;  SGRIDYSIM,SGRIDZSIM (common CSKY) sampling interval (arcsec) for 
;                                           SKY, BEAM, SKYBEAM, SKYPLUS 
;  YGRIDSIM,ZGRIDSIM (common CSKY): offset in arcsec of positions in 
;                     each coordinate of SKY, SKYBEAM (1D arrays dimension 
;                     NYGRIDSIM or NZGRIDSIM)
;  SOURCEPAROUT (common CSKY) 2D real dimension NSOURCE,0<IPAR<=NPAR-1
;              IPAR 0: source type 
;                      = 1. for point source
;                      = 2. for 2D truncated gaussian
;                      = 3. for 2D truncated exponential
;              IPAR 1: actual Y offset in arcsec from map centre (CREATESKY
;                      shifts the source centre from the position specified in
;                      SOURCEPARIN to the centre of the nearest grid point on
;                      the simulated array. This is to speed up computations
;                      (avoid DO LOOPS) to allow use of CREATESKY and SIMUL32 
;                      in interative optimisation applications, and to avoid
;                      ambiguities when input position is on a boundary of
;                      a pixel in SKY     
;              IPAR 2: actual Z offset in arcsec from map centre
;              IPAR 3: integrated flux density of source in V/s within
;                      boundary of SKY array
;              IPAR 4: PA of major axis of source in degrees antclockwise from
;                      top 
;              IPAR 5: FWHM of major axis in arcsec
;              IPAR 6: FWHM of minor axis in arcsec 
;              IPAR 7: maximum diameter of source in both dimensions in units 
;                      of FWHM 
;
;  BEAM          (common BM)
;  NYB, NZB      (common BM): dimensions of BEAM 
;  YGRIDB,ZGRIDB (common BM): offset in arcsec of positions in each coordinate
;                             of BEAM (1D arrays dimension NYB or NZB)      
;  BEAMEXTENT    (common BM) if input BEAMEXTENTIN is larger than the 17*7.5 = 
;                        127.5 arcsec in the CALG files then for option ICAM=0 
;                        BEAMEXTENT is reduced to 127.5 arcsec (C1) or 255 (C2)
;  SPECPAR       (common BM = SPECPARIN): 2 element real array specifying 
;                             spectrum of source assumed in calculating beam:
;         for monocromatic source (ICAM=1) SPECPAR = [0,lamba/um]
;         for planckian source (ICAM=1)    SPECPAR = [1,temperature]
;         for power law source (ICAM=1)    SPECPAR = [2,spectral index +ve red]
;                specpar is only used for ICAM = 1 or 2  
;  ICAM          (common BM = ICAMIN): is 1 if CAM simulated beam to be used
;                             is 0 for real PHT beams from CALG to be used
;                             is 2 for a gaussian beam with FWHM of detector 
;                             pixel
;
;                        127.5 arcsec in the CALG files then for option ICAM=0 
;                        BEAMEXTENT is reduced to 127.5 arcsec (C1) or 255 (C2)
;  SPECPAR       (common BM = SPECPARIN): 2 element real array specifying 
;                             spectrum of source assumed in calculating beam:
;         for monocromatic source (ICAM=1) SPECPAR = [0,lamba/um]
;         for planckian source (ICAM=1)    SPECPAR = [1,temperature]
;         for power law source (ICAM=1)    SPECPAR = [2,spectral index +ve red]
;                specpar is only used for ICAM = 1 or 2  
;  ICAM          (common BM = ICAMIN): is 1 if CAM simulated beam to be used
;                             is 0 for real PHT beams from CALG to be used
;                             is 2 for a gaussian beam with FWHM of detector 
;                             pixel
;
;  common SIMPAR Values of nsource,sourceparinsim,backsim,idovig,idodriftshort,
;                betasim,tausim,gammaexpshootsim,betashootsim,taushootsim,
;                gammaexpshootsim,idownoise,wnoise used in the simulation 
;                for isim32 not 1 Note that the simulation parameters can
;                currently only be changed by editing SIMUL32. Except for
;                algorithm development and testing, the parameters
;                in SIMPAR should be set to the best known parameters
;                for ISOPHOT. eg BETA TAU should be the default values used
;                in DRIFT32, and the noise linked to the NEFD.
;
;common_simpar,isim32,nsourcesim,sourceparinsim,backgsim,idovig,$
;idodriftshort,imodelsim,betasim,tausim,gammaexpsim,zerobiassim,$
;betashootsim,taushootsim,gammaexpshootsim,$
;betaexpsim,betacoeffsim,tauoffsetsim,$
;betacoeffshootsim,betaexpshootsim,tauoffsetshootsim,$
;idownoise,wnoisesim
@common_simpar
;
;common names,tdtnam,iseq,dirnam,caldir,cfilter
@common_names
;
;common steerin,array,nptin,npt,ninramp,imeas,osamp
@common_steerin
;
;common steer1,npix,pixsize,pixpitch,pixoff,nchop,dcy,dry,drz
@common_steer1
;
;common timer,time,dtime ; dtime added 12/4/99
@common_timer
;
;common mapper,sgridy,sgridz,nygrid,nzgrid,ygrid,zgrid,errflag
@common_mapper
;
;common signal,rs,cs,nch,s_in,s,badin,bad,rel,rms,glitch,jf,njf,ninchmax,rd,$
;nkf,kf,nkfmax,irelstart,nchopmed,gbad,npolyrms,nrmsmed,sigmaxfract
@common_signal
;
@common_csky
;common csky,sky,skybeam,skyplus,nygridsim,nzgridsim,ygridsim,zgridsim,$
;sgridysim,sgridzsim,sourceparout,npar
;
@common_bm
;common bm,beam,nyb,nzb,ygridb,zgridb,beamextent,specpar,icam
;
;common e32,es,ireduce,eytrue,eztrue,etime,fpoint,echop,estep,ebad,neptin
@common_e32
;
;common ework,work,work1,work2
@common_ework
;common_morepar,gammaexp,zerobias,betashoot,taushoot,gammaexpshoot,$
;betacoeffshoot,betaexpshoot,tauoffsetshoot,tauoffset,betaexp,betacoeff
@common_morepar
;
isim32=isimul32
;
expand32,1 ; expand data from s into the common_e32 common block
;
if(array eq 'c100')then begin
reqsamp=10.0 ; minimum required sampling for simulated 
                                     ; map
det=1 ; for vignecorr
beamextentin=127.5
endif ; array eq 'c100'
if(array eq 'c200')then begin
det=2 ; for vignecorr
reqsamp=20.0
beamextentin=255.0
endif ; array eq 'c200'
print,'SIMUL32: resamp (arcsec) = ',reqsamp
icompressy=long(sgridy/reqsamp)+1
icompressz=long(sgridz/reqsamp)+1
;
;--------------------SIMULATED SOURCE AND BACKGROUND -----------------------
;
nsource=1
sourceparin=fltarr(nsource,8)
sourceparin(0,0)=1 ; point source
sourceparin(0,1)=0.0 ; offset Y (arcsec)
sourceparin(0,2)=0.0 ; offset Z (arcsec)
sourceparin(0,3)=1.0 ; integrated flux density (v/s)
sourceparin(0,4)=0.0 ; PA of major axis of source (deg anticlockwise)
sourceparin(0,5)=0.0 ; FWHM of major axis in arcsec
sourceparin(0,6)=0.0 ; FWHM of major axis in arcsec
sourceparin(0,7)=3.0 ; max diameter in both dimensions in units of FWHMs
backsim=[0.001,0.0,0.0] ; constant background 0.001 V/s per square arcsec
                        ; gives 1.888 V/s on detector
;------C105 Filter Ceres
sourceparin(0,0)=1 ; point source
sourceparin(0,1)=7.5 ; offset Y (arcsec) (for reqsamp=10.0)
sourceparin(0,2)=0.0 ; offset Z (arcsec)
sourceparin(0,3)=15.0 ; to get same V/s as for for pixel 4 (central)
backsim=[0.0002,0.0,0.0] ; C105 filter Ceres
;------C160 Filter Ceres
;sourceparin(0,0)=1 ; point source
;sourceparin(0,1)=0.0 ; offset Y (arcsec) (for reqsamp=10.0)
;sourceparin(0,2)=0.0 ; offset Z (arcsec)
;sourceparin(0,3)=5.0 ; to get same V/s as for source peak
;backsim=[0.000051,0.0,0.0] ; C160 filter Ceres TDT 57901909 pixel 3
                         ; ca. 15 MJ/ster gives 0.40 V/s on detector pixel
                         ; which is 0.000051 V/s per square arcsec 
if(isim32 eq 1)then begin ; override previous simulation values with
;                             central point source noiseless no vig no drift
;                             leave this to end of SIMULATED SOURCE AND 
;                             BACKGROUND section
print,'SIMUL32: simulation of noiseless point source unity flux density'
nsource=1
sourceparin=fltarr(nsource,8)
sourceparin(0,0)=1 ; point source
sourceparin(0,1)=0.0 ; offset Y (arcsec)
sourceparin(0,2)=0.0 ; offset Z (arcsec)
sourceparin(0,3)=1.0 ; integrated flux density
sourceparin(0,4)=0.0 ; PA of major axis of source (deg anticlockwise)
sourceparin(0,5)=0.0 ; FWHM of major axis in arcsec
sourceparin(0,6)=0.0 ; FWHM of major axis in arcsec
sourceparin(0,7)=3.0 ; max diameter in both dimensions in units of FWHMs
;bk=1.0/(pixsize*pixsize)
bk=0.0
backsim=[bk,0.0,0.0]
                 ; constant unity background ca. 0.000125 V/s per square arcsec
                 ; gives 1.0 V/s on detector (simulation give units
                 ; per pixel area PIXSIZE*PIXSIZE in square arcsec 
;
endif ; isim32 eq 1
;------------------------
sourceparinsim=sourceparin
nsourcesim=nsource
backgsim=backsim
;
icamin=0 ; use calg pht beam
specparin=[0.0,0.0] ; redundant for icamin=0
;
createsky,icompressy,icompressz,beamextentin,specparin,$
icamin,backsim,nsource,sourceparin
;help,skybeam
;print,max(skybeam),min(skybeam)
;-----------------------------------------------------------------------------
;
; we now have sky in SKYBEAM(nygridsim,nzgridsim) sampled at arcsec given
; by arrays YGRIDSIM(nygridsim) and ZGRIDSIM(nzgridsim)
;
; overwrite ES(NPIX,NEPTIN) array in common_e32 according to the pointings
; EYTRUE(NPIX,NEPTIN) and EZTRUE(NPIX,NEPTIN) in arcsec and the chopper 
; position in arcsec ECHOP(NEPTIN)
;
; for each sampled position find nearest pixels on te simulated grid.
;
work=dblarr(neptin)
work1=work
work2=work1
;
for ipix=0,npix-1 do begin
print,'SIMUL32: simulating illumination for pixel ',ipix
;
; ----------------VIGNETTING-------------------------------------------------
idovig=1 ; set to 1 to include vignetting
if(isim32 eq 1)then idovig=0
vig=work
vig(*)=1.0
work(*)=echop(*) ; rjt
if(idovig eq 1)then $
; rjt work(*)=echop(*) ; chopper position in arcsec
vignecorr32,det,ipix,work,vig ; vig is vignetting for pixel ipix (LE 1)
;
; ---------------ILLUMINATION-------------------------------------------------
;
work(*)=eytrue(ipix,*)
findident,ygridsim,work,work1 
; work1 array contains nearest pixel identifier in Y on simulated grid
work(*)=eztrue(ipix,*)
findident,zgridsim,work,work2
; work2 array contains nearest pixel identifier in Z on simulated grid
if(array eq 'c100')then tol=2.0
if(array eq 'c200')then tol=2.0
toly=tol  ; only interpolate if sky position of pixel is more than this from
tolz=toly ; centre of a simulated grid pixel (saves time). toly and tolz
;           should be set to 0.0 to interpolate every data point, to
;           ca. 2 arcsec to interpolate only on slews
;
;help,work1
;print,max(work1),min(work1)
;help,work2
;print,max(work2),min(work2)
;help,vig
;print,max(vig),min(vig)
;help,skybeam
;print,max(skybeam),min(skybeam)
es(ipix,*)=vig*skybeam(work1,work2) ; simulation just using nearest pixels
;help,es
;print,max(es(ipix,*)),min(es(ipix,*))
;
ii1=work1 ; i1,i2 arrays denote nearest simulated grid
jj1=work2 ; pixel for each data point
work1=float(work1) ; otherwise integer type is retained and 
work2=float(work2) ; the eytrue-ygridsim is rounded down to the nearest integer
work1(*)=eytrue(ipix,*)-ygridsim(ii1) ; offsets of data points from 
work2(*)=eztrue(ipix,*)-zgridsim(jj1) ; nearest simulated grid pixel
;
ii=where(abs(work1) gt toly or abs(work2) gt tolz,nii) ; identifiers
;                                of points for which we need interpolation
;
;save,/xdr,work1,work2,ii1,jj1,toly,tolz,file='simul32.xdr'
print,'SIMUL32: calculating illumination with points interpolated = ',nii
if(nii gt 0)then begin
;
for k=0L,nii-1 do begin
i=ii(k)
;
i1=ii1(i) ; i1,j1 is nearest simulated grid pixel to data point i 
j1=jj1(i) ; for pi
testy=eytrue(ipix,i)-ygridsim(i1) ; offsets of data point from nearest pixel
testz=eztrue(ipix,i)-zgridsim(j1)
i2=i1-1
if(testy ge 0)then i2=i1+1
if(i1 eq nygridsim-1)then i2=i1-1
if(i1 eq 0)then i2=i1+1
j2=j1-1
if(testz ge 0)then j2=j1+1
if(j1 eq nzgridsim-1)then j2=j1-1
if(j1 eq 0)then j2=j1+1
yy=[ygridsim(i1),ygridsim(i1),ygridsim(i2)] ; yy,zz are nearest three grid 
zz=[zgridsim(j1),zgridsim(j2),zgridsim(j1)] ; points to data point
ss=[skybeam(i1,j1),skybeam(i1,j2),skybeam(i2,j1)]
p3,yy,zz,ss,a,b,c
es(ipix,i)=vig(i)*(c+a*eytrue(ipix,i)+b*eztrue(ipix,i)) ; interpolated value 
                                                        ; * vignetting
endfor ; i
endif ; nii gt 0
;
;------- DETECTOR RESPONSIVITY DRIFT (SHORT TIME SCALES) -----------------
idodriftshort=1
isimuldriftequiv=1 ; =1 to force parameters of drift model used in the 
;                       simulation to be the default parameters calculated in 
;                       subbroutine DETDEFAULTSETUP
if(isim32 eq 1)then idodriftshort=0
imodelsim=3 ; =1 for Foukes-Schubert =2 for exponential =3 for EXPSHOOT
;imodelsim=imodel
sweep=180.0 ; for both c100 and c200
yprev=eytrue(ipix,0)+(sweep/2.0) ; at start of observation previous pointing
;                                is taken to be at chopper rest position
test=[yprev,0.]
findident,ygridsim,test,identy
zprev=eztrue(ipix,0)
test=[zprev,0.]
findident,zgridsim,test,identz
vigpix,det,xvig
if(array eq 'c100')then sillumprev=xvig(ipix,12)*skybeam(identy(0),identz(0))
if(array eq 'c200')then sillumprev=xvig(ipix,6)*skybeam(identy(0),identz(0))
;print,'identy,identz = ',identy,identz
; sillumprev is illumination prior to start of observation in V/s. 
; This is set to the illumination at the other end of the chopper sweep
; for the purposes of modeling
;
signalprev=sillumprev ; = signalprev for stable detector prior to observation
signalprevshoot=0.0 ; additive constant is zero for detector in equilibrium
;signalprevshoot=1.0 ; as a test
;print,'SIMUL32: signalprev,signalprevshoot,sillumprev = ',$
;signalprev,signalprevshoot,sillumprev
;help,signalprev
;help,sillumprev
;
; note the following beta, tau should be the same as those set in 
; the DRIFT32 procedure to test the drift correction algorithm 
; using simulated data
;
if(imodelsim eq 1)then begin
if(array eq 'c100')then begin 
beta=[0.4,0.4,0.4,0.4,0.4,0.4,0.4,0.4,0.4]
tau=3.0*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0] ; in Volts
zerobias=0.0 ; for simulations use real detector response (without zero bias
;              - and add noise later)
zerobiassim=zerobias
gammaexpsim=0.49*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
betasim=beta
tausim=tau
gammaexpshootsim=gammaexpsim
betashootsim=0.2*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
taushootsim=1.0*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
betaexpsim=0.5*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
betacoeffsim=-0.04*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
tauoffsetsim=10.0*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
gammaexpshootsim=gammaexpsim
gammexpshootsim(*)=1.0
;
betacoeffshootsim=0.0*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
betaexpshootsim=0.0*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
tauoffsetshootsim=0.0*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]

;
endif ; array eq 'c100'
;
if(array eq 'c200')then begin
beta=[0.8056,0.8350,0.8075,0.8716]
tau=[52.34,24.297,21.15,22.72] ; in volts
betasim=beta
tausim=tau
zerobias=0.0
zerobiassim=zerobias
betashootsim=0.2*[1.0,1.0,1.0,1.0]
taushootsim=1.0*[1.0,1.0,1.0,1.0]
betaexpsim=0.5*[1.0,1.0,1.0,1.0]
betacoeffsim=-0.04*[1.0,1.0,1.0,1.0]
tauoffsetsim=10.0*[1.0,1.0,1.0,1.0]
;
betacoeffshootsim=0.0*[1.0,1.0,1.0,1.0]
betaexpshootsim=0.0*[1.0,1.0,1.0,1.0]
tauoffsetshootsim=0.0*[1.0,1.0,1.0,1.0]
;
gammaexpsim=0.89*[1.0,1.0,1.0,1.0] 
gammaexpshootsim=gammaexpsim
;
endif ; array eq 'c200'
endif ; imodelsim eq 1
;
if(imodelsim eq 2)then begin
if(array eq 'c100')then begin 
beta=[0.06,0.26,0.60,0.28,0.05,0.22,0.46,0.55,0.01]
tau=[38.0,24.8,69.9,30.2,27.9,27.6,115.0,30.1,35.7] ; 
betasim=beta
tausim=tau
zerobias=0.0
zerobiassim=zerobias
gammaexpsim=0.49*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
gammaexpsim=gammaexpsim
betashootsim=0.2*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
taushootsim=1.0*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
betaexpsim=0.5*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
betacoeffsim=-0.04*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
tauoffsetsim=10.0*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
gammaexpshootsim=gammaexpsim
gammexpshootsim(*)=1.0
;
betacoeffshootsim=0.0*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
betaexpshootsim=0.0*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
tauoffsetshootsim=0.0*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
;
endif ; array eq 'c100'
if(array eq 'c200')then begin
beta=[0.8056,0.8350,0.8075,0.8716]
tau=[52.34,24.297,21.15,22.72] ; in volts
betasim=beta
tausim=tau
zerobias=0.0
zerobiassim=zerobias
betashootsim=0.2*[1.0,1.0,1.0,1.0]
taushootsim=1.0*[1.0,1.0,1.0,1.0]
betaexpsim=0.5*[1.0,1.0,1.0,1.0]
betacoeffsim=-0.04*[1.0,1.0,1.0,1.0]
tauoffsetsim=10.0*[1.0,1.0,1.0,1.0]
;
betacoeffshootsim=0.0*[1.0,1.0,1.0,1.0]
betaexpshootsim=0.0*[1.0,1.0,1.0,1.0]
tauoffsetshootsim=0.0*[1.0,1.0,1.0,1.0]
;
gammaexpsim=0.89*[1.0,1.0,1.0,1.0] 
gammaexpshootsim=gammaexpsim

endif ; array eq 'c200'
endif ; imodelsim eq 2
;
if(imodelsim eq 3)then begin
;
if(array eq 'c100')then begin 
beta=0.39*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0] 
tau=35.0*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
betasim=beta
tausim=tau
zerobias=0.0
zerobiassim=zerobias
gammaexpsim=0.49*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
betaexpsim=0.5*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
betacoeffsim=-0.04*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
tauoffsetsim=10.0*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
gammaexpshootsim=gammaexpsim
gammaexpshootsim(*)=1.0
betashootsim=1.0-betasim
taushootsim=1.0*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
;
betacoeffshootsim=0.0*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
betaexpshootsim=0.0*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
tauoffsetshootsim=0.0*[1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]
;
endif ; array eq 'c100'
;
if(array eq 'c200')then begin
beta=[0.8056,0.8350,0.8075,0.8716]
tau=[52.34,24.297,21.15,22.72] ; in volts
betasim=beta
tausim=tau
zerobias=0.0
zerobiassim=zerobias
gammaexpsim=0.89*[1.0,1.0,1.0,1.0]
betaexpsim=[0.544,1.258,1.315,0.534]
betacoeffsim=[-0.038,-0.0035,-0.0030,-0.053]
;tauoffsetsim=[6.57,5.59,4.37,6.33] ; 1999
tauoffsetsim=[5.92,5.59,4.37,4.92] ; JUN00
gammaexpshootsim=gammaexpsim ; initialise though not used
betashootsim=1.0-betasim
taushootsim=2.0*[1.0,1.0,1.0,1.0]
;
betacoeffshootsim=0.0*[1.0,1.0,1.0,1.0]
betaexpshootsim=0.0*[1.0,1.0,1.0,1.0]
tauoffsetshootsim=0.0*[1.0,1.0,1.0,1.0]
;
endif ; array eq 'c200'
endif ; imodelsim eq 3
;
if(idodriftshort eq 1)then begin
;
if(isimuldriftequiv eq 1)then simuldriftequiv ; overwrite drift parameters in 
;                                               common SIMPAR with default
;                                               parameters calculated by 
;                                               DETDEFAULTSETUP
gammaexp=gammaexpsim(ipix)
betaexp=betaexpsim(ipix)
betacoeff=betacoeffsim(ipix)
tauoffset=tauoffsetsim(ipix)
gammaexpshoot=gammaexpshootsim(ipix)
betashoot=betashootsim(ipix)
taushoot=taushootsim(ipix)
tauoffsetshoot=tauoffsetshootsim(ipix)
betacoeffshoot=betacoeffshootsim(ipix)
betaexpshoot=betaexpshootsim(ipix)
betawk=betasim(ipix)
tauwk=tausim(ipix)
;
if(imodelsim eq 1)then begin
print,'SIMUL32 FS with beta,tau,gammaexp,zerobias...'
print,'...sillumprev,signalprev'
print,betawk,tauwk,gammaexp,zerobias
print,sillumprev,signalprev
endif ; imodelsim eq 1
if(imodelsim eq 2)then begin
print,'SIMUL32 EXPFUN with beta,tau,gammaexp,zerobias...'
print,'...sillumprev,signalprev'
print,betawk,tauwk,gammaexp,zerobias
print,sillumprev,signalprev
endif ; imodelsim eq 2
if(imodelsim eq 3)then begin
print,'SIMUL32 EXPSHOOT with beta,tau,gammaexp,zerobias...'
print,'...betaexp,betacoeff,tauoffset...'
print,'...betashoot,betacoeffshoot,betaexpshoot...'
print,'...tauoffsetshoot,taushoot,gammaexpshoot...'
print,'...sillumprev,signalprev,signalprevshoot'
print,betawk,tauwk,gammaexp,zerobias
print,betaexp,betacoeff,tauoffset
print,betashoot,betacoeffshoot,betaexpshoot
print,tauoffsetshoot,taushoot,gammaexpshoot
print,sillumprev,signalprev,signalprevshoot
endif ; imodelsim eq 3
work1=float(work1) ; otherwise time array etc written as integers!!!
work2=float(work2)
work=float(work)
work1(*)=es(ipix,*)
work2(*)=etime(*)-etime(0)+dtime ; time set such that it is zero at flux step
                               ; to the forst non-destructive integration
;
if(imodelsim eq 1)then foukes,betawk,tauwk,signalprev,sillumprev,$
work1,work2,work
if(imodelsim eq 2)then expfun,betawk,tauwk,signalprev,sillumprev,$
work1,work2,work
if(imodelsim eq 3)then $
expshoot,betawk,tauwk,signalprev,signalprevshoot,$
sillumprev,work1,work2,work,workdum1,workdum2
es(ipix,*)=work(*)
;
endif ; idodriftshort eq 1
;----------------- WHITE NOISE -------------------------------------------
idownoise=1
if(isim32 eq 1)then idownoise=0
; noise units V/s on 
if(array eq 'c100')then $
wnoise=[0.2,0.2,0.2,0.2,0.2,0.2,0.2,0.2,0.2]
if(array eq 'c200')then wnoise=[0.1,0.1,0.1,0.1]
wnoise=0.1*wnoise
wnoise=0.05*wnoise
;if(array eq 'c200')then wnoise=[0.001,0.001,0.001,0.001]
;if(array eq 'c200')then wnoise=[0.1,0.1,0.1,0.1]
nefd=wnoise(ipix)*sqrt(dtime) ; dtime is 
if(idownoise eq 1)then begin
print,'SIMUL32 adding white noise with rms,nefd (V/s) = ',wnoise(ipix),nefd
work(*)=wnoise(ipix)*randomn(seed,neptin)
es(ipix,*)=es(ipix,*)+work(*)
endif ; idownoise eq 1
wnoisesim=wnoise
;------------------------------------------------------------------------
;
endfor ; ipix
;
; write data back to S array
;
work=long(work)
work=where(ireduce eq 0,nii)
if(nii ne nptin)then begin
print,'SIMUL32: error - number of points transmitted not number of ',$
'input points'
goto,c999
endif
work1=float(work1)
for ipix=0,npix-1 do begin
work1(*)=es(ipix,*)
s(ipix,*)=work1(work)
endfor ; ipix
;
c999:
;
return
end



